@@ -75,103 +75,103 @@ static const GimpActionEntry image_actions[] =
   { "image-new", GIMP_ICON_DOCUMENT_NEW,
     NC_("image-action", "_New..."), "<primary>N",
     NC_("image-action", "Create a new image"),
-    G_CALLBACK (image_new_cmd_callback),
+    image_new_cmd_callback,
     GIMP_HELP_FILE_NEW },
 
   { "image-duplicate", GIMP_ICON_OBJECT_DUPLICATE,
     NC_("image-action", "_Duplicate"), "<primary>D",
     NC_("image-action", "Create a duplicate of this image"),
-    G_CALLBACK (image_duplicate_cmd_callback),
+    image_duplicate_cmd_callback,
     GIMP_HELP_IMAGE_DUPLICATE },
 
   { "image-color-profile-assign", NULL,
     NC_("image-action", "_Assign Color Profile..."), NULL,
     NC_("image-action", "Set a color profile on the image"),
-    G_CALLBACK (image_color_profile_assign_cmd_callback),
+    image_color_profile_assign_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_ASSIGN },
 
   { "image-color-profile-convert", NULL,
     NC_("image-action", "_Convert to Color Profile..."), NULL,
     NC_("image-action", "Apply a color profile to the image"),
-    G_CALLBACK (image_color_profile_convert_cmd_callback),
+    image_color_profile_convert_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_CONVERT },
 
   { "image-color-profile-discard", NULL,
     NC_("image-action", "_Discard Color Profile"), NULL,
     NC_("image-action", "Remove the image's color profile"),
-    G_CALLBACK (image_color_profile_discard_cmd_callback),
+    image_color_profile_discard_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_DISCARD },
 
   { "image-color-profile-save", NULL,
     NC_("image-action", "_Save Color Profile to File..."), NULL,
     NC_("image-action", "Save the image's color profile to an ICC file"),
-    G_CALLBACK (image_color_profile_save_cmd_callback),
+    image_color_profile_save_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_SAVE },
 
   { "image-resize", GIMP_ICON_OBJECT_RESIZE,
     NC_("image-action", "Can_vas Size..."), NULL,
     NC_("image-action", "Adjust the image dimensions"),
-    G_CALLBACK (image_resize_cmd_callback),
+    image_resize_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE },
 
   { "image-resize-to-layers", NULL,
     NC_("image-action", "Fit Canvas to L_ayers"), NULL,
     NC_("image-action", "Resize the image to enclose all layers"),
-    G_CALLBACK (image_resize_to_layers_cmd_callback),
+    image_resize_to_layers_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE_TO_LAYERS },
 
   { "image-resize-to-selection", NULL,
     NC_("image-action", "F_it Canvas to Selection"), NULL,
     NC_("image-action", "Resize the image to the extents of the selection"),
-    G_CALLBACK (image_resize_to_selection_cmd_callback),
+    image_resize_to_selection_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE_TO_SELECTION },
 
   { "image-print-size", GIMP_ICON_DOCUMENT_PRINT_RESOLUTION,
     NC_("image-action", "_Print Size..."), NULL,
     NC_("image-action", "Adjust the print resolution"),
-    G_CALLBACK (image_print_size_cmd_callback),
+    image_print_size_cmd_callback,
     GIMP_HELP_IMAGE_PRINT_SIZE },
 
   { "image-scale", GIMP_ICON_OBJECT_SCALE,
     NC_("image-action", "_Scale Image..."), NULL,
     NC_("image-action", "Change the size of the image content"),
-    G_CALLBACK (image_scale_cmd_callback),
+    image_scale_cmd_callback,
     GIMP_HELP_IMAGE_SCALE },
 
   { "image-crop-to-selection", GIMP_ICON_TOOL_CROP,
     NC_("image-action", "_Crop to Selection"), NULL,
     NC_("image-action", "Crop the image to the extents of the selection"),
-    G_CALLBACK (image_crop_to_selection_cmd_callback),
+    image_crop_to_selection_cmd_callback,
     GIMP_HELP_IMAGE_CROP },
 
   { "image-crop-to-content", GIMP_ICON_TOOL_CROP,
     NC_("image-action", "Crop to C_ontent"), NULL,
     NC_("image-action", "Crop the image to the extents of its content (remove empty borders from the image)"),
-    G_CALLBACK (image_crop_to_content_cmd_callback),
+    image_crop_to_content_cmd_callback,
     GIMP_HELP_IMAGE_CROP },
 
   { "image-merge-layers", NULL,
     NC_("image-action", "Merge Visible _Layers..."), "<primary>M",
     NC_("image-action", "Merge all visible layers into one layer"),
-    G_CALLBACK (image_merge_layers_cmd_callback),
+    image_merge_layers_cmd_callback,
     GIMP_HELP_IMAGE_MERGE_LAYERS },
 
   { "image-flatten", NULL,
     NC_("image-action", "_Flatten Image"), NULL,
     NC_("image-action", "Merge all layers into one and remove transparency"),
-    G_CALLBACK (image_flatten_image_cmd_callback),
+    image_flatten_image_cmd_callback,
     GIMP_HELP_IMAGE_FLATTEN },
 
   { "image-configure-grid", GIMP_ICON_GRID,
     NC_("image-action", "Configure G_rid..."), NULL,
     NC_("image-action", "Configure the grid for this image"),
-    G_CALLBACK (image_configure_grid_cmd_callback),
+    image_configure_grid_cmd_callback,
     GIMP_HELP_IMAGE_GRID },
 
   { "image-properties", "dialog-information",
     NC_("image-action", "Image Pr_operties"), "<alt>Return",
     NC_("image-action", "Display information about this image"),
-    G_CALLBACK (image_properties_cmd_callback),
+    image_properties_cmd_callback,
     GIMP_HELP_IMAGE_PROPERTIES }
 };
 
@@ -182,7 +182,7 @@ static const GimpToggleActionEntry image_toggle_actions[] =
     NC_("image-action", "Whether the image is color managed. Disabling "
         "color management is equivalent to assigning a built-in sRGB "
         "color profile. Better leave color management enabled."),
-    G_CALLBACK (image_color_management_enabled_cmd_callback),
+    image_color_management_enabled_cmd_callback,
     TRUE,
     GIMP_HELP_IMAGE_COLOR_MANAGEMENT_ENABLED }
 };
@@ -311,29 +311,29 @@ image_actions_setup (GimpActionGroup *group)
                                        image_convert_base_type_actions,
                                        G_N_ELEMENTS (image_convert_base_type_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_base_type_cmd_callback));
+                                       image_convert_base_type_cmd_callback);
 
   gimp_action_group_add_radio_actions (group, "image-convert-action",
                                        image_convert_precision_actions,
                                        G_N_ELEMENTS (image_convert_precision_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_precision_cmd_callback));
+                                       image_convert_precision_cmd_callback);
 
   gimp_action_group_add_radio_actions (group, "image-convert-action",
                                        image_convert_gamma_actions,
                                        G_N_ELEMENTS (image_convert_gamma_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_gamma_cmd_callback));
+                                       image_convert_gamma_cmd_callback);
 
   gimp_action_group_add_enum_actions (group, "image-action",
                                       image_flip_actions,
                                       G_N_ELEMENTS (image_flip_actions),
-                                      G_CALLBACK (image_flip_cmd_callback));
+                                      image_flip_cmd_callback);
 
   gimp_action_group_add_enum_actions (group, "image-action",
                                       image_rotate_actions,
                                       G_N_ELEMENTS (image_rotate_actions),
-                                      G_CALLBACK (image_rotate_cmd_callback));
+                                      image_rotate_cmd_callback);
 
 #define SET_ALWAYS_SHOW_IMAGE(action,show) \
         gimp_action_group_set_action_always_show_image (group, action, show)
