@@ -92,8 +92,9 @@ static void   view_padding_color_dialog_update (GimpColorDialog          *dialog
 /*  public functions  */
 
 void
-view_new_cmd_callback (GtkAction *action,
-                       gpointer   data)
+view_new_cmd_callback (GimpAction *action,
+                       GVariant   *value,
+                       gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -109,8 +110,9 @@ view_new_cmd_callback (GtkAction *action,
 }
 
 void
-view_close_cmd_callback (GtkAction *action,
-                         gpointer   data)
+view_close_cmd_callback (GimpAction *action,
+                         GVariant   *value,
+                         gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -126,8 +128,9 @@ view_close_cmd_callback (GtkAction *action,
 }
 
 void
-view_scroll_center_cmd_callback (GtkAction *action,
-                                 gpointer   data)
+view_scroll_center_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
   GimpDisplay *display;
   return_if_no_display (display, data);
@@ -137,8 +140,9 @@ view_scroll_center_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_fit_in_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_zoom_fit_in_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
 {
   GimpDisplay *display;
   return_if_no_display (display, data);
@@ -147,8 +151,9 @@ view_zoom_fit_in_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_fill_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_zoom_fill_cmd_callback (GimpAction *action,
+                             GVariant   *value,
+                             gpointer    data)
 {
   GimpDisplay *display;
   return_if_no_display (display, data);
@@ -157,8 +162,9 @@ view_zoom_fill_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_selection_cmd_callback (GtkAction *action,
-                                  gpointer   data)
+view_zoom_selection_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
 {
   GimpDisplay *display;
   GimpImage   *image;
@@ -176,8 +182,9 @@ view_zoom_selection_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_revert_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_zoom_revert_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
 {
   GimpDisplay *display;
   return_if_no_display (display, data);
@@ -186,14 +193,17 @@ view_zoom_revert_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_cmd_callback (GtkAction *action,
-                        gint       value,
-                        gpointer   data)
+view_zoom_cmd_callback (GimpAction *action,
+                        GVariant   *value,
+                        gpointer    data)
 {
-  GimpDisplayShell *shell;
+  GimpDisplayShell     *shell;
+  GimpActionSelectType  select_type;
   return_if_no_shell (shell, data);
 
-  switch ((GimpActionSelectType) value)
+  select_type = (GimpActionSelectType) g_variant_get_int32 (value);
+
+  switch (select_type)
     {
     case GIMP_ACTION_SELECT_FIRST:
       gimp_display_shell_scale (shell,
@@ -241,7 +251,7 @@ view_zoom_cmd_callback (GtkAction *action,
       {
         gdouble scale = gimp_zoom_model_get_factor (shell->zoom);
 
-        scale = action_select_value ((GimpActionSelectType) value,
+        scale = action_select_value (select_type,
                                      scale,
                                      0.0, 512.0, 1.0,
                                      1.0 / 8.0, 1.0, 16.0, 0.0,
@@ -261,37 +271,39 @@ view_zoom_cmd_callback (GtkAction *action,
 }
 
 void
-view_zoom_explicit_cmd_callback (GtkAction *action,
-                                 GtkAction *current,
-                                 gpointer   data)
+view_zoom_explicit_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
   GimpDisplayShell *shell;
-  gint              value;
+  gint              factor;
   return_if_no_shell (shell, data);
 
-  value = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
+  factor = g_variant_get_int32 (value);
 
-  if (value != 0 /* not Other... */)
+  if (factor != 0 /* not Other... */)
     {
-      if (fabs (value - gimp_zoom_model_get_factor (shell->zoom)) > 0.0001)
+      if (fabs (factor - gimp_zoom_model_get_factor (shell->zoom)) > 0.0001)
         gimp_display_shell_scale (shell,
                                   GIMP_ZOOM_TO,
-                                  (gdouble) value / 10000,
+                                  (gdouble) factor / 10000,
                                   GIMP_ZOOM_FOCUS_RETAIN_CENTERING_ELSE_BEST_GUESS);
     }
 }
 
+/* not a GimpActionCallback */
 void
-view_zoom_other_cmd_callback (GtkAction *action,
-                              gpointer   data)
+view_zoom_other_cmd_callback (GimpAction *action,
+                              gpointer    data)
 {
   GimpDisplayShell *shell;
   return_if_no_shell (shell, data);
 
   /* check if we are activated by the user or from
-   * view_actions_set_zoom()
+   * view_actions_set_zoom(), also this is really a GtkToggleAction
+   * NOT a GimpToggleAction
    */
-  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) &&
+  if (gtk_toggle_action_get_active ((GtkToggleAction *) action) &&
       shell->other_scale != gimp_zoom_model_get_factor (shell->zoom))
     {
       gimp_display_shell_scale_dialog (shell);
@@ -299,8 +311,9 @@ view_zoom_other_cmd_callback (GtkAction *action,
 }
 
 void
-view_dot_for_dot_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_show_all_cmd_callback (GimpAction *action,
+                            GVariant   *value,
+                            gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -309,7 +322,37 @@ view_dot_for_dot_cmd_callback (GtkAction *action,
 
   shell = gimp_display_get_shell (display);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
+
+  if (active != shell->show_all)
+    {
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+      gimp_display_shell_set_show_all (shell, active);
+
+      if (window)
+        SET_ACTIVE (gimp_image_window_get_ui_manager (window),
+                    "view-show-all", shell->show_all);
+
+      if (IS_ACTIVE_DISPLAY (display))
+        SET_ACTIVE (shell->popup_manager, "view-show-all",
+                    shell->show_all);
+    }
+}
+
+void
+view_dot_for_dot_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
+{
+  GimpDisplay      *display;
+  GimpDisplayShell *shell;
+  gboolean          active;
+  return_if_no_display (display, data);
+
+  shell = gimp_display_get_shell (display);
+
+  active = g_variant_get_boolean (value);
 
   if (active != shell->dot_for_dot)
     {
@@ -328,8 +371,9 @@ view_dot_for_dot_cmd_callback (GtkAction *action,
 }
 
 void
-view_flip_horizontally_cmd_callback (GtkAction *action,
-                                     gpointer   data)
+view_flip_horizontally_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -338,7 +382,7 @@ view_flip_horizontally_cmd_callback (GtkAction *action,
 
   shell = gimp_display_get_shell (display);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != shell->flip_horizontally)
     {
@@ -347,8 +391,9 @@ view_flip_horizontally_cmd_callback (GtkAction *action,
 }
 
 void
-view_flip_vertically_cmd_callback (GtkAction *action,
-                                   gpointer   data)
+view_flip_vertically_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -357,7 +402,7 @@ view_flip_vertically_cmd_callback (GtkAction *action,
 
   shell = gimp_display_get_shell (display);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != shell->flip_vertically)
     {
@@ -366,18 +411,21 @@ view_flip_vertically_cmd_callback (GtkAction *action,
 }
 
 void
-view_rotate_absolute_cmd_callback (GtkAction *action,
-                                   gint       value,
-                                   gpointer   data)
+view_rotate_absolute_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
-  GimpDisplay      *display;
-  GimpDisplayShell *shell;
-  gdouble           angle = 0.0;
+  GimpDisplay          *display;
+  GimpDisplayShell     *shell;
+  GimpActionSelectType  select_type;
+  gdouble               angle = 0.0;
   return_if_no_display (display, data);
 
+  select_type = (GimpActionSelectType) g_variant_get_int32 (value);
+
   shell = gimp_display_get_shell (display);
 
-  angle = action_select_value ((GimpActionSelectType) value,
+  angle = action_select_value (select_type,
                                0.0,
                                -180.0, 180.0, 0.0,
                                1.0, 15.0, 90.0, 0.0,
@@ -385,23 +433,26 @@ view_rotate_absolute_cmd_callback (GtkAction *action,
 
   gimp_display_shell_rotate_to (shell, angle);
 
-  if (value == GIMP_ACTION_SELECT_SET_TO_DEFAULT)
+  if (select_type == GIMP_ACTION_SELECT_SET_TO_DEFAULT)
     gimp_display_shell_flip (shell, FALSE, FALSE);
 }
 
 void
-view_rotate_relative_cmd_callback (GtkAction *action,
-                                   gint       value,
-                                   gpointer   data)
+view_rotate_relative_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
-  GimpDisplay      *display;
-  GimpDisplayShell *shell;
-  gdouble           delta = 0.0;
+  GimpDisplay          *display;
+  GimpDisplayShell     *shell;
+  GimpActionSelectType  select_type;
+  gdouble               delta = 0.0;
   return_if_no_display (display, data);
 
+  select_type = (GimpActionSelectType) g_variant_get_int32 (value);
+
   shell = gimp_display_get_shell (display);
 
-  delta = action_select_value ((GimpActionSelectType) value,
+  delta = action_select_value (select_type,
                                0.0,
                                -180.0, 180.0, 0.0,
                                1.0, 15.0, 90.0, 0.0,
@@ -411,8 +462,9 @@ view_rotate_relative_cmd_callback (GtkAction *action,
 }
 
 void
-view_rotate_other_cmd_callback (GtkAction *action,
-                                gpointer   data)
+view_rotate_other_cmd_callback (GimpAction *action,
+                                GVariant   *value,
+                                gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -424,23 +476,29 @@ view_rotate_other_cmd_callback (GtkAction *action,
 }
 
 void
-view_scroll_horizontal_cmd_callback (GtkAction *action,
-                                     gint       value,
-                                     gpointer   data)
+view_scroll_horizontal_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
-  GimpDisplayShell *shell;
-  gdouble           offset;
+  GimpDisplayShell     *shell;
+  GtkAdjustment        *adj;
+  GimpActionSelectType  select_type;
+  gdouble               offset;
   return_if_no_shell (shell, data);
 
-  offset = action_select_value ((GimpActionSelectType) value,
-                                gtk_adjustment_get_value (shell->hsbdata),
-                                gtk_adjustment_get_lower (shell->hsbdata),
-                                gtk_adjustment_get_upper (shell->hsbdata) -
-                                gtk_adjustment_get_page_size (shell->hsbdata),
-                                gtk_adjustment_get_lower (shell->hsbdata),
+  select_type = (GimpActionSelectType) g_variant_get_int32 (value);
+
+  adj = shell->hsbdata;
+
+  offset = action_select_value (select_type,
+                                gtk_adjustment_get_value (adj),
+                                gtk_adjustment_get_lower (adj),
+                                gtk_adjustment_get_upper (adj) -
+                                gtk_adjustment_get_page_size (adj),
+                                gtk_adjustment_get_lower (adj),
                                 1,
-                                gtk_adjustment_get_step_increment (shell->hsbdata),
-                                gtk_adjustment_get_page_increment (shell->hsbdata),
+                                gtk_adjustment_get_step_increment (adj),
+                                gtk_adjustment_get_page_increment (adj),
                                 0,
                                 FALSE);
 
@@ -448,23 +506,29 @@ view_scroll_horizontal_cmd_callback (GtkAction *action,
 }
 
 void
-view_scroll_vertical_cmd_callback (GtkAction *action,
-                                   gint       value,
-                                   gpointer   data)
+view_scroll_vertical_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
-  GimpDisplayShell *shell;
-  gdouble           offset;
+  GimpDisplayShell     *shell;
+  GtkAdjustment        *adj;
+  GimpActionSelectType  select_type;
+  gdouble               offset;
   return_if_no_shell (shell, data);
 
-  offset = action_select_value ((GimpActionSelectType) value,
-                                gtk_adjustment_get_value (shell->vsbdata),
-                                gtk_adjustment_get_lower (shell->vsbdata),
-                                gtk_adjustment_get_upper (shell->vsbdata) -
-                                gtk_adjustment_get_page_size (shell->vsbdata),
-                                gtk_adjustment_get_lower (shell->vsbdata),
+  select_type = (GimpActionSelectType) g_variant_get_int32 (value);
+
+  adj = shell->vsbdata;
+
+  offset = action_select_value (select_type,
+                                gtk_adjustment_get_value (adj),
+                                gtk_adjustment_get_lower (adj),
+                                gtk_adjustment_get_upper (adj) -
+                                gtk_adjustment_get_page_size (adj),
+                                gtk_adjustment_get_lower (adj),
                                 1,
-                                gtk_adjustment_get_step_increment (shell->vsbdata),
-                                gtk_adjustment_get_page_increment (shell->vsbdata),
+                                gtk_adjustment_get_step_increment (adj),
+                                gtk_adjustment_get_page_increment (adj),
                                 0,
                                 FALSE);
 
@@ -472,8 +536,9 @@ view_scroll_vertical_cmd_callback (GtkAction *action,
 }
 
 void
-view_navigation_window_cmd_callback (GtkAction *action,
-                                     gpointer   data)
+view_navigation_window_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
   Gimp             *gimp;
   GimpDisplayShell *shell;
@@ -489,8 +554,9 @@ view_navigation_window_cmd_callback (GtkAction *action,
 }
 
 void
-view_display_filters_cmd_callback (GtkAction *action,
-                                   gpointer   data)
+view_display_filters_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
   GimpDisplayShell *shell;
   GtkWidget        *dialog;
@@ -511,8 +577,9 @@ view_display_filters_cmd_callback (GtkAction *action,
 }
 
 void
-view_color_management_reset_cmd_callback (GtkAction *action,
-                                          gpointer   data)
+view_color_management_reset_cmd_callback (GimpAction *action,
+                                          GVariant   *value,
+                                          gpointer    data)
 {
   GimpDisplayShell *shell;
   GimpColorConfig  *global_config;
@@ -529,8 +596,9 @@ view_color_management_reset_cmd_callback (GtkAction *action,
 }
 
 void
-view_color_management_enable_cmd_callback (GtkAction *action,
-                                           gpointer   data)
+view_color_management_enable_cmd_callback (GimpAction *action,
+                                           GVariant   *value,
+                                           gpointer    data)
 {
   GimpDisplayShell        *shell;
   GimpColorConfig         *color_config;
@@ -540,7 +608,7 @@ view_color_management_enable_cmd_callback (GtkAction *action,
 
   color_config = gimp_display_shell_get_color_config (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   mode = gimp_color_config_get_mode (color_config);
 
@@ -564,8 +632,9 @@ view_color_management_enable_cmd_callback (GtkAction *action,
 }
 
 void
-view_color_management_softproof_cmd_callback (GtkAction *action,
-                                              gpointer   data)
+view_color_management_softproof_cmd_callback (GimpAction *action,
+                                              GVariant   *value,
+                                              gpointer    data)
 {
   GimpDisplayShell        *shell;
   GimpColorConfig         *color_config;
@@ -575,7 +644,7 @@ view_color_management_softproof_cmd_callback (GtkAction *action,
 
   color_config = gimp_display_shell_get_color_config (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   mode = gimp_color_config_get_mode (color_config);
 
@@ -599,31 +668,32 @@ view_color_management_softproof_cmd_callback (GtkAction *action,
 }
 
 void
-view_display_intent_cmd_callback (GtkAction *action,
-                                  GtkAction *current,
-                                  gpointer   data)
+view_display_intent_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
 {
   GimpDisplayShell          *shell;
   GimpColorConfig           *color_config;
-  GimpColorRenderingIntent   value;
+  GimpColorRenderingIntent   intent;
   return_if_no_shell (shell, data);
 
+  intent = (GimpColorRenderingIntent) g_variant_get_int32 (value);
+
   color_config = gimp_display_shell_get_color_config (shell);
 
-  value = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
-
-  if (value != gimp_color_config_get_display_intent (color_config))
+  if (intent != gimp_color_config_get_display_intent (color_config))
     {
       g_object_set (color_config,
-                    "display-rendering-intent", value,
+                    "display-rendering-intent", intent,
                     NULL);
       shell->color_config_set = TRUE;
     }
 }
 
 void
-view_display_bpc_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_display_bpc_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
 {
   GimpDisplayShell *shell;
   GimpColorConfig  *color_config;
@@ -632,7 +702,7 @@ view_display_bpc_cmd_callback (GtkAction *action,
 
   color_config = gimp_display_shell_get_color_config (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_color_config_get_display_bpc (color_config))
     {
@@ -644,8 +714,9 @@ view_display_bpc_cmd_callback (GtkAction *action,
 }
 
 void
-view_softproof_profile_cmd_callback (GtkAction *action,
-                                     gpointer   data)
+view_softproof_profile_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
   GimpImage        *image;
   GimpDisplayShell *shell;
@@ -685,31 +756,32 @@ view_softproof_profile_cmd_callback (GtkAction *action,
 }
 
 void
-view_softproof_intent_cmd_callback (GtkAction *action,
-                                    GtkAction *current,
-                                    gpointer   data)
+view_softproof_intent_cmd_callback (GimpAction *action,
+                                    GVariant   *value,
+                                    gpointer    data)
 {
   GimpDisplayShell          *shell;
   GimpColorConfig           *color_config;
-  GimpColorRenderingIntent   value;
+  GimpColorRenderingIntent   intent;
   return_if_no_shell (shell, data);
 
+  intent = (GimpColorRenderingIntent) g_variant_get_int32 (value);
+
   color_config = gimp_display_shell_get_color_config (shell);
 
-  value = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
-
-  if (value != gimp_color_config_get_simulation_intent (color_config))
+  if (intent != gimp_color_config_get_simulation_intent (color_config))
     {
       g_object_set (color_config,
-                    "simulation-rendering-intent", value,
+                    "simulation-rendering-intent", intent,
                     NULL);
       shell->color_config_set = TRUE;
     }
 }
 
 void
-view_softproof_bpc_cmd_callback (GtkAction *action,
-                                 gpointer   data)
+view_softproof_bpc_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
   GimpDisplayShell *shell;
   GimpColorConfig  *color_config;
@@ -718,7 +790,7 @@ view_softproof_bpc_cmd_callback (GtkAction *action,
 
   color_config = gimp_display_shell_get_color_config (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_color_config_get_simulation_bpc (color_config))
     {
@@ -730,8 +802,9 @@ view_softproof_bpc_cmd_callback (GtkAction *action,
 }
 
 void
-view_softproof_gamut_check_cmd_callback (GtkAction *action,
-                                         gpointer   data)
+view_softproof_gamut_check_cmd_callback (GimpAction *action,
+                                         GVariant   *value,
+                                         gpointer    data)
 {
   GimpDisplayShell *shell;
   GimpColorConfig  *color_config;
@@ -740,7 +813,7 @@ view_softproof_gamut_check_cmd_callback (GtkAction *action,
 
   color_config = gimp_display_shell_get_color_config (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_color_config_get_simulation_gamut_check (color_config))
     {
@@ -752,14 +825,15 @@ view_softproof_gamut_check_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_selection_cmd_callback (GtkAction *action,
-                                    gpointer   data)
+view_toggle_selection_cmd_callback (GimpAction *action,
+                                    GVariant   *value,
+                                    gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_selection (shell))
     {
@@ -768,14 +842,15 @@ view_toggle_selection_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_layer_boundary_cmd_callback (GtkAction *action,
-                                         gpointer   data)
+view_toggle_layer_boundary_cmd_callback (GimpAction *action,
+                                         GVariant   *value,
+                                         gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_layer (shell))
     {
@@ -784,14 +859,32 @@ view_toggle_layer_boundary_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_menubar_cmd_callback (GtkAction *action,
-                                  gpointer   data)
+view_toggle_canvas_boundary_cmd_callback (GimpAction *action,
+                                          GVariant   *value,
+                                          gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
+
+  if (active != gimp_display_shell_get_show_canvas (shell))
+    {
+      gimp_display_shell_set_show_canvas (shell, active);
+    }
+}
+
+void
+view_toggle_menubar_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
+{
+  GimpDisplayShell *shell;
+  gboolean          active;
+  return_if_no_shell (shell, data);
+
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_menubar (shell))
     {
@@ -800,14 +893,15 @@ view_toggle_menubar_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_rulers_cmd_callback (GtkAction *action,
-                                 gpointer   data)
+view_toggle_rulers_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_rulers (shell))
     {
@@ -816,14 +910,15 @@ view_toggle_rulers_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_scrollbars_cmd_callback (GtkAction *action,
-                                     gpointer   data)
+view_toggle_scrollbars_cmd_callback (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_scrollbars (shell))
     {
@@ -832,14 +927,15 @@ view_toggle_scrollbars_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_statusbar_cmd_callback (GtkAction *action,
-                                    gpointer   data)
+view_toggle_statusbar_cmd_callback (GimpAction *action,
+                                    GVariant   *value,
+                                    gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_statusbar (shell))
     {
@@ -848,14 +944,15 @@ view_toggle_statusbar_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_guides_cmd_callback (GtkAction *action,
-                                 gpointer   data)
+view_toggle_guides_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_guides (shell))
     {
@@ -864,14 +961,15 @@ view_toggle_guides_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_grid_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_toggle_grid_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_grid (shell))
     {
@@ -880,14 +978,15 @@ view_toggle_grid_cmd_callback (GtkAction *action,
 }
 
 void
-view_toggle_sample_points_cmd_callback (GtkAction *action,
-                                        gpointer   data)
+view_toggle_sample_points_cmd_callback (GimpAction *action,
+                                        GVariant   *value,
+                                        gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_show_sample_points (shell))
     {
@@ -896,14 +995,15 @@ view_toggle_sample_points_cmd_callback (GtkAction *action,
 }
 
 void
-view_snap_to_guides_cmd_callback (GtkAction *action,
-                                  gpointer   data)
+view_snap_to_guides_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_snap_to_guides (shell))
     {
@@ -912,14 +1012,15 @@ view_snap_to_guides_cmd_callback (GtkAction *action,
 }
 
 void
-view_snap_to_grid_cmd_callback (GtkAction *action,
-                                gpointer   data)
+view_snap_to_grid_cmd_callback (GimpAction *action,
+                                GVariant   *value,
+                                gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_snap_to_grid (shell))
     {
@@ -928,14 +1029,15 @@ view_snap_to_grid_cmd_callback (GtkAction *action,
 }
 
 void
-view_snap_to_canvas_cmd_callback (GtkAction *action,
-                                  gpointer   data)
+view_snap_to_canvas_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_snap_to_canvas (shell))
     {
@@ -944,14 +1046,15 @@ view_snap_to_canvas_cmd_callback (GtkAction *action,
 }
 
 void
-view_snap_to_vectors_cmd_callback (GtkAction *action,
-                                   gpointer   data)
+view_snap_to_vectors_cmd_callback (GimpAction *action,
+                                   GVariant   *value,
+                                   gpointer    data)
 {
   GimpDisplayShell *shell;
   gboolean          active;
   return_if_no_shell (shell, data);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  active = g_variant_get_boolean (value);
 
   if (active != gimp_display_shell_get_snap_to_vectors (shell))
     {
@@ -960,17 +1063,20 @@ view_snap_to_vectors_cmd_callback (GtkAction *action,
 }
 
 void
-view_padding_color_cmd_callback (GtkAction *action,
-                                 gint       value,
-                                 gpointer   data)
+view_padding_color_cmd_callback (GimpAction *action,
+                                 GVariant   *value,
+                                 gpointer    data)
 {
-  GimpDisplay        *display;
-  GimpImageWindow    *window;
-  GimpDisplayShell   *shell;
-  GimpDisplayOptions *options;
-  gboolean            fullscreen;
+  GimpDisplay           *display;
+  GimpImageWindow       *window;
+  GimpDisplayShell      *shell;
+  GimpDisplayOptions    *options;
+  GimpCanvasPaddingMode  padding_mode;
+  gboolean               fullscreen;
   return_if_no_display (display, data);
 
+  padding_mode = (GimpCanvasPaddingMode) g_variant_get_int32 (value);
+
   shell  = gimp_display_get_shell (display);
   window = gimp_display_shell_get_window (shell);
 
@@ -986,7 +1092,7 @@ view_padding_color_cmd_callback (GtkAction *action,
 
 #define PADDING_COLOR_DIALOG_KEY "gimp-padding-color-dialog"
 
-  switch ((GimpCanvasPaddingMode) value)
+  switch (padding_mode)
     {
     case GIMP_CANVAS_PADDING_MODE_DEFAULT:
     case GIMP_CANVAS_PADDING_MODE_LIGHT_CHECK:
@@ -995,7 +1101,7 @@ view_padding_color_cmd_callback (GtkAction *action,
 
       options->padding_mode_set = TRUE;
 
-      gimp_display_shell_set_padding (shell, (GimpCanvasPaddingMode) value,
+      gimp_display_shell_set_padding (shell, padding_mode,
                                       &options->padding_color);
       break;
 
@@ -1049,14 +1155,34 @@ view_padding_color_cmd_callback (GtkAction *action,
         gimp_display_shell_set_padding (shell,
                                         default_options->padding_mode,
                                         &default_options->padding_color);
+        gimp_display_shell_set_padding_in_show_all (shell,
+                                                    default_options->padding_in_show_all);
       }
       break;
     }
 }
 
 void
-view_shrink_wrap_cmd_callback (GtkAction *action,
-                               gpointer   data)
+view_padding_color_in_show_all_cmd_callback (GimpAction *action,
+                                             GVariant   *value,
+                                             gpointer    data)
+{
+  GimpDisplayShell *shell;
+  gboolean          active;
+  return_if_no_shell (shell, data);
+
+  active = g_variant_get_boolean (value);
+
+  if (active != gimp_display_shell_get_padding_in_show_all (shell))
+    {
+      gimp_display_shell_set_padding_in_show_all (shell, active);
+    }
+}
+
+void
+view_shrink_wrap_cmd_callback (GimpAction *action,
+                               GVariant   *value,
+                               gpointer    data)
 {
   GimpDisplayShell *shell;
   return_if_no_shell (shell, data);
@@ -1065,8 +1191,9 @@ view_shrink_wrap_cmd_callback (GtkAction *action,
 }
 
 void
-view_fullscreen_cmd_callback (GtkAction *action,
-                              gpointer   data)
+view_fullscreen_cmd_callback (GimpAction *action,
+                              GVariant   *value,
+                              gpointer    data)
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
@@ -1078,9 +1205,7 @@ view_fullscreen_cmd_callback (GtkAction *action,
 
   if (window)
     {
-      gboolean active;
-
-      active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+      gboolean active = g_variant_get_boolean (value);
 
       gimp_image_window_set_fullscreen (window, active);
     }
