@@ -38,8 +38,9 @@
 /*  public functions  */
 
 void
-window_close_cmd_callback (GtkAction *action,
-                           gpointer   data)
+window_close_cmd_callback (GimpAction *action,
+                           GVariant   *value,
+                           gpointer    data)
 {
   GtkWidget *widget;
   return_if_no_widget (widget, data);
@@ -60,8 +61,9 @@ window_close_cmd_callback (GtkAction *action,
 }
 
 void
-window_open_display_cmd_callback (GtkAction *action,
-                                  gpointer   data)
+window_open_display_cmd_callback (GimpAction *action,
+                                  GVariant   *value,
+                                  gpointer    data)
 {
   GtkWidget *widget;
   GtkWidget *dialog;
@@ -132,14 +134,17 @@ window_open_display_cmd_callback (GtkAction *action,
 }
 
 void
-window_move_to_screen_cmd_callback (GtkAction *action,
-                                    GtkAction *current,
-                                    gpointer   data)
+window_move_to_screen_cmd_callback (GimpAction *action,
+                                    GVariant   *value,
+                                    gpointer    data)
 {
   GtkWidget *widget;
+#if 0
   GdkScreen *screen;
+#endif
   return_if_no_widget (widget, data);
 
+#if 0
   if (! gtk_widget_is_toplevel (widget))
     widget = gtk_widget_get_toplevel (widget);
 
@@ -149,4 +154,5 @@ window_move_to_screen_cmd_callback (GtkAction *action,
     {
       gtk_window_set_screen (GTK_WINDOW (widget), screen);
     }
+#endif
 }
