@@ -56,7 +56,6 @@
 #include "core/gimp.h"
 #include "core/gimp-batch.h"
 #include "core/gimp-user-install.h"
-#include "core/gimpbacktrace.h"
 #include "core/gimpimage.h"
 
 #include "file/file-open.h"
@@ -74,6 +73,7 @@
 #include "gimp-debug.h"
 
 #include "gimp-intl.h"
+#include "gimp-update.h"
 
 
 /*  local prototypes  */
@@ -192,6 +192,7 @@ app_run (const gchar         *full_prog_name,
   GimpLangRc         *temprc;
   gchar              *language   = NULL;
   GError             *font_error = NULL;
+  gboolean            save_gimprc_at_exit = FALSE;
 
   if (filenames && filenames[0] && ! filenames[1] &&
       g_file_test (filenames[0], G_FILE_TEST_IS_DIR))
@@ -213,12 +214,6 @@ app_run (const gchar         *full_prog_name,
       filenames = NULL;
     }
 
-  /* Initialize GimpBacktrace early on.  In particular, we want the
-   * Windows backend to catch the SET_THREAD_NAME exceptions of newly
-   * created threads.
-   */
-  gimp_backtrace_init ();
-
   /* Language needs to be determined first, before any GimpContext is
    * instantiated (which happens when the Gimp object is created)
    * because its properties need to be properly localized in the
@@ -288,6 +283,12 @@ app_run (const gchar         *full_prog_name,
 
   gimp_load_config (gimp, alternate_system_gimprc, alternate_gimprc);
 
+  /* We usually only save gimprc when Preferences are edited.
+   * Thus we have to add a special flag when we make an update
+   * check so that the timestamp is saved.
+   */
+  save_gimprc_at_exit = gimp_update_auto_check (gimp->config);
+
   /* Initialize the error handling after creating/migrating the config
    * directory because it will create some folders for backup and crash
    * logs in advance. Therefore running this before
@@ -444,6 +445,9 @@ app_run (const gchar         *full_prog_name,
   if (gimp->be_verbose)
     g_print ("EXIT: %s\n", G_STRFUNC);
 
+  if (save_gimprc_at_exit)
+    gimp_rc_save (GIMP_RC (gimp->config));
+
   g_main_loop_unref (loop);
 
   gimp_gegl_exit (gimp);
