@@ -127,6 +127,11 @@ enum
   PROP_EXPORT_METADATA_XMP,
   PROP_EXPORT_METADATA_IPTC,
   PROP_DEBUG_POLICY,
+  PROP_CHECK_UPDATES,
+  PROP_CHECK_UPDATE_TIMESTAMP,
+  PROP_LAST_RELEASE_TIMESTAMP,
+  PROP_LAST_REVISION,
+  PROP_LAST_KNOWN_RELEASE,
 
   /* ignored, only for backward compatibility: */
   PROP_INSTALL_COLORMAP,
@@ -675,6 +680,41 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                            GIMP_PARAM_STATIC_STRINGS |
                            GIMP_CONFIG_PARAM_AGGREGATE);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_CHECK_UPDATES,
+                            "check-updates",
+                            "Check for updates",
+                            CHECK_UPDATES_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_INT64 (object_class, PROP_CHECK_UPDATE_TIMESTAMP,
+                          "check-update-timestamp",
+                          "timestamp of the last update check",
+                          CHECK_UPDATE_TIMESTAMP_BLURB,
+                          0, G_MAXINT64, 0,
+                          GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_INT64 (object_class, PROP_LAST_RELEASE_TIMESTAMP,
+                          "last-release-timestamp",
+                          "timestamp of the last release",
+                          LAST_RELEASE_TIMESTAMP_BLURB,
+                          0, G_MAXINT64, 0,
+                          GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_LAST_KNOWN_RELEASE,
+                           "last-known-release",
+                           "last known release of GIMP",
+                           LAST_KNOWN_RELEASE_BLURB,
+                           NULL,
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_INT (object_class, PROP_LAST_REVISION,
+                        "last-revision",
+                        "Last revision of current release",
+                        LAST_RELEASE_TIMESTAMP_BLURB,
+                        0, G_MAXINT, 0,
+                        GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SAVE_DOCUMENT_HISTORY,
                             "save-document-history",
                             "Save document history",
@@ -841,6 +881,8 @@ gimp_core_config_finalize (GObject *object)
   g_free (core_config->plug_in_rc_path);
   g_free (core_config->import_raw_plug_in);
 
+  g_clear_pointer (&core_config->last_known_release, g_free);
+
   g_clear_object (&core_config->default_image);
   g_clear_object (&core_config->default_grid);
   g_clear_object (&core_config->color_management);
@@ -1044,6 +1086,21 @@ gimp_core_config_set_property (GObject      *object,
         gimp_config_sync (g_value_get_object (value),
                           G_OBJECT (core_config->color_management), 0);
       break;
+    case PROP_CHECK_UPDATES:
+      core_config->check_updates = g_value_get_boolean (value);
+      break;
+    case PROP_CHECK_UPDATE_TIMESTAMP:
+      core_config->check_update_timestamp = g_value_get_int64 (value);
+      break;
+    case PROP_LAST_RELEASE_TIMESTAMP:
+      core_config->last_release_timestamp = g_value_get_int64 (value);
+      break;
+    case PROP_LAST_REVISION:
+      core_config->last_revision = g_value_get_int (value);
+      break;
+    case PROP_LAST_KNOWN_RELEASE:
+      core_config->last_known_release = g_value_dup_string (value);
+      break;
     case PROP_SAVE_DOCUMENT_HISTORY:
       core_config->save_document_history = g_value_get_boolean (value);
       break;
@@ -1253,6 +1310,21 @@ gimp_core_config_get_property (GObject    *object,
     case PROP_COLOR_MANAGEMENT:
       g_value_set_object (value, core_config->color_management);
       break;
+    case PROP_CHECK_UPDATES:
+      g_value_set_boolean (value, core_config->check_updates);
+      break;
+    case PROP_CHECK_UPDATE_TIMESTAMP:
+      g_value_set_int64 (value, core_config->check_update_timestamp);
+      break;
+    case PROP_LAST_RELEASE_TIMESTAMP:
+      g_value_set_int64 (value, core_config->last_release_timestamp);
+      break;
+    case PROP_LAST_REVISION:
+      g_value_set_int (value, core_config->last_revision);
+      break;
+    case PROP_LAST_KNOWN_RELEASE:
+      g_value_set_string (value, core_config->last_known_release);
+      break;
     case PROP_SAVE_DOCUMENT_HISTORY:
       g_value_set_boolean (value, core_config->save_document_history);
       break;
