@@ -102,6 +102,12 @@ struct _GimpCoreConfig
   gboolean                export_metadata_xmp;
   gboolean                export_metadata_iptc;
   GimpDebugPolicy         debug_policy;
+
+  gboolean                check_updates;
+  gint64                  check_update_timestamp;
+  gchar                  *last_known_release;
+  gint64                  last_release_timestamp;
+  gint                    last_revision;
 };
 
 struct _GimpCoreConfigClass
