@@ -52,6 +52,7 @@ enum
 enum
 {
   PROP_0,
+  PROP_EDIT_NON_VISIBLE,
   PROP_MOVE_TOOL_CHANGES_ACTIVE,
   PROP_FILTER_TOOL_MAX_RECENT,
   PROP_FILTER_TOOL_USE_LAST_SETTINGS,
@@ -63,6 +64,7 @@ enum
   PROP_RESTORE_SESSION,
   PROP_RESTORE_MONITOR,
   PROP_SAVE_TOOL_OPTIONS,
+  PROP_COMPACT_SLIDERS,
   PROP_SHOW_TOOLTIPS,
   PROP_TEAROFF_MENUS,
   PROP_CAN_CHANGE_ACCELS,
@@ -74,6 +76,7 @@ enum
   PROP_TOOLBOX_FOO_AREA,
   PROP_TOOLBOX_IMAGE_AREA,
   PROP_TOOLBOX_WILBER,
+  PROP_TOOLBOX_GROUPS,
   PROP_THEME_PATH,
   PROP_THEME,
   PROP_ICON_THEME_PATH,
@@ -95,6 +98,7 @@ enum
 
   PROP_HIDE_DOCKS,
   PROP_SINGLE_WINDOW_MODE,
+  PROP_SHOW_TABS,
   PROP_TABS_POSITION,
   PROP_LAST_TIP_SHOWN,
 
@@ -150,6 +154,13 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
   object_class->set_property = gimp_gui_config_set_property;
   object_class->get_property = gimp_gui_config_get_property;
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_EDIT_NON_VISIBLE,
+                            "edit-non-visible",
+                            "Non-visible layers can be edited",
+                            EDIT_NON_VISIBLE_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_MOVE_TOOL_CHANGES_ACTIVE,
                             "move-tool-changes-active",
                             "Move tool changes active layer",
@@ -227,6 +238,13 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_COMPACT_SLIDERS,
+                            "compact-sliders",
+                            "Compact sliders",
+                            COMPACT_SLIDERS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_TOOLTIPS,
                             "show-tooltips",
                             "Show tooltips",
@@ -306,6 +324,13 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TOOLBOX_GROUPS,
+                            "toolbox-groups",
+                            "Use toolbox groups",
+                            TOOLBOX_GROUPS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   path = gimp_config_build_data_path ("themes");
   GIMP_CONFIG_PROP_PATH (object_class, PROP_THEME_PATH,
                          "theme-path",
@@ -456,6 +481,15 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                                                          G_PARAM_CONSTRUCT |
                                                          GIMP_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_SHOW_TABS,
+                                   g_param_spec_boolean ("show-tabs",
+                                                         NULL,
+                                                         SHOW_TABS_BLURB,
+                                                         TRUE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT |
+                                                         GIMP_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (object_class, PROP_TABS_POSITION,
                                    g_param_spec_enum ("tabs-position", NULL, NULL,
                                                       GIMP_TYPE_POSITION,
@@ -570,6 +604,9 @@ gimp_gui_config_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_EDIT_NON_VISIBLE:
+      gui_config->edit_non_visible = g_value_get_boolean (value);
+      break;
     case PROP_MOVE_TOOL_CHANGES_ACTIVE:
       gui_config->move_tool_changes_active = g_value_get_boolean (value);
       break;
@@ -604,6 +641,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_SAVE_TOOL_OPTIONS:
       gui_config->save_tool_options = g_value_get_boolean (value);
       break;
+    case PROP_COMPACT_SLIDERS:
+      gui_config->compact_sliders = g_value_get_boolean (value);
+      break;
     case PROP_SHOW_TOOLTIPS:
       gui_config->show_tooltips = g_value_get_boolean (value);
       break;
@@ -637,6 +677,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_TOOLBOX_WILBER:
       gui_config->toolbox_wilber = g_value_get_boolean (value);
       break;
+    case PROP_TOOLBOX_GROUPS:
+      gui_config->toolbox_groups = g_value_get_boolean (value);
+      break;
      case PROP_THEME_PATH:
       g_free (gui_config->theme_path);
       gui_config->theme_path = g_value_dup_string (value);
@@ -721,6 +764,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_SINGLE_WINDOW_MODE:
       gui_config->single_window_mode = g_value_get_boolean (value);
       break;
+    case PROP_SHOW_TABS:
+      gui_config->show_tabs = g_value_get_boolean (value);
+      break;
     case PROP_TABS_POSITION:
       gui_config->tabs_position = g_value_get_enum (value);
       break;
@@ -755,6 +801,9 @@ gimp_gui_config_get_property (GObject    *object,
 
   switch (property_id)
     {
+    case PROP_EDIT_NON_VISIBLE:
+      g_value_set_boolean (value, gui_config->edit_non_visible);
+      break;
     case PROP_MOVE_TOOL_CHANGES_ACTIVE:
       g_value_set_boolean (value, gui_config->move_tool_changes_active);
       break;
@@ -789,6 +838,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_SAVE_TOOL_OPTIONS:
       g_value_set_boolean (value, gui_config->save_tool_options);
       break;
+    case PROP_COMPACT_SLIDERS:
+      g_value_set_boolean (value, gui_config->compact_sliders);
+      break;
     case PROP_SHOW_TOOLTIPS:
       g_value_set_boolean (value, gui_config->show_tooltips);
       break;
@@ -822,6 +874,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_TOOLBOX_WILBER:
       g_value_set_boolean (value, gui_config->toolbox_wilber);
       break;
+    case PROP_TOOLBOX_GROUPS:
+      g_value_set_boolean (value, gui_config->toolbox_groups);
+      break;
     case PROP_THEME_PATH:
       g_value_set_string (value, gui_config->theme_path);
       break;
@@ -881,6 +936,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_SINGLE_WINDOW_MODE:
       g_value_set_boolean (value, gui_config->single_window_mode);
       break;
+    case PROP_SHOW_TABS:
+      g_value_set_boolean (value, gui_config->show_tabs);
+      break;
     case PROP_TABS_POSITION:
       g_value_set_enum (value, gui_config->tabs_position);
       break;
