@@ -101,6 +101,7 @@ typedef struct _GimpFilterStack                 GimpFilterStack;
 typedef struct _GimpItemStack                   GimpItemStack;
 typedef struct _GimpLayerStack                  GimpLayerStack;
 typedef struct _GimpTaggedContainer             GimpTaggedContainer;
+typedef struct _GimpTreeProxy                   GimpTreeProxy;
 
 
 /*  not really a container  */
@@ -119,7 +120,9 @@ typedef struct _GimpToolOptions                 GimpToolOptions;
 /*  info objects  */
 
 typedef struct _GimpPaintInfo                   GimpPaintInfo;
+typedef struct _GimpToolGroup                   GimpToolGroup;
 typedef struct _GimpToolInfo                    GimpToolInfo;
+typedef struct _GimpToolItem                    GimpToolItem;
 
 
 /*  data objects  */
@@ -186,7 +189,7 @@ typedef struct _GimpEnvironTable                GimpEnvironTable;
 typedef struct _GimpHistogram                   GimpHistogram;
 typedef struct _GimpIdTable                     GimpIdTable;
 typedef struct _GimpImagefile                   GimpImagefile;
-typedef struct _GimpImageViewable               GimpImageViewable;
+typedef struct _GimpImageProxy                  GimpImageProxy;
 typedef struct _GimpInterpreterDB               GimpInterpreterDB;
 typedef struct _GimpLineArt                     GimpLineArt;
 typedef struct _GimpObjectQueue                 GimpObjectQueue;
