@@ -70,6 +70,7 @@
 #include "gimppattern.h"
 #include "gimptemplate.h"
 #include "gimptoolinfo.h"
+#include "gimptreeproxy.h"
 
 #include "gimp-intl.h"
 
@@ -262,6 +263,18 @@ gimp_init (Gimp *gimp)
   gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_info_list),
                                "tool infos");
 
+  gimp->tool_item_list = g_object_new (GIMP_TYPE_LIST,
+                                       "children-type", GIMP_TYPE_TOOL_ITEM,
+                                       "append",        TRUE,
+                                       NULL);
+  gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_item_list),
+                               "tool items");
+
+  gimp->tool_item_ui_list = gimp_tree_proxy_new_for_container (
+    gimp->tool_item_list);
+  gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_item_ui_list),
+                               "ui tool items");
+
   gimp->documents = gimp_document_list_new (gimp);
 
   gimp->templates = gimp_list_new (GIMP_TYPE_TEMPLATE, TRUE);
@@ -378,6 +391,9 @@ gimp_finalize (GObject *object)
 
   gimp_tool_info_set_standard (gimp, NULL);
 
+  g_clear_object (&gimp->tool_item_list);
+  g_clear_object (&gimp->tool_item_ui_list);
+
   if (gimp->tool_info_list)
     {
       gimp_container_foreach (gimp->tool_info_list,
@@ -900,6 +916,22 @@ gimp_get_tool_info_iter (Gimp *gimp)
   return GIMP_LIST (gimp->tool_info_list)->queue->head;
 }
 
+GList *
+gimp_get_tool_item_iter (Gimp *gimp)
+{
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
+
+  return GIMP_LIST (gimp->tool_item_list)->queue->head;
+}
+
+GList *
+gimp_get_tool_item_ui_iter (Gimp *gimp)
+{
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
+
+  return GIMP_LIST (gimp->tool_item_ui_list)->queue->head;
+}
+
 GimpObject *
 gimp_get_clipboard_object (Gimp *gimp)
 {
