@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_H__
@@ -110,6 +110,9 @@ struct _Gimp
   GimpContainer          *tool_info_list;
   GimpToolInfo           *standard_tool_info;
 
+  GimpContainer          *tool_item_list;
+  GimpContainer          *tool_item_ui_list;
+
   /*  the opened and saved images in MRU order  */
   GimpContainer          *documents;
 
@@ -186,6 +189,8 @@ GList        * gimp_get_display_iter       (Gimp                *gimp);
 GList        * gimp_get_image_windows      (Gimp                *gimp);
 GList        * gimp_get_paint_info_iter    (Gimp                *gimp);
 GList        * gimp_get_tool_info_iter     (Gimp                *gimp);
+GList        * gimp_get_tool_item_iter     (Gimp                *gimp);
+GList        * gimp_get_tool_item_ui_iter  (Gimp                *gimp);
 
 GimpObject   * gimp_get_clipboard_object   (Gimp                *gimp);
 
