@@ -14,17 +14,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_DRAWABLE_SHADOW_H__
 #define __GIMP_DRAWABLE_SHADOW_H__
 
 
-TileManager * gimp_drawable_get_shadow_tiles   (GimpDrawable *drawable);
-void          gimp_drawable_free_shadow_tiles  (GimpDrawable *drawable);
+GeglBuffer * gimp_drawable_get_shadow_buffer   (GimpDrawable *drawable);
+void         gimp_drawable_free_shadow_buffer  (GimpDrawable *drawable);
 
-void          gimp_drawable_merge_shadow_tiles (GimpDrawable *drawable,
+void         gimp_drawable_merge_shadow_buffer (GimpDrawable *drawable,
                                                 gboolean      push_undo,
                                                 const gchar  *undo_desc);
 
