@@ -44,6 +44,7 @@
 #include "gimpdrawable-filters.h"
 #include "gimpdrawablefilter.h"
 #include "gimpimage.h"
+#include "gimplayer.h"
 #include "gimpmarshal.h"
 #include "gimpprogress.h"
 
@@ -64,7 +65,10 @@ struct _GimpDrawableFilter
 
   gboolean                has_input;
 
+  gboolean                clip;
   GimpFilterRegion        region;
+  gboolean                crop_enabled;
+  GeglRectangle           crop_rect;
   gboolean                preview_enabled;
   GimpAlignmentType       preview_alignment;
   gdouble                 preview_position;
@@ -73,10 +77,14 @@ struct _GimpDrawableFilter
   GimpLayerColorSpace     blend_space;
   GimpLayerColorSpace     composite_space;
   GimpLayerCompositeMode  composite_mode;
+  gboolean                add_alpha;
   gboolean                color_managed;
   gboolean                gamma_hack;
 
+  gboolean                override_constraints;
+
   GeglRectangle           filter_area;
+  gboolean                filter_clip;
 
   GeglNode               *translate;
   GeglNode               *crop_before;
@@ -89,37 +97,49 @@ struct _GimpDrawableFilter
 };
 
 
-static void       gimp_drawable_filter_dispose          (GObject             *object);
-static void       gimp_drawable_filter_finalize         (GObject             *object);
+static void       gimp_drawable_filter_dispose               (GObject             *object);
+static void       gimp_drawable_filter_finalize              (GObject             *object);
 
-static void       gimp_drawable_filter_sync_region      (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_preview     (GimpDrawableFilter  *filter,
-                                                         gboolean             old_enabled,
-                                                         GimpAlignmentType    old_alignment,
-                                                         gdouble              old_position);
-static void       gimp_drawable_filter_sync_opacity     (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_mode        (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_affect      (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_mask        (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_transform   (GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_sync_gamma_hack  (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_clip             (GimpDrawableFilter  *filter,
+                                                              gboolean             sync_region);
+static void       gimp_drawable_filter_sync_region           (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_crop             (GimpDrawableFilter  *filter,
+                                                              gboolean             old_crop_enabled,
+                                                              const GeglRectangle *old_crop_rect,
+                                                              gboolean             old_preview_enabled,
+                                                              GimpAlignmentType    old_preview_alignment,
+                                                              gdouble              old_preview_position,
+                                                              gboolean             update);
+static void       gimp_drawable_filter_sync_opacity          (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_mode             (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_affect           (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_format           (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_mask             (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_transform        (GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_sync_gamma_hack       (GimpDrawableFilter  *filter);
 
-static gboolean   gimp_drawable_filter_is_filtering     (GimpDrawableFilter  *filter);
-static gboolean   gimp_drawable_filter_add_filter       (GimpDrawableFilter  *filter);
-static gboolean   gimp_drawable_filter_remove_filter    (GimpDrawableFilter  *filter);
+static gboolean   gimp_drawable_filter_is_filtering          (GimpDrawableFilter  *filter);
+static gboolean   gimp_drawable_filter_add_filter            (GimpDrawableFilter  *filter);
+static gboolean   gimp_drawable_filter_remove_filter         (GimpDrawableFilter  *filter);
 
-static void       gimp_drawable_filter_update_drawable  (GimpDrawableFilter  *filter,
-                                                         const GeglRectangle *area);
+static void       gimp_drawable_filter_update_drawable       (GimpDrawableFilter  *filter,
+                                                              const GeglRectangle *area);
 
-static void       gimp_drawable_filter_affect_changed   (GimpImage           *image,
-                                                         GimpChannelType      channel,
-                                                         GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_mask_changed     (GimpImage           *image,
-                                                         GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_profile_changed  (GimpColorManaged    *managed,
-                                                         GimpDrawableFilter  *filter);
-static void       gimp_drawable_filter_drawable_removed (GimpDrawable        *drawable,
-                                                         GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_affect_changed        (GimpImage           *image,
+                                                              GimpChannelType      channel,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_mask_changed          (GimpImage           *image,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_profile_changed       (GimpColorManaged    *managed,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_lock_position_changed (GimpDrawable        *drawable,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_format_changed        (GimpDrawable        *drawable,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_drawable_removed      (GimpDrawable        *drawable,
+                                                              GimpDrawableFilter  *filter);
+static void       gimp_drawable_filter_lock_alpha_changed    (GimpLayer           *layer,
+                                                              GimpDrawableFilter  *filter);
 
 
 G_DEFINE_TYPE (GimpDrawableFilter, gimp_drawable_filter, GIMP_TYPE_FILTER)
@@ -150,6 +170,7 @@ gimp_drawable_filter_class_init (GimpDrawableFilterClass *klass)
 static void
 gimp_drawable_filter_init (GimpDrawableFilter *drawable_filter)
 {
+  drawable_filter->clip              = TRUE;
   drawable_filter->region            = GIMP_FILTER_REGION_SELECTION;
   drawable_filter->preview_alignment = GIMP_ALIGN_LEFT;
   drawable_filter->preview_position  = 1.0;
@@ -208,11 +229,14 @@ gimp_drawable_filter_new (GimpDrawable *drawable,
   node = gimp_filter_get_node (GIMP_FILTER (filter));
 
   gegl_node_add_child (node, operation);
+  gimp_gegl_node_set_underlying_operation (node, operation);
 
-  filter->applicator = gimp_applicator_new (node, TRUE, TRUE);
+  filter->applicator = gimp_applicator_new (node);
 
   gimp_filter_set_applicator (GIMP_FILTER (filter), filter->applicator);
 
+  gimp_applicator_set_cache (filter->applicator, TRUE);
+
   filter->has_input = gegl_node_has_pad (filter->operation, "input");
 
   if (filter->has_input)
@@ -270,6 +294,20 @@ gimp_drawable_filter_new (GimpDrawable *drawable,
   return filter;
 }
 
+void
+gimp_drawable_filter_set_clip (GimpDrawableFilter *filter,
+                               gboolean            clip)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  if (clip != filter->clip)
+    {
+      filter->clip = clip;
+
+      gimp_drawable_filter_sync_clip (filter, TRUE);
+    }
+}
+
 void
 gimp_drawable_filter_set_region (GimpDrawableFilter *filter,
                                  GimpFilterRegion    region)
@@ -287,6 +325,39 @@ gimp_drawable_filter_set_region (GimpDrawableFilter *filter,
     }
 }
 
+void
+gimp_drawable_filter_set_crop (GimpDrawableFilter  *filter,
+                               const GeglRectangle *rect,
+                               gboolean             update)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  if ((rect != NULL) != filter->crop_enabled ||
+      (rect && ! gegl_rectangle_equal (rect, &filter->crop_rect)))
+    {
+      gboolean      old_enabled = filter->crop_enabled;
+      GeglRectangle old_rect    = filter->crop_rect;
+
+      if (rect)
+        {
+          filter->crop_enabled = TRUE;
+          filter->crop_rect    = *rect;
+        }
+      else
+        {
+          filter->crop_enabled = FALSE;
+        }
+
+      gimp_drawable_filter_sync_crop (filter,
+                                      old_enabled,
+                                      &old_rect,
+                                      filter->preview_enabled,
+                                      filter->preview_alignment,
+                                      filter->preview_position,
+                                      update);
+    }
+}
+
 void
 gimp_drawable_filter_set_preview (GimpDrawableFilter  *filter,
                                   gboolean             enabled,
@@ -313,9 +384,13 @@ gimp_drawable_filter_set_preview (GimpDrawableFilter  *filter,
       filter->preview_alignment = alignment;
       filter->preview_position  = position;
 
-      gimp_drawable_filter_sync_preview (filter,
-                                         old_enabled,
-                                         old_alignment, old_position);
+      gimp_drawable_filter_sync_crop (filter,
+                                      filter->crop_enabled,
+                                      &filter->crop_rect,
+                                      old_enabled,
+                                      old_alignment,
+                                      old_position,
+                                      TRUE);
     }
 }
 
@@ -362,6 +437,23 @@ gimp_drawable_filter_set_mode (GimpDrawableFilter     *filter,
     }
 }
 
+void
+gimp_drawable_filter_set_add_alpha (GimpDrawableFilter *filter,
+                                    gboolean            add_alpha)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  if (add_alpha != filter->add_alpha)
+    {
+      filter->add_alpha = add_alpha;
+
+      gimp_drawable_filter_sync_format (filter);
+
+      if (gimp_drawable_filter_is_filtering (filter))
+        gimp_drawable_filter_update_drawable (filter, NULL);
+    }
+}
+
 void
 gimp_drawable_filter_set_color_managed (GimpDrawableFilter *filter,
                                         gboolean            color_managed)
@@ -397,6 +489,25 @@ gimp_drawable_filter_set_gamma_hack (GimpDrawableFilter *filter,
     }
 }
 
+void
+gimp_drawable_filter_set_override_constraints (GimpDrawableFilter *filter,
+                                               gboolean            override_constraints)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  if (override_constraints != filter->override_constraints)
+    {
+      filter->override_constraints = override_constraints;
+
+      gimp_drawable_filter_sync_affect (filter);
+      gimp_drawable_filter_sync_format (filter);
+      gimp_drawable_filter_sync_clip   (filter, TRUE);
+
+      if (gimp_drawable_filter_is_filtering (filter))
+        gimp_drawable_filter_update_drawable (filter, NULL);
+    }
+}
+
 void
 gimp_drawable_filter_apply (GimpDrawableFilter  *filter,
                             const GeglRectangle *area)
@@ -405,6 +516,11 @@ gimp_drawable_filter_apply (GimpDrawableFilter  *filter,
   g_return_if_fail (gimp_item_is_attached (GIMP_ITEM (filter->drawable)));
 
   gimp_drawable_filter_add_filter (filter);
+
+  gimp_drawable_filter_sync_clip (filter, TRUE);
+
+  gimp_drawable_update_bounding_box (filter->drawable);
+
   gimp_drawable_filter_update_drawable (filter, area);
 }
 
@@ -422,14 +538,28 @@ gimp_drawable_filter_commit (GimpDrawableFilter *filter,
 
   if (gimp_drawable_filter_is_filtering (filter))
     {
+      const Babl *format;
+
+      format = gimp_applicator_get_output_format (filter->applicator);
+
+      gimp_drawable_filter_set_preview (filter, FALSE,
+                                        filter->preview_alignment,
+                                        filter->preview_position);
+
       success = gimp_drawable_merge_filter (filter->drawable,
                                             GIMP_FILTER (filter),
                                             progress,
                                             gimp_object_get_name (filter),
-                                            cancellable);
+                                            format,
+                                            filter->filter_clip,
+                                            cancellable,
+                                            FALSE);
 
       gimp_drawable_filter_remove_filter (filter);
 
+      if (! success)
+        gimp_drawable_filter_update_drawable (filter, NULL);
+
       g_signal_emit (filter, drawable_filter_signals[FLUSH], 0);
     }
 
@@ -450,6 +580,45 @@ gimp_drawable_filter_abort (GimpDrawableFilter *filter)
 
 /*  private functions  */
 
+static void
+gimp_drawable_filter_sync_clip (GimpDrawableFilter *filter,
+                                gboolean            sync_region)
+{
+  gboolean clip;
+
+  if (filter->override_constraints)
+    clip = filter->clip;
+  else
+    clip = gimp_item_get_clip (GIMP_ITEM (filter->drawable), filter->clip);
+
+  if (! clip)
+    {
+      GimpImage   *image = gimp_item_get_image (GIMP_ITEM (filter->drawable));
+      GimpChannel *mask  = gimp_image_get_mask (image);
+
+      if (! gimp_channel_is_empty (mask))
+        clip = TRUE;
+    }
+
+  if (! clip)
+    {
+      GeglRectangle bounding_box;
+
+      bounding_box = gegl_node_get_bounding_box (filter->operation);
+
+      if (gegl_rectangle_is_infinite_plane (&bounding_box))
+        clip = TRUE;
+    }
+
+  if (clip != filter->filter_clip)
+    {
+      filter->filter_clip = clip;
+
+      if (sync_region)
+        gimp_drawable_filter_sync_region (filter);
+    }
+}
+
 static void
 gimp_drawable_filter_sync_region (GimpDrawableFilter *filter)
 {
@@ -468,10 +637,22 @@ gimp_drawable_filter_sync_region (GimpDrawableFilter *filter)
                          NULL);
         }
 
-      gegl_node_set (filter->crop_after,
-                     "width",  (gdouble) filter->filter_area.width,
-                     "height", (gdouble) filter->filter_area.height,
-                     NULL);
+      if (filter->filter_clip)
+        {
+          gegl_node_set (filter->crop_after,
+                         "operation", "gegl:crop",
+                         "x",         0.0,
+                         "y",         0.0,
+                         "width",     (gdouble) filter->filter_area.width,
+                         "height",    (gdouble) filter->filter_area.height,
+                         NULL);
+        }
+      else
+        {
+          gegl_node_set (filter->crop_after,
+                         "operation", "gegl:nop",
+                         NULL);
+        }
 
       gimp_applicator_set_apply_offset (filter->applicator,
                                         filter->filter_area.x,
@@ -496,113 +677,137 @@ gimp_drawable_filter_sync_region (GimpDrawableFilter *filter)
                          NULL);
         }
 
-      gegl_node_set (filter->crop_after,
-                     "width",  width,
-                     "height", height,
-                     NULL);
+      if (filter->filter_clip)
+        {
+          gegl_node_set (filter->crop_after,
+                         "operation", "gegl:crop",
+                         "x",         (gdouble) filter->filter_area.x,
+                         "y",         (gdouble) filter->filter_area.y,
+                         "width",     (gdouble) filter->filter_area.width,
+                         "height",    (gdouble) filter->filter_area.height,
+                         NULL);
+        }
+      else
+        {
+          gegl_node_set (filter->crop_after,
+                         "operation", "gegl:nop",
+                         NULL);
+        }
 
       gimp_applicator_set_apply_offset (filter->applicator, 0, 0);
     }
-}
 
-static void
-gimp_drawable_filter_get_preview_rect (GimpDrawableFilter *filter,
-                                       gboolean            enabled,
-                                       GimpAlignmentType   alignment,
-                                       gdouble             position,
-                                       GeglRectangle      *rect)
-{
-  gint width;
-  gint height;
-
-  rect->x      = 0;
-  rect->y      = 0;
-  rect->width  = gimp_item_get_width  (GIMP_ITEM (filter->drawable));
-  rect->height = gimp_item_get_height (GIMP_ITEM (filter->drawable));
-
-  width  = rect->width;
-  height = rect->height;
-
-  if (enabled)
+  if (gimp_drawable_filter_is_filtering (filter))
     {
-      switch (alignment)
-        {
-        case GIMP_ALIGN_LEFT:
-          rect->width *= position;
-          break;
-
-        case GIMP_ALIGN_RIGHT:
-          rect->width *= (1.0 - position);
-          rect->x = width - rect->width;
-          break;
-
-        case GIMP_ALIGN_TOP:
-          rect->height *= position;
-          break;
-
-        case GIMP_ALIGN_BOTTOM:
-          rect->height *= (1.0 - position);
-          rect->y = height - rect->height;
-          break;
-
-        default:
-          g_return_if_reached ();
-        }
+      if (gimp_drawable_update_bounding_box (filter->drawable))
+        g_signal_emit (filter, drawable_filter_signals[FLUSH], 0);
     }
 }
 
+static gboolean
+gimp_drawable_filter_get_crop_rect (GimpDrawableFilter  *filter,
+                                    gboolean             crop_enabled,
+                                    const GeglRectangle *crop_rect,
+                                    gboolean             preview_enabled,
+                                    GimpAlignmentType    preview_alignment,
+                                    gdouble              preview_position,
+                                    GeglRectangle       *rect)
+{
+  GeglRectangle bounds;
+  gint          width;
+  gint          height;
+  gint          x1, x2;
+  gint          y1, y2;
+
+  bounds = gegl_rectangle_infinite_plane ();
+
+  width  = gimp_item_get_width  (GIMP_ITEM (filter->drawable));
+  height = gimp_item_get_height (GIMP_ITEM (filter->drawable));
+
+  x1 = bounds.x;
+  x2 = bounds.x + bounds.width;
+
+  y1 = bounds.y;
+  y2 = bounds.y + bounds.height;
+
+  if (preview_enabled)
+    {
+      switch (preview_alignment)
+        {
+        case GIMP_ALIGN_LEFT:
+          x2 = width * preview_position;
+          break;
+
+        case GIMP_ALIGN_RIGHT:
+          x1 = width * preview_position;
+          break;
+
+        case GIMP_ALIGN_TOP:
+          y2 = height * preview_position;
+          break;
+
+        case GIMP_ALIGN_BOTTOM:
+          y1 = height * preview_position;
+          break;
+
+        default:
+          g_return_val_if_reached (FALSE);
+        }
+    }
+
+  gegl_rectangle_set (rect, x1, y1, x2 - x1, y2 - y1);
+
+  if (crop_enabled)
+    gegl_rectangle_intersect (rect, rect, crop_rect);
+
+  return ! gegl_rectangle_equal (rect, &bounds);
+}
+
 static void
-gimp_drawable_filter_sync_preview (GimpDrawableFilter *filter,
-                                   gboolean            old_enabled,
-                                   GimpAlignmentType   old_alignment,
-                                   gdouble             old_position)
+gimp_drawable_filter_sync_crop (GimpDrawableFilter  *filter,
+                                gboolean             old_crop_enabled,
+                                const GeglRectangle *old_crop_rect,
+                                gboolean             old_preview_enabled,
+                                GimpAlignmentType    old_preview_alignment,
+                                gdouble              old_preview_position,
+                                gboolean             update)
 {
   GeglRectangle old_rect;
   GeglRectangle new_rect;
+  gboolean      enabled;
 
-  gimp_drawable_filter_get_preview_rect (filter,
-                                         old_enabled,
-                                         old_alignment,
-                                         old_position,
-                                         &old_rect);
+  gimp_drawable_filter_get_crop_rect (filter,
+                                      old_crop_enabled,
+                                      old_crop_rect,
+                                      old_preview_enabled,
+                                      old_preview_alignment,
+                                      old_preview_position,
+                                      &old_rect);
 
-  gimp_drawable_filter_get_preview_rect (filter,
-                                         filter->preview_enabled,
-                                         filter->preview_alignment,
-                                         filter->preview_position,
-                                         &new_rect);
+  enabled = gimp_drawable_filter_get_crop_rect (filter,
+                                                filter->crop_enabled,
+                                                &filter->crop_rect,
+                                                filter->preview_enabled,
+                                                filter->preview_alignment,
+                                                filter->preview_position,
+                                                &new_rect);
 
-  gimp_applicator_set_preview (filter->applicator,
-                               filter->preview_enabled,
-                               &new_rect);
+  gimp_applicator_set_crop (filter->applicator, enabled ? &new_rect : NULL);
 
-  if (old_rect.x      != new_rect.x     ||
-      old_rect.y      != new_rect.y     ||
-      old_rect.width  != new_rect.width ||
-      old_rect.height != new_rect.height)
+  if (update                                     &&
+      gimp_drawable_filter_is_filtering (filter) &&
+      ! gegl_rectangle_equal (&old_rect, &new_rect))
     {
-      cairo_region_t *region;
-      gint            n_rects;
-      gint            i;
+      GeglRectangle diff_rects[4];
+      gint          n_diff_rects;
+      gint          i;
 
-      region = cairo_region_create_rectangle ((cairo_rectangle_int_t *)
-                                              &old_rect);
-      cairo_region_xor_rectangle (region,
-                                  (cairo_rectangle_int_t *) &new_rect);
+      gimp_drawable_update_bounding_box (filter->drawable);
 
-      n_rects = cairo_region_num_rectangles (region);
+      n_diff_rects = gegl_rectangle_xor (diff_rects, &old_rect, &new_rect);
 
-      for (i = 0; i < n_rects; i++)
-        {
-          cairo_rectangle_int_t rect;
-
-          cairo_region_get_rectangle (region, i, &rect);
-
-          gimp_drawable_filter_update_drawable (filter,
-                                                (const GeglRectangle *) &rect);
-        }
-
-      cairo_region_destroy (region);
+      for (i = 0; i < n_diff_rects; i++)
+        gimp_drawable_filter_update_drawable (filter, &diff_rects[i]);
     }
 }
 
@@ -637,18 +842,34 @@ gimp_drawable_filter_sync_mode (GimpDrawableFilter *filter)
 static void
 gimp_drawable_filter_sync_affect (GimpDrawableFilter *filter)
 {
-  GimpComponentMask active_mask;
+  gimp_applicator_set_affect (
+    filter->applicator,
+    filter->override_constraints ?
 
-  active_mask = gimp_drawable_get_active_mask (filter->drawable);
+      GIMP_COMPONENT_MASK_RED   |
+      GIMP_COMPONENT_MASK_GREEN |
+      GIMP_COMPONENT_MASK_BLUE  |
+      GIMP_COMPONENT_MASK_ALPHA :
 
-  /*  don't let the filter affect the drawable projection's alpha,
-   *  because it can't affect the drawable buffer's alpha either when
-   *  finally merged (see bug #699279)
-   */
-  if (! gimp_drawable_has_alpha (filter->drawable))
-    active_mask &= ~GIMP_COMPONENT_MASK_ALPHA;
+      gimp_drawable_get_active_mask (filter->drawable));
+}
 
-  gimp_applicator_set_affect (filter->applicator, active_mask);
+static void
+gimp_drawable_filter_sync_format (GimpDrawableFilter *filter)
+{
+  const Babl *format;
+
+  if (filter->add_alpha &&
+      (GIMP_IS_LAYER (filter->drawable) || filter->override_constraints))
+    {
+      format = gimp_drawable_get_format_with_alpha (filter->drawable);
+    }
+  else
+    {
+      format = gimp_drawable_get_format (filter->drawable);
+    }
+
+  gimp_applicator_set_output_format (filter->applicator, format);
 }
 
 static void
@@ -846,20 +1067,27 @@ gimp_drawable_filter_add_filter (GimpDrawableFilter *filter)
       gimp_viewable_preview_freeze (GIMP_VIEWABLE (filter->drawable));
 
       gimp_drawable_filter_sync_mask (filter);
+      gimp_drawable_filter_sync_clip (filter, FALSE);
       gimp_drawable_filter_sync_region (filter);
-      gimp_drawable_filter_sync_preview (filter,
-                                         filter->preview_enabled,
-                                         filter->preview_alignment,
-                                         filter->preview_position);
+      gimp_drawable_filter_sync_crop (filter,
+                                      filter->crop_enabled,
+                                      &filter->crop_rect,
+                                      filter->preview_enabled,
+                                      filter->preview_alignment,
+                                      filter->preview_position,
+                                      TRUE);
       gimp_drawable_filter_sync_opacity (filter);
       gimp_drawable_filter_sync_mode (filter);
       gimp_drawable_filter_sync_affect (filter);
+      gimp_drawable_filter_sync_format (filter);
       gimp_drawable_filter_sync_transform (filter);
       gimp_drawable_filter_sync_gamma_hack (filter);
 
       gimp_drawable_add_filter (filter->drawable,
                                 GIMP_FILTER (filter));
 
+      gimp_drawable_update_bounding_box (filter->drawable);
+
       g_signal_connect (image, "component-active-changed",
                         G_CALLBACK (gimp_drawable_filter_affect_changed),
                         filter);
@@ -869,10 +1097,23 @@ gimp_drawable_filter_add_filter (GimpDrawableFilter *filter)
       g_signal_connect (image, "profile-changed",
                         G_CALLBACK (gimp_drawable_filter_profile_changed),
                         filter);
+      g_signal_connect (filter->drawable, "lock-position-changed",
+                        G_CALLBACK (gimp_drawable_filter_lock_position_changed),
+                        filter);
+      g_signal_connect (filter->drawable, "format-changed",
+                        G_CALLBACK (gimp_drawable_filter_format_changed),
+                        filter);
       g_signal_connect (filter->drawable, "removed",
                         G_CALLBACK (gimp_drawable_filter_drawable_removed),
                         filter);
 
+      if (GIMP_IS_LAYER (filter->drawable))
+        {
+          g_signal_connect (filter->drawable, "lock-alpha-changed",
+                            G_CALLBACK (gimp_drawable_filter_lock_alpha_changed),
+                            filter);
+        }
+
       return TRUE;
     }
 
@@ -886,9 +1127,22 @@ gimp_drawable_filter_remove_filter (GimpDrawableFilter *filter)
     {
       GimpImage *image = gimp_item_get_image (GIMP_ITEM (filter->drawable));
 
+      if (GIMP_IS_LAYER (filter->drawable))
+        {
+          g_signal_handlers_disconnect_by_func (filter->drawable,
+                                                gimp_drawable_filter_lock_alpha_changed,
+                                                filter);
+        }
+
       g_signal_handlers_disconnect_by_func (filter->drawable,
                                             gimp_drawable_filter_drawable_removed,
                                             filter);
+      g_signal_handlers_disconnect_by_func (filter->drawable,
+                                            gimp_drawable_filter_format_changed,
+                                            filter);
+      g_signal_handlers_disconnect_by_func (filter->drawable,
+                                            gimp_drawable_filter_lock_position_changed,
+                                            filter);
       g_signal_handlers_disconnect_by_func (image,
                                             gimp_drawable_filter_profile_changed,
                                             filter);
@@ -902,6 +1156,8 @@ gimp_drawable_filter_remove_filter (GimpDrawableFilter *filter)
       gimp_drawable_remove_filter (filter->drawable,
                                    GIMP_FILTER (filter));
 
+      gimp_drawable_update_bounding_box (filter->drawable);
+
       gimp_viewable_preview_thaw (GIMP_VIEWABLE (filter->drawable));
 
       return TRUE;
@@ -914,29 +1170,34 @@ static void
 gimp_drawable_filter_update_drawable (GimpDrawableFilter  *filter,
                                       const GeglRectangle *area)
 {
+  GeglRectangle bounding_box;
   GeglRectangle update_area;
 
+  bounding_box = gimp_drawable_get_bounding_box (filter->drawable);
+
   if (area)
     {
-      if (! gimp_rectangle_intersect (area->x,
-                                      area->y,
-                                      area->width,
-                                      area->height,
-                                      filter->filter_area.x,
-                                      filter->filter_area.y,
-                                      filter->filter_area.width,
-                                      filter->filter_area.height,
-                                      &update_area.x,
-                                      &update_area.y,
-                                      &update_area.width,
-                                      &update_area.height))
+      if (! gegl_rectangle_intersect (&update_area,
+                                      area, &bounding_box))
         {
           return;
         }
     }
   else
     {
-      update_area = filter->filter_area;
+      gimp_drawable_filter_get_crop_rect (filter,
+                                          filter->crop_enabled,
+                                          &filter->crop_rect,
+                                          filter->preview_enabled,
+                                          filter->preview_alignment,
+                                          filter->preview_position,
+                                          &update_area);
+
+      if (! gegl_rectangle_intersect (&update_area,
+                                      &update_area, &bounding_box))
+        {
+          return;
+        }
     }
 
   if (update_area.width  > 0 &&
@@ -968,6 +1229,7 @@ gimp_drawable_filter_mask_changed (GimpImage          *image,
   gimp_drawable_filter_update_drawable (filter, NULL);
 
   gimp_drawable_filter_sync_mask (filter);
+  gimp_drawable_filter_sync_clip (filter, FALSE);
   gimp_drawable_filter_sync_region (filter);
 
   gimp_drawable_filter_update_drawable (filter, NULL);
@@ -981,9 +1243,33 @@ gimp_drawable_filter_profile_changed (GimpColorManaged   *managed,
   gimp_drawable_filter_update_drawable (filter, NULL);
 }
 
+static void
+gimp_drawable_filter_lock_position_changed (GimpDrawable       *drawable,
+                                            GimpDrawableFilter *filter)
+{
+  gimp_drawable_filter_sync_clip (filter, TRUE);
+  gimp_drawable_filter_update_drawable (filter, NULL);
+}
+
+static void
+gimp_drawable_filter_format_changed (GimpDrawable       *drawable,
+                                     GimpDrawableFilter *filter)
+{
+  gimp_drawable_filter_sync_format (filter);
+  gimp_drawable_filter_update_drawable (filter, NULL);
+}
+
 static void
 gimp_drawable_filter_drawable_removed (GimpDrawable       *drawable,
                                        GimpDrawableFilter *filter)
 {
   gimp_drawable_filter_remove_filter (filter);
 }
+
+static void
+gimp_drawable_filter_lock_alpha_changed (GimpLayer          *layer,
+                                         GimpDrawableFilter *filter)
+{
+  gimp_drawable_filter_sync_affect (filter);
+  gimp_drawable_filter_update_drawable (filter, NULL);
+}
