@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -524,7 +524,9 @@ static void
 gimp_image_duplicate_color_profile (GimpImage *image,
                                     GimpImage *new_image)
 {
-  GimpColorProfile *profile = gimp_image_get_color_profile (image);
+  GimpColorProfile *profile          = gimp_image_get_color_profile (image);
+  gboolean          is_color_managed = gimp_image_get_is_color_managed (image);
 
-  gimp_image_set_color_profile (new_image, profile, NULL);
+  gimp_image_set_color_profile    (new_image, profile, NULL);
+  gimp_image_set_is_color_managed (new_image, is_color_managed, FALSE);
 }
