@@ -51,6 +51,7 @@ enum
   PROP_CENTER_Y,
   PROP_SIZE,
   PROP_DISABLE_TRANSFORMATION,
+  PROP_ENABLE_REFLECTION,
 };
 
 
@@ -154,6 +155,14 @@ gimp_mandala_class_init (GimpMandalaClass *klass)
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS |
                             GIMP_SYMMETRY_PARAM_GUI);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_ENABLE_REFLECTION,
+                            "enable-reflection",
+                            _("Kaleidoscope"),
+                            _("Reflect consecutive strokes"),
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_SYMMETRY_PARAM_GUI);
 }
 
 static void
@@ -243,6 +252,10 @@ gimp_mandala_set_property (GObject      *object,
       mandala->disable_transformation = g_value_get_boolean (value);
       break;
 
+    case PROP_ENABLE_REFLECTION:
+      mandala->enable_reflection = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -271,6 +284,9 @@ gimp_mandala_get_property (GObject    *object,
     case PROP_DISABLE_TRANSFORMATION:
       g_value_set_boolean (value, mandala->disable_transformation);
       break;
+    case PROP_ENABLE_REFLECTION:
+      g_value_set_boolean (value, mandala->enable_reflection);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -441,6 +457,8 @@ gimp_mandala_update_strokes (GimpSymmetry *sym,
   GimpCoords  *coords;
   GimpMatrix3  matrix;
   gint         i;
+  gdouble slice_angle;
+  gdouble mid_slice_angle = 0.0;
 
   g_list_free_full (sym->strokes, g_free);
   sym->strokes = NULL;
@@ -448,6 +466,20 @@ gimp_mandala_update_strokes (GimpSymmetry *sym,
   coords = g_memdup (sym->origin, sizeof (GimpCoords));
   sym->strokes = g_list_prepend (sym->strokes, coords);
 
+  /* The angle of each slice, in radians. */
+  slice_angle = 2.0 * G_PI / mandala->size;
+
+  if (mandala->enable_reflection)
+    {
+      /* Find out in which slice the user is currently drawing. */
+      gdouble angle = atan2 (sym->origin->y - mandala->center_y,
+                             sym->origin->x - mandala->center_x);
+      gint slice_no = (int) floor(angle/slice_angle);
+
+      /* Angle where the middle of that slice is. */
+      mid_slice_angle = slice_no * slice_angle + slice_angle / 2.0;
+    }
+
   for (i = 1; i < mandala->size; i++)
     {
       gdouble new_x, new_y;
@@ -457,7 +489,17 @@ gimp_mandala_update_strokes (GimpSymmetry *sym,
       gimp_matrix3_translate (&matrix,
                               - mandala->center_x,
                               - mandala->center_y);
-      gimp_matrix3_rotate (&matrix, - i * 2.0 * G_PI / (gdouble) mandala->size);
+      if (mandala->enable_reflection && i % 2 == 1)
+        {
+          /* Reflecting over the mid_slice_angle axis, reflects slice without changing position. */
+          gimp_matrix3_rotate(&matrix, -mid_slice_angle);
+          gimp_matrix3_scale (&matrix, 1, -1);
+          gimp_matrix3_rotate(&matrix, mid_slice_angle - i * slice_angle);
+        }
+      else
+        {
+          gimp_matrix3_rotate (&matrix, - i * slice_angle);
+        }
       gimp_matrix3_translate (&matrix,
                               mandala->center_x,
                               mandala->center_y);
@@ -483,11 +525,30 @@ gimp_mandala_get_transform (GimpSymmetry *sym,
                             gboolean     *reflect)
 {
   GimpMandala *mandala = GIMP_MANDALA (sym);
+  gdouble     slice_angle;
 
   if (mandala->disable_transformation)
     return;
 
-  *angle = 360.0 * stroke / mandala->size;
+  slice_angle = 360.0 / mandala->size;
+
+  if (mandala->enable_reflection && stroke % 2 == 1)
+    {
+      /* Find out in which slice the user is currently drawing. */
+      gdouble origin_angle = gimp_rad_to_deg (atan2 (sym->origin->y - mandala->center_y,
+                                                     sym->origin->x - mandala->center_x));
+      gint slice_no = (int) floor(origin_angle/slice_angle);
+
+      /* Angle where the middle of that slice is. */
+      gdouble mid_slice_angle = slice_no * slice_angle + slice_angle / 2.0;
+
+      *angle = 180.0 - (-2 * mid_slice_angle + stroke * slice_angle) ;
+      *reflect = TRUE;
+    }
+  else
+    {
+      *angle = stroke * slice_angle;
+    }
 }
 
 static void
