@@ -28,6 +28,8 @@
 
 #include "dialogs-types.h"
 
+#include "config/gimpcoreconfig.h"
+
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 
@@ -38,6 +40,7 @@
 
 #include "about-dialog.h"
 #include "authors.h"
+#include "gimp-update.h"
 
 #include "gimp-intl.h"
 
@@ -53,6 +56,8 @@ typedef struct
 {
   GtkWidget   *dialog;
 
+  GtkWidget   *update_frame;
+
   GtkWidget   *anim_area;
   PangoLayout *layout;
 
@@ -79,6 +84,8 @@ static void        about_dialog_add_animation (GtkWidget       *vbox,
 static gboolean    about_dialog_anim_expose   (GtkWidget       *widget,
                                                GdkEventExpose  *event,
                                                GimpAboutDialog *dialog);
+static void        about_dialog_add_update    (GimpAboutDialog *dialog,
+                                               GimpCoreConfig  *config);
 static void        about_dialog_reshuffle     (GimpAboutDialog *dialog);
 static gboolean    about_dialog_timer         (gpointer         data);
 
@@ -87,13 +94,17 @@ static void        about_dialog_add_unstable_message
                                               (GtkWidget       *vbox);
 #endif /* GIMP_UNSTABLE */
 
+static void        about_dialog_last_release_changed
+                                              (GimpCoreConfig   *config,
+                                               const GParamSpec *pspec,
+                                               GimpAboutDialog  *dialog);
 
 GtkWidget *
-about_dialog_create (GimpContext *context)
+about_dialog_create (GimpCoreConfig *config)
 {
   static GimpAboutDialog dialog;
 
-  g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
+  g_return_val_if_fail (GIMP_IS_CORE_CONFIG (config), NULL);
 
   if (! dialog.dialog)
     {
@@ -160,6 +171,7 @@ about_dialog_create (GimpContext *context)
 #ifdef GIMP_UNSTABLE
           about_dialog_add_unstable_message (children->data);
 #endif /* GIMP_UNSTABLE */
+          about_dialog_add_update (&dialog, config);
         }
       else
         g_warning ("%s: ooops, no box in this container?", G_STRLOC);
@@ -235,7 +247,7 @@ about_dialog_add_animation (GtkWidget       *vbox,
 
   dialog->anim_area = gtk_drawing_area_new ();
   gtk_box_pack_start (GTK_BOX (vbox), dialog->anim_area, FALSE, FALSE, 0);
-  gtk_box_reorder_child (GTK_BOX (vbox), dialog->anim_area, 4);
+  gtk_box_reorder_child (GTK_BOX (vbox), dialog->anim_area, 5);
   gtk_widget_show (dialog->anim_area);
 
   dialog->layout = gtk_widget_create_pango_layout (dialog->anim_area, NULL);
@@ -251,6 +263,141 @@ about_dialog_add_animation (GtkWidget       *vbox,
                     dialog);
 }
 
+static void
+about_dialog_add_update (GimpAboutDialog *dialog,
+                         GimpCoreConfig  *config)
+{
+  GtkWidget *container;
+  GList     *children;
+  GtkWidget *vbox;
+
+  GtkWidget *frame;
+  GtkWidget *box;
+  GtkWidget *label;
+  GtkWidget *button;
+  GDateTime *datetime;
+  gchar     *date;
+  gchar     *text;
+
+  /* Get the dialog vbox. */
+  container = gtk_dialog_get_content_area (GTK_DIALOG (dialog->dialog));
+  children = gtk_container_get_children (GTK_CONTAINER (container));
+  g_return_if_fail (GTK_IS_BOX (children->data));
+  vbox = children->data;
+  g_list_free (children);
+
+  /* The preferred localized date representation without the time. */
+  datetime = g_date_time_new_from_unix_local (config->last_release_timestamp);
+  date = g_date_time_format (datetime, "%x");
+  g_date_time_unref (datetime);
+
+  /* The update frame. */
+  frame = gtk_frame_new (NULL);
+  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 2);
+
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_container_add (GTK_CONTAINER (frame), box);
+  if (config->last_known_release != NULL)
+    {
+      /* There is a newer version. */
+      GtkWidget *link;
+      GtkWidget *box2;
+      GtkWidget *image;
+
+      /* We want the frame to stand out. */
+      label = gtk_label_new (NULL);
+      text = g_strdup_printf ("<tt><b><big>%s</big></b></tt>",
+                              _("New version available!"));
+      gtk_label_set_markup (GTK_LABEL (label), text);
+      g_free (text);
+      gtk_widget_show (label);
+      gtk_frame_set_label_widget (GTK_FRAME (frame), label);
+      gtk_frame_set_label_align (GTK_FRAME (frame), 0.5, 0.5);
+      gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
+      gtk_box_reorder_child (GTK_BOX (vbox), frame, 3);
+
+      /* Explanation text with update image. */
+      box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+      gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 0);
+      gtk_widget_show (box2);
+
+      image = gtk_image_new_from_icon_name ("software-update-available",
+                                            GTK_ICON_SIZE_DIALOG);
+      gtk_box_pack_start (GTK_BOX (box2), image, FALSE, FALSE, 0);
+      gtk_widget_show (image);
+
+      if (config->last_revision > 0)
+        {
+          /* This is actually a new revision of current version. */
+          text = g_strdup_printf (_("A revision of GIMP %s was released on %s.\n"
+                                    "Even though you use the last version, it is recommended to reinstall.\n"
+                                    "New revisions come with package fixes."),
+                                  config->last_known_release, date);
+        }
+      else
+        {
+          text = g_strdup_printf (_("A new version of GIMP (%s) was released on %s.\n"
+                                    "It is recommended to update."),
+                                  config->last_known_release, date);
+        }
+      label = gtk_label_new (text);
+      g_free (text);
+      g_free (date);
+
+      gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
+      gtk_widget_show (label);
+
+      /* Finally the download link. */
+      link = gtk_link_button_new_with_label ("https://www.gimp.org/downloads/",
+                                             _("Go to download page"));
+      gtk_box_pack_start (GTK_BOX (box), link, FALSE, FALSE, 0);
+      gtk_widget_show (link);
+    }
+
+  /* Show a check update button. */
+  gtk_box_reorder_child (GTK_BOX (vbox), frame, 4);
+
+  if (config->check_update_timestamp > 0)
+    {
+      gchar *subtext;
+
+      datetime = g_date_time_new_from_unix_local (config->check_update_timestamp);
+      date = g_date_time_format (datetime, "%x");
+      subtext = g_strdup_printf (_("Last checked on %s"), date);
+      g_date_time_unref (datetime);
+      g_free (date);
+
+      text = g_strdup_printf ("%s\n<i>%s</i>",
+                              _("Check for updates"), subtext);
+      g_free (subtext);
+    }
+  else
+    {
+      text = g_strdup_printf ("%s", _("Check for updates"));
+    }
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), text);
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
+  g_free (text);
+
+  button = gtk_button_new ();
+  gtk_container_add (GTK_CONTAINER (button), label);
+  gtk_widget_show (label);
+
+  g_signal_connect (config, "notify::last-known-release",
+                    (GCallback) about_dialog_last_release_changed,
+                    dialog);
+  g_signal_connect_swapped (button, "clicked",
+                            (GCallback) gimp_update_check, config);
+  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  gtk_widget_show (box);
+  gtk_widget_show (frame);
+
+  dialog->update_frame = frame;
+}
+
 static void
 about_dialog_reshuffle (GimpAboutDialog *dialog)
 {
@@ -618,3 +765,23 @@ about_dialog_add_unstable_message (GtkWidget *vbox)
 }
 
 #endif /* GIMP_UNSTABLE */
+
+static void
+about_dialog_last_release_changed (GimpCoreConfig   *config,
+                                   const GParamSpec *pspec,
+                                   GimpAboutDialog  *dialog)
+{
+  g_signal_handlers_disconnect_by_func (config,
+                                        (GCallback) about_dialog_last_release_changed,
+                                        dialog);
+  if (! dialog->dialog)
+    return;
+
+  if (dialog->update_frame)
+    {
+      gtk_widget_destroy (dialog->update_frame);
+      dialog->update_frame = NULL;
+    }
+
+  about_dialog_add_update (dialog, config);
+}
