@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -82,8 +82,6 @@ G_DEFINE_TYPE (GimpMotionBuffer, gimp_motion_buffer, GIMP_TYPE_OBJECT)
 
 static guint motion_buffer_signals[LAST_SIGNAL] = { 0 };
 
-static const GimpCoords default_coords = GIMP_COORDS_DEFAULT_VALUES;
-
 
 static void
 gimp_motion_buffer_class_init (GimpMotionBufferClass *klass)
@@ -245,7 +243,7 @@ gimp_motion_buffer_end_stroke (GimpMotionBuffer *buffer)
  * tilt values, to allow for even more dynamic brushes. Calculated
  * distance to last event is stored in GimpCoords because its a
  * sideproduct of velocity calculation and is currently calculated in
- * each tool. If they were to use this distance, more resouces on
+ * each tool. If they were to use this distance, more resources on
  * recalculating the same value would be saved.
  *
  * Return value: %TRUE if the motion was significant enough to be
@@ -542,24 +540,22 @@ static void
 gimp_motion_buffer_interpolate_stroke (GimpMotionBuffer *buffer,
                                        GimpCoords       *coords)
 {
-  GArray *ret_coords;
-  gint    i = buffer->event_history->len - 1;
+  GimpCoords  catmull[4];
+  GArray     *ret_coords;
+  gint        i = buffer->event_history->len - 1;
 
   /* Note that there must be exactly one event in buffer or bad things
    * can happen. This must never get called under other circumstances.
    */
   ret_coords = g_array_new (FALSE, FALSE, sizeof (GimpCoords));
 
-  gimp_coords_interpolate_catmull (g_array_index (buffer->event_history,
-                                                  GimpCoords, i - 1),
-                                   g_array_index (buffer->event_history,
-                                                  GimpCoords, i),
-                                   g_array_index (buffer->event_queue,
-                                                  GimpCoords, 0),
-                                   *coords,
-                                   EVENT_FILL_PRECISION / 2,
-                                   &ret_coords,
-                                   NULL);
+  catmull[0] = g_array_index (buffer->event_history, GimpCoords, i - 1);
+  catmull[1] = g_array_index (buffer->event_history, GimpCoords, i);
+  catmull[2] = g_array_index (buffer->event_queue,   GimpCoords, 0);
+  catmull[3] = *coords;
+
+  gimp_coords_interpolate_catmull (catmull, EVENT_FILL_PRECISION / 2,
+                                   ret_coords, NULL);
 
   /* Push the last actual event in history */
   gimp_motion_buffer_push_event_history (buffer,
