@@ -20,6 +20,8 @@
 
 #include "config.h"
 
+#include <stdarg.h>
+
 #include <gegl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -56,6 +58,7 @@ enum
   SNAP_OFFSETS,
   STATUS,
   STATUS_COORDS,
+  MESSAGE,
   FOCUS_CHANGED,
   LAST_SIGNAL
 };
@@ -71,6 +74,7 @@ struct _GimpToolWidgetPrivate
   gint              snap_width;
   gint              snap_height;
 
+  gboolean          visible;
   gboolean          focus;
 };
 
@@ -173,6 +177,16 @@ gimp_tool_widget_class_init (GimpToolWidgetClass *klass)
                   G_TYPE_DOUBLE,
                   G_TYPE_STRING);
 
+  widget_signals[MESSAGE] =
+    g_signal_new ("message",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpToolWidgetClass, message),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__STRING,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_STRING);
+
   widget_signals[FOCUS_CHANGED] =
     g_signal_new ("focus-changed",
                   G_TYPE_FROM_CLASS (klass),
@@ -200,6 +214,8 @@ static void
 gimp_tool_widget_init (GimpToolWidget *widget)
 {
   widget->private = gimp_tool_widget_get_instance_private (widget);
+
+  widget->private->visible = TRUE;
 }
 
 static void
@@ -213,6 +229,8 @@ gimp_tool_widget_constructed (GObject *object)
   gimp_assert (GIMP_IS_DISPLAY_SHELL (private->shell));
 
   private->item = gimp_canvas_group_new (private->shell);
+
+  gimp_canvas_item_set_visible (private->item, private->visible);
 }
 
 static void
@@ -338,6 +356,32 @@ gimp_tool_widget_get_item (GimpToolWidget *widget)
   return widget->private->item;
 }
 
+void
+gimp_tool_widget_set_visible (GimpToolWidget *widget,
+                              gboolean        visible)
+{
+  g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
+
+  if (visible != widget->private->visible)
+    {
+      widget->private->visible = visible;
+
+      if (widget->private->item)
+        gimp_canvas_item_set_visible (widget->private->item, visible);
+
+      if (! visible)
+        gimp_tool_widget_set_status (widget, NULL);
+    }
+}
+
+gboolean
+gimp_tool_widget_get_visible (GimpToolWidget *widget)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), FALSE);
+
+  return widget->private->visible;
+}
+
 void
 gimp_tool_widget_set_focus (GimpToolWidget *widget,
                             gboolean        focus)
@@ -449,6 +493,39 @@ gimp_tool_widget_set_status_coords (GimpToolWidget *widget,
                  title, x, separator, y, help);
 }
 
+void
+gimp_tool_widget_message (GimpToolWidget *widget,
+                          const gchar    *format,
+                          ...)
+{
+  va_list  args;
+  gchar   *message;
+
+  g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
+  g_return_if_fail (format != NULL);
+
+  va_start (args, format);
+
+  message = g_strdup_vprintf (format, args);
+
+  va_end (args);
+
+  gimp_tool_widget_message_literal (widget, message);
+
+  g_free (message);
+}
+
+void
+gimp_tool_widget_message_literal (GimpToolWidget *widget,
+                                  const gchar    *message)
+{
+  g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
+  g_return_if_fail (message != NULL);
+
+  g_signal_emit (widget, widget_signals[MESSAGE], 0,
+                 message);
+}
+
 void
 gimp_tool_widget_add_item (GimpToolWidget *widget,
                            GimpCanvasItem *item)
@@ -772,10 +849,14 @@ gimp_tool_widget_button_press (GimpToolWidget      *widget,
   g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), 0);
   g_return_val_if_fail (coords != NULL, 0);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_press)
-    return GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_press (widget,
-                                                              coords, time, state,
-                                                              press_type);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_press)
+    {
+      return GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_press (widget,
+                                                                coords, time,
+                                                                state,
+                                                                press_type);
+    }
 
   return 0;
 }
@@ -790,10 +871,13 @@ gimp_tool_widget_button_release (GimpToolWidget        *widget,
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
   g_return_if_fail (coords != NULL);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_release)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_release (widget,
-                                                         coords, time, state,
-                                                         release_type);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_release)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->button_release (widget,
+                                                           coords, time, state,
+                                                           release_type);
+    }
 }
 
 void
@@ -805,9 +889,12 @@ gimp_tool_widget_motion (GimpToolWidget   *widget,
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
   g_return_if_fail (coords != NULL);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion (widget,
-                                                 coords, time, state);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion (widget,
+                                                   coords, time, state);
+    }
 }
 
 GimpHit
@@ -819,9 +906,13 @@ gimp_tool_widget_hit (GimpToolWidget   *widget,
   g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), GIMP_HIT_NONE);
   g_return_val_if_fail (coords != NULL, GIMP_HIT_NONE);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->hit)
-    return GIMP_TOOL_WIDGET_GET_CLASS (widget)->hit (widget,
-                                                     coords, state, proximity);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->hit)
+    {
+      return GIMP_TOOL_WIDGET_GET_CLASS (widget)->hit (widget,
+                                                       coords, state,
+                                                       proximity);
+    }
 
   return GIMP_HIT_NONE;
 }
@@ -835,9 +926,12 @@ gimp_tool_widget_hover (GimpToolWidget   *widget,
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
   g_return_if_fail (coords != NULL);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover (widget,
-                                                coords, state, proximity);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover (widget,
+                                                  coords, state, proximity);
+    }
 }
 
 void
@@ -845,8 +939,11 @@ gimp_tool_widget_leave_notify (GimpToolWidget *widget)
 {
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->leave_notify)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->leave_notify (widget);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->leave_notify)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->leave_notify (widget);
+    }
 }
 
 gboolean
@@ -856,8 +953,11 @@ gimp_tool_widget_key_press (GimpToolWidget *widget,
   g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), FALSE);
   g_return_val_if_fail (kevent != NULL, FALSE);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_press)
-    return GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_press (widget, kevent);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_press)
+    {
+      return GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_press (widget, kevent);
+    }
 
   return FALSE;
 }
@@ -869,8 +969,11 @@ gimp_tool_widget_key_release (GimpToolWidget *widget,
   g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), FALSE);
   g_return_val_if_fail (kevent != NULL, FALSE);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_release)
-    return GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_release (widget, kevent);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_release)
+    {
+      return GIMP_TOOL_WIDGET_GET_CLASS (widget)->key_release (widget, kevent);
+    }
 
   return FALSE;
 }
@@ -883,9 +986,12 @@ gimp_tool_widget_motion_modifier (GimpToolWidget  *widget,
 {
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion_modifier)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion_modifier (widget,
-                                                          key, press, state);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion_modifier)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->motion_modifier (widget,
+                                                            key, press, state);
+    }
 }
 
 void
@@ -896,9 +1002,12 @@ gimp_tool_widget_hover_modifier (GimpToolWidget  *widget,
 {
   g_return_if_fail (GIMP_IS_TOOL_WIDGET (widget));
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover_modifier)
-    GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover_modifier (widget,
-                                                         key, press, state);
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover_modifier)
+    {
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->hover_modifier (widget,
+                                                           key, press, state);
+    }
 }
 
 gboolean
@@ -913,7 +1022,8 @@ gimp_tool_widget_get_cursor (GimpToolWidget      *widget,
   g_return_val_if_fail (GIMP_IS_TOOL_WIDGET (widget), FALSE);
   g_return_val_if_fail (coords != NULL, FALSE);
 
-  if (GIMP_TOOL_WIDGET_GET_CLASS (widget)->get_cursor)
+  if (widget->private->visible &&
+      GIMP_TOOL_WIDGET_GET_CLASS (widget)->get_cursor)
     {
       GimpCursorType     my_cursor;
       GimpToolCursorType my_tool_cursor;
