@@ -84,6 +84,7 @@ file_open_image (Gimp                *gimp,
   GFile          *local_file  = NULL;
   gchar          *path        = NULL;
   gchar          *entered_uri = NULL;
+  gboolean        mounted     = TRUE;
   GError         *my_error    = NULL;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
@@ -150,18 +151,27 @@ file_open_image (Gimp                *gimp,
       ! file_remote_mount_file (gimp, file, progress, &my_error))
     {
       if (my_error)
-        g_propagate_error (error, my_error);
-      else
-        *status = GIMP_PDB_CANCEL;
+        {
+          g_printerr ("%s: mounting remote volume failed, trying to download"
+                      "the file: %s\n",
+                      G_STRFUNC, my_error->message);
+          g_clear_error (&my_error);
 
-      return NULL;
+          mounted = FALSE;
+        }
+      else
+        {
+          *status = GIMP_PDB_CANCEL;
+
+          return NULL;
+        }
     }
 
-  if (! file_proc || ! file_proc->handles_uri)
+  if (! file_proc || ! file_proc->handles_uri || ! mounted)
     {
-      path = g_file_get_path (file);
+      gchar *my_path = g_file_get_path (file);
 
-      if (! path)
+      if (! my_path)
         {
           local_file = file_remote_download_image (gimp, file, progress,
                                                    &my_error);
@@ -192,12 +202,22 @@ file_open_image (Gimp                *gimp,
               return NULL;
             }
 
-          path = g_file_get_path (local_file);
+          if (file_proc->handles_uri)
+            path = g_file_get_uri (local_file);
+          else
+            path = g_file_get_path (local_file);
         }
+
+      g_free (my_path);
     }
 
   if (! path)
-    path = g_file_get_uri (file);
+    {
+      if (file_proc->handles_uri)
+        path = g_file_get_uri (file);
+      else
+        path = g_file_get_path (file);
+    }
 
   entered_uri = g_file_get_uri (entered_file);
 
@@ -265,7 +285,7 @@ file_open_image (Gimp                *gimp,
     {
       if (error && ! *error)
         g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                     _("%s plug-In could not open image"),
+                     _("%s plug-in could not open image"),
                      gimp_procedure_get_label (GIMP_PROCEDURE (file_proc)));
     }
 
@@ -636,7 +656,8 @@ file_open_layers (Gimp                *gimp,
 
           layer = gimp_image_merge_visible_layers (new_image, context,
                                                    GIMP_CLIP_TO_IMAGE,
-                                                   FALSE, FALSE);
+                                                   FALSE, FALSE,
+                                                   NULL);
 
           layers = g_list_prepend (NULL, layer);
         }
@@ -752,9 +773,7 @@ file_open_sanitize_image (GimpImage *image,
    * layers, this is needed in case something triggers projection or
    * image preview creation before all layers are loaded, see bug #767663.
    */
-  gimp_image_invalidate (image, 0, 0,
-                         gimp_image_get_width  (image),
-                         gimp_image_get_height (image));
+  gimp_image_invalidate_all (image);
 
   /* Make sure all image states are up-to-date */
   gimp_image_flush (image);
