@@ -25,12 +25,11 @@
 #include <gegl.h>
 #include <gegl-plugin.h>
 
-#include "libgimpmath/gimpmath.h"
-
 #include "gimp-gegl-types.h"
 
 #include "core/gimpprogress.h"
 
+#include "gimp-gegl-loops.h"
 #include "gimp-gegl-utils.h"
 
 
@@ -149,6 +148,41 @@ gimp_gegl_node_is_point_operation (GeglNode *node)
          GEGL_IS_OPERATION_POINT_COMPOSER3 (operation);
 }
 
+gboolean
+gimp_gegl_node_is_area_filter_operation (GeglNode *node)
+{
+  GeglOperation *operation;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), FALSE);
+
+  operation = gegl_node_get_gegl_operation (node);
+
+  if (! operation)
+    return FALSE;
+
+  return GEGL_IS_OPERATION_AREA_FILTER (operation) ||
+         /* be conservative and return TRUE for meta ops, since they may
+          * involve an area op
+          */
+         GEGL_IS_OPERATION_META (operation);
+}
+
+const gchar *
+gimp_gegl_node_get_key (GeglNode    *node,
+                        const gchar *key)
+{
+  const gchar *operation_name;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+
+  operation_name = gegl_node_get_operation (node);
+
+  if (operation_name)
+    return gegl_operation_get_key (operation_name, key);
+  else
+    return NULL;
+}
+
 const Babl *
 gimp_gegl_node_get_format (GeglNode    *node,
                            const gchar *pad_name)
@@ -214,20 +248,22 @@ gimp_gegl_param_spec_has_key (GParamSpec  *pspec,
   return FALSE;
 }
 
-void
-gimp_gegl_rectangle_align_to_tile_grid (GeglRectangle       *dest,
-                                        const GeglRectangle *src,
-                                        GeglBuffer          *buffer)
+GeglBuffer *
+gimp_gegl_buffer_dup (GeglBuffer *buffer)
 {
-  gint          shift_x;
-  gint          shift_y;
-  gint          tile_width;
-  gint          tile_height;
-  GeglRectangle rect;
+  GeglBuffer          *new_buffer;
+  const GeglRectangle *extent;
+  const GeglRectangle *abyss;
+  GeglRectangle        rect;
+  gint                 shift_x;
+  gint                 shift_y;
+  gint                 tile_width;
+  gint                 tile_height;
 
-  g_return_if_fail (dest != NULL);
-  g_return_if_fail (src != NULL);
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
+  extent = gegl_buffer_get_extent (buffer);
+  abyss  = gegl_buffer_get_abyss  (buffer);
 
   g_object_get (buffer,
                 "shift-x",     &shift_x,
@@ -236,14 +272,70 @@ gimp_gegl_rectangle_align_to_tile_grid (GeglRectangle       *dest,
                 "tile-height", &tile_height,
                 NULL);
 
-  rect.x      = (gint) floor ((gdouble) (src->x               + shift_x) /
-                              tile_width)  * tile_width;
-  rect.y      = (gint) floor ((gdouble) (src->y               + shift_y) /
-                              tile_height) * tile_height;
-  rect.width  = (gint) ceil  ((gdouble) (src->x + src->width  + shift_x) /
-                              tile_width)  * tile_width  - rect.x;
-  rect.height = (gint) ceil  ((gdouble) (src->y + src->height + shift_y) /
-                              tile_height) * tile_height - rect.y;
+  new_buffer = g_object_new (GEGL_TYPE_BUFFER,
+                             "format",       gegl_buffer_get_format (buffer),
+                             "x",            extent->x,
+                             "y",            extent->y,
+                             "width",        extent->width,
+                             "height",       extent->height,
+                             "abyss-x",      abyss->x,
+                             "abyss-y",      abyss->y,
+                             "abyss-width",  abyss->width,
+                             "abyss-height", abyss->height,
+                             "shift-x",      shift_x,
+                             "shift-y",      shift_y,
+                             "tile-width",   tile_width,
+                             "tile-height",  tile_height,
+                             NULL);
 
-  *dest = rect;
+  gegl_rectangle_align_to_buffer (&rect, extent, buffer,
+                                  GEGL_RECTANGLE_ALIGNMENT_SUPERSET);
+
+  gimp_gegl_buffer_copy (buffer, &rect, GEGL_ABYSS_NONE,
+                         new_buffer, &rect);
+
+  return new_buffer;
+}
+
+gboolean
+gimp_gegl_buffer_set_extent (GeglBuffer          *buffer,
+                             const GeglRectangle *extent)
+{
+  GeglRectangle aligned_old_extent;
+  GeglRectangle aligned_extent;
+  GeglRectangle old_extent_rem;
+  GeglRectangle diff_rects[4];
+  gint          n_diff_rects;
+  gint          i;
+
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), FALSE);
+  g_return_val_if_fail (extent != NULL, FALSE);
+
+  gegl_rectangle_align_to_buffer (&aligned_old_extent,
+                                  gegl_buffer_get_extent (buffer), buffer,
+                                  GEGL_RECTANGLE_ALIGNMENT_SUPERSET);
+  gegl_rectangle_align_to_buffer (&aligned_extent,
+                                  extent, buffer,
+                                  GEGL_RECTANGLE_ALIGNMENT_SUPERSET);
+
+  n_diff_rects = gegl_rectangle_subtract (diff_rects,
+                                          &aligned_old_extent,
+                                          &aligned_extent);
+
+  for (i = 0; i < n_diff_rects; i++)
+    gegl_buffer_clear (buffer, &diff_rects[i]);
+
+  if (gegl_rectangle_intersect (&old_extent_rem,
+                                gegl_buffer_get_extent (buffer),
+                                &aligned_extent))
+    {
+      n_diff_rects = gegl_rectangle_subtract (diff_rects,
+                                              &old_extent_rem,
+                                              extent);
+
+      for (i = 0; i < n_diff_rects; i++)
+        gegl_buffer_clear (buffer, &diff_rects[i]);
+    }
+
+  return gegl_buffer_set_extent (buffer, extent);
 }
