@@ -22,9 +22,19 @@
 
 #include "gimp-gegl-types.h"
 
+#include "core/gimpmarshal.h"
+
+#include "gimp-gegl-loops.h"
+#include "gimp-gegl-utils.h"
 #include "gimptilehandlervalidate.h"
 
 
+enum
+{
+  INVALIDATED,
+  LAST_SIGNAL
+};
+
 enum
 {
   PROP_0,
@@ -35,28 +45,33 @@ enum
 };
 
 
-static void     gimp_tile_handler_validate_finalize      (GObject         *object);
-static void     gimp_tile_handler_validate_set_property  (GObject         *object,
-                                                          guint            property_id,
-                                                          const GValue    *value,
-                                                          GParamSpec      *pspec);
-static void     gimp_tile_handler_validate_get_property  (GObject         *object,
-                                                          guint            property_id,
-                                                          GValue          *value,
-                                                          GParamSpec      *pspec);
+static void     gimp_tile_handler_validate_finalize             (GObject         *object);
+static void     gimp_tile_handler_validate_set_property         (GObject         *object,
+                                                                 guint            property_id,
+                                                                 const GValue    *value,
+                                                                 GParamSpec      *pspec);
+static void     gimp_tile_handler_validate_get_property         (GObject         *object,
+                                                                 guint            property_id,
+                                                                 GValue          *value,
+                                                                 GParamSpec      *pspec);
 
-static void     gimp_tile_handler_validate_real_validate (GimpTileHandlerValidate *validate,
-                                                          const GeglRectangle     *rect,
-                                                          const Babl              *format,
-                                                          gpointer                 dest_buf,
-                                                          gint                     dest_stride);
+static void     gimp_tile_handler_validate_real_begin_validate  (GimpTileHandlerValidate *validate);
+static void     gimp_tile_handler_validate_real_end_validate    (GimpTileHandlerValidate *validate);
+static void     gimp_tile_handler_validate_real_validate        (GimpTileHandlerValidate *validate,
+                                                                 const GeglRectangle     *rect,
+                                                                 const Babl              *format,
+                                                                 gpointer                 dest_buf,
+                                                                 gint                     dest_stride);
+static void     gimp_tile_handler_validate_real_validate_buffer (GimpTileHandlerValidate *validate,
+                                                                 const GeglRectangle     *rect,
+                                                                 GeglBuffer              *buffer);
 
-static gpointer gimp_tile_handler_validate_command       (GeglTileSource  *source,
-                                                          GeglTileCommand  command,
-                                                          gint             x,
-                                                          gint             y,
-                                                          gint             z,
-                                                          gpointer         data);
+static gpointer gimp_tile_handler_validate_command              (GeglTileSource  *source,
+                                                                 GeglTileCommand  command,
+                                                                 gint             x,
+                                                                 gint             y,
+                                                                 gint             z,
+                                                                 gpointer         data);
 
 
 G_DEFINE_TYPE (GimpTileHandlerValidate, gimp_tile_handler_validate,
@@ -64,17 +79,32 @@ G_DEFINE_TYPE (GimpTileHandlerValidate, gimp_tile_handler_validate,
 
 #define parent_class gimp_tile_handler_validate_parent_class
 
+static guint gimp_tile_handler_validate_signals[LAST_SIGNAL];
+
 
 static void
 gimp_tile_handler_validate_class_init (GimpTileHandlerValidateClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  gimp_tile_handler_validate_signals[INVALIDATED] =
+    g_signal_new ("invalidated",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpTileHandlerValidateClass, invalidated),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__BOXED,
+                  G_TYPE_NONE, 1,
+                  GEGL_TYPE_RECTANGLE);
+
   object_class->finalize     = gimp_tile_handler_validate_finalize;
   object_class->set_property = gimp_tile_handler_validate_set_property;
   object_class->get_property = gimp_tile_handler_validate_get_property;
 
+  klass->begin_validate      = gimp_tile_handler_validate_real_begin_validate;
+  klass->end_validate        = gimp_tile_handler_validate_real_end_validate;
   klass->validate            = gimp_tile_handler_validate_real_validate;
+  klass->validate_buffer     = gimp_tile_handler_validate_real_validate_buffer;
 
   g_object_class_install_property (object_class, PROP_FORMAT,
                                    g_param_spec_pointer ("format", NULL, NULL,
@@ -178,6 +208,18 @@ gimp_tile_handler_validate_get_property (GObject    *object,
     }
 }
 
+static void
+gimp_tile_handler_validate_real_begin_validate (GimpTileHandlerValidate *validate)
+{
+  validate->suspend_validate++;
+}
+
+static void
+gimp_tile_handler_validate_real_end_validate (GimpTileHandlerValidate *validate)
+{
+  validate->suspend_validate--;
+}
+
 static void
 gimp_tile_handler_validate_real_validate (GimpTileHandlerValidate *validate,
                                           const GeglRectangle     *rect,
@@ -198,19 +240,49 @@ gimp_tile_handler_validate_real_validate (GimpTileHandlerValidate *validate,
                   GEGL_BLIT_DEFAULT);
 }
 
+static void
+gimp_tile_handler_validate_real_validate_buffer (GimpTileHandlerValidate *validate,
+                                                 const GeglRectangle     *rect,
+                                                 GeglBuffer              *buffer)
+{
+  GimpTileHandlerValidateClass *klass;
+
+  klass = GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate);
+
+  if (klass->validate == gimp_tile_handler_validate_real_validate)
+    {
+      gegl_node_blit_buffer (validate->graph, buffer, rect, 0,
+                             GEGL_ABYSS_NONE);
+    }
+  else
+    {
+      const Babl *format = gegl_buffer_get_format (buffer);
+      gpointer    data;
+      gint        stride;
+
+      data = gegl_buffer_linear_open (buffer, rect, &stride, format);
+
+      klass->validate (validate, rect, format, data, stride);
+
+      gegl_buffer_linear_close (buffer, data);
+    }
+}
+
 static GeglTile *
-gimp_tile_handler_validate_validate (GeglTileSource *source,
-                                     GeglTile       *tile,
-                                     gint            x,
-                                     gint            y)
+gimp_tile_handler_validate_validate_tile (GeglTileSource *source,
+                                          gint            x,
+                                          gint            y)
 {
   GimpTileHandlerValidate *validate = GIMP_TILE_HANDLER_VALIDATE (source);
+  GeglTile                *tile;
   cairo_rectangle_int_t    tile_rect;
+  cairo_region_overlap_t   overlap;
 
   if (validate->suspend_validate ||
       cairo_region_is_empty (validate->dirty_region))
     {
-      return tile;
+      return gegl_tile_handler_source_command (source,
+                                               GEGL_TILE_GET, x, y, 0, NULL);
     }
 
   tile_rect.x      = x * validate->tile_width;
@@ -218,95 +290,115 @@ gimp_tile_handler_validate_validate (GeglTileSource *source,
   tile_rect.width  = validate->tile_width;
   tile_rect.height = validate->tile_height;
 
-  if (validate->whole_tile)
+  overlap = cairo_region_contains_rectangle (validate->dirty_region,
+                                             &tile_rect);
+
+  if (overlap == CAIRO_REGION_OVERLAP_OUT)
     {
-      if (cairo_region_contains_rectangle (validate->dirty_region, &tile_rect)
-          != CAIRO_REGION_OVERLAP_OUT)
-        {
-          gint tile_bpp;
-          gint tile_stride;
+      return gegl_tile_handler_source_command (source,
+                                               GEGL_TILE_GET, x, y, 0, NULL);
+    }
 
-          if (! tile)
-            tile = gegl_tile_handler_create_tile (GEGL_TILE_HANDLER (source),
-                                                  x, y, 0);
+  if (overlap == CAIRO_REGION_OVERLAP_IN || validate->whole_tile)
+    {
+      gint tile_bpp;
+      gint tile_stride;
 
-          cairo_region_subtract_rectangle (validate->dirty_region, &tile_rect);
+      cairo_region_subtract_rectangle (validate->dirty_region, &tile_rect);
 
-          tile_bpp    = babl_format_get_bytes_per_pixel (validate->format);
-          tile_stride = tile_bpp * validate->tile_width;
+      tile_bpp    = babl_format_get_bytes_per_pixel (validate->format);
+      tile_stride = tile_bpp * validate->tile_width;
 
-          gegl_tile_lock (tile);
+      tile = gegl_tile_handler_get_source_tile (GEGL_TILE_HANDLER (source),
+                                                x, y, 0, FALSE);
 
-          GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->validate
-            (validate,
-             GEGL_RECTANGLE (tile_rect.x,
-                             tile_rect.y,
-                             tile_rect.width,
-                             tile_rect.height),
-             validate->format,
-             gegl_tile_get_data (tile),
-             tile_stride);
+      gimp_tile_handler_validate_begin_validate (validate);
 
-          gegl_tile_unlock (tile);
-        }
+      gegl_tile_lock (tile);
+
+      GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->validate
+        (validate,
+         GEGL_RECTANGLE (tile_rect.x,
+                         tile_rect.y,
+                         tile_rect.width,
+                         tile_rect.height),
+         validate->format,
+         gegl_tile_get_data (tile),
+         tile_stride);
+
+      gegl_tile_unlock (tile);
+
+      gimp_tile_handler_validate_end_validate (validate);
     }
   else
     {
-      cairo_region_t *tile_region = cairo_region_copy (validate->dirty_region);
+      cairo_region_t *tile_region;
+      gint            tile_bpp;
+      gint            tile_stride;
+      gint            n_rects;
+      gint            i;
 
+      tile_region = cairo_region_copy (validate->dirty_region);
       cairo_region_intersect_rectangle (tile_region, &tile_rect);
 
-      if (! cairo_region_is_empty (tile_region))
+      cairo_region_subtract_rectangle (validate->dirty_region, &tile_rect);
+
+      tile_bpp    = babl_format_get_bytes_per_pixel (validate->format);
+      tile_stride = tile_bpp * validate->tile_width;
+
+      tile = gegl_tile_handler_source_command (source,
+                                               GEGL_TILE_GET, x, y, 0, NULL);
+
+      if (! tile)
         {
-          gint tile_bpp;
-          gint tile_stride;
-          gint n_rects;
-          gint i;
+          tile = gegl_tile_handler_create_tile (GEGL_TILE_HANDLER (source),
+                                                x, y, 0);
 
-          cairo_region_subtract_rectangle (validate->dirty_region, &tile_rect);
+          memset (gegl_tile_get_data (tile),
+                  0, tile_stride * validate->tile_height);
+        }
 
-          tile_bpp    = babl_format_get_bytes_per_pixel (validate->format);
-          tile_stride = tile_bpp * validate->tile_width;
+      gimp_tile_handler_validate_begin_validate (validate);
 
-          if (! tile)
-            {
-              tile = gegl_tile_handler_create_tile (GEGL_TILE_HANDLER (source),
-                                                    x, y, 0);
+      gegl_tile_lock (tile);
 
-              memset (gegl_tile_get_data (tile),
-                      0, tile_stride * validate->tile_height);
-            }
-
-          gegl_tile_lock (tile);
-
-          n_rects = cairo_region_num_rectangles (tile_region);
+      n_rects = cairo_region_num_rectangles (tile_region);
 
 #if 0
-          g_printerr ("%d chunks\n", n_rects);
+      g_printerr ("%d chunks\n", n_rects);
 #endif
 
-          for (i = 0; i < n_rects; i++)
-            {
-              cairo_rectangle_int_t blit_rect;
+      for (i = 0; i < n_rects; i++)
+        {
+          cairo_rectangle_int_t blit_rect;
+          gint                  tile_x;
+          gint                  tile_y;
 
-              cairo_region_get_rectangle (tile_region, i, &blit_rect);
+          cairo_region_get_rectangle (tile_region, i, &blit_rect);
 
-              GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->validate
-                (validate,
-                 GEGL_RECTANGLE (blit_rect.x,
-                                 blit_rect.y,
-                                 blit_rect.width,
-                                 blit_rect.height),
-                 validate->format,
-                 gegl_tile_get_data (tile) +
-                 (blit_rect.y % validate->tile_height) * tile_stride +
-                 (blit_rect.x % validate->tile_width)  * tile_bpp,
-                 tile_stride);
-            }
+          tile_x = blit_rect.x % validate->tile_width;
+          if (tile_x < 0) tile_x += validate->tile_width;
 
-          gegl_tile_unlock (tile);
+          tile_y = blit_rect.y % validate->tile_height;
+          if (tile_y < 0) tile_y += validate->tile_height;
+
+          GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->validate
+            (validate,
+             GEGL_RECTANGLE (blit_rect.x,
+                             blit_rect.y,
+                             blit_rect.width,
+                             blit_rect.height),
+             validate->format,
+             gegl_tile_get_data (tile) +
+             tile_y * tile_stride      +
+             tile_x * tile_bpp,
+             tile_stride);
         }
 
+      gegl_tile_unlock (tile);
+
+      gimp_tile_handler_validate_end_validate (validate);
+
       cairo_region_destroy (tile_region);
     }
 
@@ -321,14 +413,10 @@ gimp_tile_handler_validate_command (GeglTileSource  *source,
                                     gint             z,
                                     gpointer         data)
 {
-  gpointer retval;
-
-  retval = gegl_tile_handler_source_command (source, command, x, y, z, data);
-
   if (command == GEGL_TILE_GET && z == 0)
-    retval = gimp_tile_handler_validate_validate (source, retval, x, y);
+    return gimp_tile_handler_validate_validate_tile (source, x, y);
 
-  return retval;
+  return gegl_tile_handler_source_command (source, command, x, y, z, data);
 }
 
 
@@ -402,6 +490,9 @@ gimp_tile_handler_validate_invalidate (GimpTileHandlerValidate *validate,
                                 (cairo_rectangle_int_t *) rect);
 
   gegl_tile_handler_damage_rect (GEGL_TILE_HANDLER (validate), rect);
+
+  g_signal_emit (validate, gimp_tile_handler_validate_signals[INVALIDATED],
+                 0, rect, NULL);
 }
 
 void
@@ -415,6 +506,122 @@ gimp_tile_handler_validate_undo_invalidate (GimpTileHandlerValidate *validate,
                                    (cairo_rectangle_int_t *) rect);
 }
 
+void
+gimp_tile_handler_validate_begin_validate (GimpTileHandlerValidate *validate)
+{
+  g_return_if_fail (GIMP_IS_TILE_HANDLER_VALIDATE (validate));
+
+  if (validate->validating++ == 0)
+    GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->begin_validate (validate);
+}
+
+void
+gimp_tile_handler_validate_end_validate (GimpTileHandlerValidate *validate)
+{
+  g_return_if_fail (GIMP_IS_TILE_HANDLER_VALIDATE (validate));
+  g_return_if_fail (validate->validating > 0);
+
+  if (--validate->validating == 0)
+    GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate)->end_validate (validate);
+}
+
+void
+gimp_tile_handler_validate_validate (GimpTileHandlerValidate *validate,
+                                     GeglBuffer              *buffer,
+                                     const GeglRectangle     *rect,
+                                     gboolean                 intersect)
+{
+  GimpTileHandlerValidateClass *klass;
+
+  g_return_if_fail (GIMP_IS_TILE_HANDLER_VALIDATE (validate));
+  g_return_if_fail (gimp_tile_handler_validate_get_assigned (buffer) ==
+                    validate);
+
+  klass = GIMP_TILE_HANDLER_VALIDATE_GET_CLASS (validate);
+
+  if (! rect)
+    rect = gegl_buffer_get_extent (buffer);
+
+  if (intersect)
+    {
+      cairo_region_t *region = cairo_region_copy (validate->dirty_region);
+
+      cairo_region_intersect_rectangle (region,
+                                        (const cairo_rectangle_int_t *) rect);
+
+      if (! cairo_region_is_empty (region))
+        {
+          gint n_rects;
+          gint i;
+
+          gimp_tile_handler_validate_begin_validate (validate);
+
+          n_rects = cairo_region_num_rectangles (region);
+
+          for (i = 0; i < n_rects; i++)
+            {
+              cairo_rectangle_int_t blit_rect;
+
+              cairo_region_get_rectangle (region, i, &blit_rect);
+
+              klass->validate_buffer (validate,
+                                      (const GeglRectangle *) &blit_rect,
+                                      buffer);
+            }
+
+          gimp_tile_handler_validate_end_validate (validate);
+
+          cairo_region_subtract_rectangle (
+            validate->dirty_region,
+            (const cairo_rectangle_int_t *) rect);
+        }
+
+      cairo_region_destroy (region);
+    }
+  else
+    {
+      gimp_tile_handler_validate_begin_validate (validate);
+
+      klass->validate_buffer (validate, rect, buffer);
+
+      gimp_tile_handler_validate_end_validate (validate);
+
+      cairo_region_subtract_rectangle (
+            validate->dirty_region,
+            (const cairo_rectangle_int_t *) rect);
+    }
+}
+
+gboolean
+gimp_tile_handler_validate_buffer_set_extent (GeglBuffer          *buffer,
+                                              const GeglRectangle *extent)
+{
+  GimpTileHandlerValidate *validate;
+
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), FALSE);
+  g_return_val_if_fail (extent != NULL, FALSE);
+
+  validate = gimp_tile_handler_validate_get_assigned (buffer);
+
+  g_return_val_if_fail (validate != NULL, FALSE);
+
+  validate->suspend_validate++;
+
+  if (gimp_gegl_buffer_set_extent (buffer, extent))
+    {
+      validate->suspend_validate--;
+
+      cairo_region_intersect_rectangle (validate->dirty_region,
+                                        (const cairo_rectangle_int_t *) extent);
+
+      return TRUE;
+    }
+
+  validate->suspend_validate--;
+
+  return FALSE;
+}
+
 void
 gimp_tile_handler_validate_buffer_copy (GeglBuffer          *src_buffer,
                                         const GeglRectangle *src_rect,
@@ -454,16 +661,31 @@ gimp_tile_handler_validate_buffer_copy (GeglBuffer          *src_buffer,
   real_src_rect.width  = CLAMP (real_src_rect.width,  0, real_dst_rect.width);
   real_src_rect.height = CLAMP (real_src_rect.height, 0, real_dst_rect.height);
 
+  /* temporarily remove the source buffer's validate handler, so that
+   * gegl_buffer_copy() can use fast tile copying, using the TILE_COPY command.
+   * currently, gegl only uses TILE_COPY when the source buffer has no user-
+   * provided tile handlers.
+   */
   if (src_validate)
-    src_validate->suspend_validate++;
+    {
+      g_object_ref (src_validate);
+
+      gimp_tile_handler_validate_unassign (src_validate, src_buffer);
+    }
+
   dst_validate->suspend_validate++;
 
-  gegl_buffer_copy (src_buffer, &real_src_rect, GEGL_ABYSS_NONE,
-                    dst_buffer, &real_dst_rect);
+  gimp_gegl_buffer_copy (src_buffer, &real_src_rect, GEGL_ABYSS_NONE,
+                         dst_buffer, &real_dst_rect);
+
+  dst_validate->suspend_validate--;
 
   if (src_validate)
-    src_validate->suspend_validate--;
-  dst_validate->suspend_validate--;
+    {
+      gimp_tile_handler_validate_assign (src_validate, src_buffer);
+
+      g_object_unref (src_validate);
+    }
 
   cairo_region_subtract_rectangle (dst_validate->dirty_region,
                                    (cairo_rectangle_int_t *) &real_dst_rect);
