@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -36,6 +36,7 @@
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpimagewindow.h"
 
 #include "file/file-open.h"
 
@@ -166,6 +167,7 @@ gui_unique_win32_message_handler (HWND   hWnd,
       if (unique_gimp)
         {
           COPYDATASTRUCT *copydata = (COPYDATASTRUCT *) lParam;
+          GimpObject     *display;
 
           if (copydata->cbData > 0)
             {
@@ -193,6 +195,11 @@ gui_unique_win32_message_handler (HWND   hWnd,
               g_source_attach (source, NULL);
               g_source_unref (source);
             }
+
+          /* Deiconify the window if minimized. */
+          display = gimp_container_get_first_child (unique_gimp->displays);
+          if (display)
+            gimp_display_shell_present (gimp_display_get_shell (GIMP_DISPLAY (display)));
         }
       return TRUE;
 
@@ -334,7 +341,7 @@ gui_unique_quartz_init (Gimp *gimp)
                     G_CALLBACK (gui_unique_quartz_nsopen_file_callback),
                     gimp);
 
-  /* Using the event handler is a hack, it is neccesary becuase
+  /* Using the event handler is a hack, it is necessary because
    * gtkosx_application will drop the file open events if any
    * event processing is done before gtkosx_application_ready is
    * called, which we unfortuantly can't avoid doing right now.
