@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /* Win32 language lookup table:
@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include <locale.h>
+
 #include <glib.h>
 
 #ifdef G_OS_WIN32
@@ -30,7 +31,6 @@
 #endif
 
 #include "language.h"
-#include "gimp-intl.h"
 
 
 void
@@ -735,5 +735,6 @@ language_init (const gchar *language)
     return;
 
   g_setenv ("LANGUAGE", language, TRUE);
+  g_setenv ("LC_TIME", language, TRUE);
   setlocale (LC_ALL, "");
 }
