@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_OPERATIONS_H__
 #define __GIMP_OPERATIONS_H__
 
 
-void   gimp_operations_init (void);
+void   gimp_operations_init (Gimp *gimp);
 
 
 #endif /* __GIMP_OPERATIONS_H__ */
