@@ -23,6 +23,7 @@
 #include <string.h>
 
 #include <gegl-plugin.h>
+#include <gegl-buffer-backend.h>
 
 #include "../operations-types.h"
 
@@ -31,12 +32,12 @@
 
 
 static GeglRectangle              gimp_operation_replace_get_bounding_box    (GeglOperation        *op);
+
 static gboolean                   gimp_operation_replace_parent_process      (GeglOperation        *op,
                                                                               GeglOperationContext *context,
                                                                               const gchar          *output_prop,
                                                                               const GeglRectangle  *result,
                                                                               gint                  level);
-
 static gboolean                   gimp_operation_replace_process             (GeglOperation          *op,
                                                                               void                   *in,
                                                                               void                   *layer,
@@ -66,8 +67,8 @@ gimp_operation_replace_class_init (GimpOperationReplaceClass *klass)
                                  NULL);
 
   operation_class->get_bounding_box     = gimp_operation_replace_get_bounding_box;
-  operation_class->process              = gimp_operation_replace_parent_process;
 
+  layer_mode_class->parent_process      = gimp_operation_replace_parent_process;
   layer_mode_class->process             = gimp_operation_replace_process;
   layer_mode_class->get_affected_region = gimp_operation_replace_get_affected_region;
 }
@@ -114,9 +115,9 @@ gimp_operation_replace_get_bounding_box (GeglOperation *op)
                                                              self->composite_mode);
     }
 
-  if (self->opacity == 0.0)
+  if (self->prop_opacity == 0.0)
     included_region &= ~GIMP_LAYER_COMPOSITE_REGION_SOURCE;
-  else if (self->opacity == 1.0 && ! aux2_rect)
+  else if (self->prop_opacity == 1.0 && ! aux2_rect)
     included_region &= ~GIMP_LAYER_COMPOSITE_REGION_DESTINATION;
 
   gegl_rectangle_intersect (&result, &src_rect, &dst_rect);
@@ -137,17 +138,19 @@ gimp_operation_replace_parent_process (GeglOperation        *op,
                                        const GeglRectangle  *result,
                                        gint                  level)
 {
-  GimpOperationLayerMode *layer_mode = (gpointer) op;
+  GimpOperationLayerMode   *layer_mode = (gpointer) op;
+  GimpLayerCompositeRegion  included_region;
+
+  included_region = gimp_layer_mode_get_included_region
+    (layer_mode->layer_mode, layer_mode->composite_mode);
 
   /* if the layer's opacity is 100%, it has no mask, and its composite mode
    * contains "aux" (the latter should always be the case in practice,
    * currently,) we can just pass "aux" directly as output.
    */
-  if (layer_mode->opacity == 1.0                            &&
+  if (layer_mode->opacity == 1.0                       &&
       ! gegl_operation_context_get_object (context, "aux2") &&
-      (gimp_layer_mode_get_included_region (layer_mode->layer_mode,
-                                            layer_mode->real_composite_mode) &
-       GIMP_LAYER_COMPOSITE_REGION_SOURCE))
+      (included_region & GIMP_LAYER_COMPOSITE_REGION_SOURCE))
     {
       GObject *aux;
 
@@ -160,9 +163,69 @@ gimp_operation_replace_parent_process (GeglOperation        *op,
   /* the opposite case, where the opacity is 0%, is handled by
    * GimpOperationLayerMode.
    */
+  else if (layer_mode->opacity == 0.0)
+    {
+    }
+  /* if both buffers are included in the result, and if both of them have the
+   * same content -- i.e., if they share the same storage, same alignment, and
+   * same abyss (or if the abyss is irrelevant) -- we can just pass either of
+   * them directly as output.
+   */
+  else if (included_region == GIMP_LAYER_COMPOSITE_REGION_UNION)
+    {
+      GObject *input;
+      GObject *aux;
 
-  return GEGL_OPERATION_CLASS (parent_class)->process (op, context, output_prop,
-                                                       result, level);
+      input = gegl_operation_context_get_object (context, "input");
+      aux   = gegl_operation_context_get_object (context, "aux");
+
+      if (input && aux)
+        {
+          GObject *buffer = aux;
+
+          while (buffer != input && GEGL_IS_BUFFER (buffer))
+            buffer = (GObject *) gegl_tile_handler_get_source (buffer);
+
+          if (buffer == input)
+            {
+              gint input_shift_x;
+              gint input_shift_y;
+              gint aux_shift_x;
+              gint aux_shift_y;
+
+              g_object_get (input,
+                            "shift-x", &input_shift_x,
+                            "shift-y", &input_shift_y,
+                            NULL);
+              g_object_get (aux,
+                            "shift-x", &aux_shift_x,
+                            "shift-y", &aux_shift_y,
+                            NULL);
+
+              if (input_shift_x == aux_shift_x && input_shift_y == aux_shift_y)
+                {
+                  const GeglRectangle *input_abyss;
+                  const GeglRectangle *aux_abyss;
+
+                  input_abyss = gegl_buffer_get_abyss (GEGL_BUFFER (input));
+                  aux_abyss   = gegl_buffer_get_abyss (GEGL_BUFFER (aux));
+
+                  if (gegl_rectangle_equal (input_abyss, aux_abyss)  ||
+                      (gegl_rectangle_contains (input_abyss, result) &&
+                       gegl_rectangle_contains (aux_abyss,   result)))
+                    {
+                      gegl_operation_context_set_object (context,
+                                                         "output", input);
+
+                      return TRUE;
+                    }
+                }
+            }
+        }
+    }
+
+  return GIMP_OPERATION_LAYER_MODE_CLASS (parent_class)->parent_process (
+    op, context, output_prop, result, level);
 }
 
 static gboolean
@@ -183,7 +246,7 @@ gimp_operation_replace_process (GeglOperation       *op,
   gfloat                  opacity    = layer_mode->opacity;
   const gboolean          has_mask   = mask != NULL;
 
-  switch (layer_mode->real_composite_mode)
+  switch (layer_mode->composite_mode)
     {
     case GIMP_LAYER_COMPOSITE_UNION:
     case GIMP_LAYER_COMPOSITE_AUTO:
@@ -282,7 +345,7 @@ gimp_operation_replace_get_affected_region (GimpOperationLayerMode *layer_mode)
 {
   GimpLayerCompositeRegion affected_region = GIMP_LAYER_COMPOSITE_REGION_INTERSECTION;
 
-  if (layer_mode->opacity != 0.0)
+  if (layer_mode->prop_opacity != 0.0)
     affected_region |= GIMP_LAYER_COMPOSITE_REGION_DESTINATION;
 
   /* if opacity != 1.0, or we have a mask, then we also affect SOURCE, but this
