@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __PAINT_TYPES_H__
@@ -36,7 +36,7 @@ typedef struct _GimpDodgeBurn        GimpDodgeBurn;
 typedef struct _GimpEraser           GimpEraser;
 typedef struct _GimpHeal             GimpHeal;
 typedef struct _GimpInk              GimpInk;
-typedef struct _GimpMybrush          GimpMybrush;
+typedef struct _GimpMybrushCore      GimpMybrushCore;
 typedef struct _GimpPaintbrush       GimpPaintbrush;
 typedef struct _GimpPencil           GimpPencil;
 typedef struct _GimpPerspectiveClone GimpPerspectiveClone;
@@ -60,12 +60,6 @@ typedef struct _GimpPerspectiveCloneOptions GimpPerspectiveCloneOptions;
 typedef struct _GimpSmudgeOptions           GimpSmudgeOptions;
 
 
-/*  paint undos  */
-
-typedef struct _GimpPaintCoreUndo GimpPaintCoreUndo;
-typedef struct _GimpInkUndo       GimpInkUndo;
-
-
 /*  functions  */
 
 typedef void (* GimpPaintRegisterCallback) (Gimp        *gimp,
