@@ -15,7 +15,7 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Less General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /* This is a simple interpreter for the GUM language (the GEGL UI Meta
@@ -882,7 +882,7 @@ gimp_prop_eval_parse_reference (GObject      *config,
 
   if (*error)
     {
-      g_free (ref_key);
+      g_free (*ref_key);
 
       return FALSE;
     }
