@@ -15,13 +15,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
+#include <cairo.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "text-types.h"
@@ -195,8 +196,7 @@ gimp_text_layer_from_layer (GimpLayer *layer,
 
   drawable = GIMP_DRAWABLE (text_layer);
 
-  drawable->private->buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (layer));
-  GIMP_DRAWABLE (layer)->private->buffer = NULL;
+  gimp_drawable_steal_buffer (drawable, GIMP_DRAWABLE (layer));
 
   gimp_layer_set_opacity         (GIMP_LAYER (text_layer),
                                   gimp_layer_get_opacity (layer), FALSE);
