@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef  __GIMP_COLOR_TOOL_H__
@@ -36,16 +36,18 @@ typedef struct _GimpColorToolClass GimpColorToolClass;
 
 struct _GimpColorTool
 {
-  GimpDrawTool       parent_instance;
+  GimpDrawTool         parent_instance;
 
-  gboolean           enabled;
-  gint               center_x;
-  gint               center_y;
-  GimpColorPickMode  pick_mode;
+  gboolean             enabled;
+  GimpColorOptions    *options;
+  gboolean             saved_snap_to;
 
-  GimpColorOptions  *options;
+  GimpColorPickTarget  pick_target;
 
-  GimpSamplePoint   *sample_point;
+  gboolean             can_pick;
+  gint                 center_x;
+  gint                 center_y;
+  GimpSamplePoint     *sample_point;
 };
 
 struct _GimpColorToolClass
@@ -53,21 +55,24 @@ struct _GimpColorToolClass
   GimpDrawToolClass  parent_class;
 
   /*  virtual functions  */
-  gboolean (* pick)   (GimpColorTool      *tool,
-                       gint                x,
-                       gint                y,
-                       const Babl        **sample_format,
-                       gpointer            pixel,
-                       GimpRGB            *color);
+  gboolean (* can_pick) (GimpColorTool      *tool,
+                         const GimpCoords   *coords,
+                         GimpDisplay        *display);
+  gboolean (* pick)     (GimpColorTool      *tool,
+                         const GimpCoords   *coords,
+                         GimpDisplay        *display,
+                         const Babl        **sample_format,
+                         gpointer            pixel,
+                         GimpRGB            *color);
 
   /*  signals  */
-  void     (* picked) (GimpColorTool      *tool,
-                       GimpColorPickState  pick_state,
-                       gdouble             x,
-                       gdouble             y,
-                       const Babl         *sample_format,
-                       gpointer            pixel,
-                       const GimpRGB      *color);
+  void     (* picked)   (GimpColorTool      *tool,
+                         const GimpCoords   *coords,
+                         GimpDisplay        *display,
+                         GimpColorPickState  pick_state,
+                         const Babl         *sample_format,
+                         gpointer            pixel,
+                         const GimpRGB      *color);
 };
 
 
