@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -148,6 +148,7 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:alien-map",
     "gegl:antialias",
     "gegl:apply-lens",
+    "gegl:bayer-matrix",
     "gegl:bump-map",
     "gegl:c2g",
     "gegl:cartoon",
@@ -191,8 +192,11 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:invert-gamma",
     "gegl:lens-distortion",
     "gegl:lens-flare",
+    "gegl:linear-sinusoid",
+    "gegl:long-shadow",
     "gegl:mantiuk06",
     "gegl:maze",
+    "gegl:mean-curvature-blur",
     "gegl:median-blur",
     "gegl:mirrors",
     "gegl:mono-mixer",
@@ -200,6 +204,7 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:motion-blur-circular",
     "gegl:motion-blur-linear",
     "gegl:motion-blur-zoom",
+    "gegl:newsprint",
     "gegl:noise-cie-lch",
     "gegl:noise-hsv",
     "gegl:noise-hurl",
@@ -209,6 +214,7 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:noise-slur",
     "gegl:noise-solid",
     "gegl:noise-spread",
+    "gegl:normal-map",
     "gegl:oilify",
     "gegl:panorama-projection",
     "gegl:perlin-noise",
@@ -232,6 +238,7 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:softglow",
     "gegl:spherize",
     "gegl:spiral",
+    "gegl:stereographic-projection",
     "gegl:stretch-contrast",
     "gegl:stretch-contrast-hsv",
     "gegl:stress",
