@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -70,46 +70,51 @@ struct _AuxInput
 
 /*  local function prototypes  */
 
-static void        gimp_operation_tool_finalize        (GObject           *object);
+static void        gimp_operation_tool_finalize                  (GObject           *object);
 
-static gboolean    gimp_operation_tool_initialize      (GimpTool          *tool,
-                                                        GimpDisplay       *display,
-                                                        GError           **error);
-static void        gimp_operation_tool_control         (GimpTool          *tool,
-                                                        GimpToolAction     action,
-                                                        GimpDisplay       *display);
-static void        gimp_operation_tool_options_notify  (GimpTool          *tool,
-                                                        GimpToolOptions   *options,
-                                                        const GParamSpec  *pspec);
+static gboolean    gimp_operation_tool_initialize                (GimpTool          *tool,
+                                                                  GimpDisplay       *display,
+                                                                  GError           **error);
+static void        gimp_operation_tool_control                   (GimpTool          *tool,
+                                                                  GimpToolAction     action,
+                                                                  GimpDisplay       *display);
 
-static gchar     * gimp_operation_tool_get_operation   (GimpFilterTool    *filter_tool,
-                                                        gchar            **description);
-static void        gimp_operation_tool_dialog          (GimpFilterTool    *filter_tool);
-static void        gimp_operation_tool_reset           (GimpFilterTool    *filter_tool);
-static void        gimp_operation_tool_set_config      (GimpFilterTool    *filter_tool,
-                                                        GimpConfig        *config);
-static void        gimp_operation_tool_color_picked    (GimpFilterTool    *filter_tool,
-                                                        gpointer           identifier,
-                                                        gdouble            x,
-                                                        gdouble            y,
-                                                        const Babl        *sample_format,
-                                                        const GimpRGB     *color);
+static gchar     * gimp_operation_tool_get_operation             (GimpFilterTool    *filter_tool,
+                                                                  gchar            **description);
+static void        gimp_operation_tool_dialog                    (GimpFilterTool    *filter_tool);
+static void        gimp_operation_tool_reset                     (GimpFilterTool    *filter_tool);
+static void        gimp_operation_tool_set_config                (GimpFilterTool    *filter_tool,
+                                                                  GimpConfig        *config);
+static void        gimp_operation_tool_region_changed            (GimpFilterTool    *filter_tool);
+static void        gimp_operation_tool_color_picked              (GimpFilterTool    *filter_tool,
+                                                                  gpointer           identifier,
+                                                                  gdouble            x,
+                                                                  gdouble            y,
+                                                                  const Babl        *sample_format,
+                                                                  const GimpRGB     *color);
 
-static void        gimp_operation_tool_halt            (GimpOperationTool *op_tool);
-static void        gimp_operation_tool_commit          (GimpOperationTool *op_tool);
+static void        gimp_operation_tool_options_box_size_allocate (GtkWidget         *options_box,
+                                                                  GdkRectangle      *allocation,
+                                                                  GimpOperationTool *tool);
 
-static void        gimp_operation_tool_sync_op         (GimpOperationTool *op_tool,
-                                                        gboolean           sync_colors);
-static void        gimp_operation_tool_create_gui      (GimpOperationTool *tool);
-static void        gimp_operation_tool_add_gui         (GimpOperationTool *tool);
+static void        gimp_operation_tool_halt                      (GimpOperationTool *op_tool);
+static void        gimp_operation_tool_commit                    (GimpOperationTool *op_tool);
 
-static AuxInput *  gimp_operation_tool_aux_input_new   (GimpOperationTool *tool,
-                                                        GeglNode          *operation,
-                                                        const gchar       *input_pad,
-                                                        const gchar       *label);
-static void        gimp_operation_tool_aux_input_detach(AuxInput          *input);
-static void        gimp_operation_tool_aux_input_clear (AuxInput          *input);
-static void        gimp_operation_tool_aux_input_free  (AuxInput          *input);
+static void        gimp_operation_tool_sync_op                   (GimpOperationTool *op_tool,
+                                                                  gboolean           sync_colors);
+static void        gimp_operation_tool_create_gui                (GimpOperationTool *tool);
+static void        gimp_operation_tool_add_gui                   (GimpOperationTool *tool);
+
+static AuxInput *  gimp_operation_tool_aux_input_new             (GimpOperationTool *tool,
+                                                                  GeglNode          *operation,
+                                                                  const gchar       *input_pad,
+                                                                  const gchar       *label);
+static void        gimp_operation_tool_aux_input_detach          (AuxInput          *input);
+static void        gimp_operation_tool_aux_input_clear           (AuxInput          *input);
+static void        gimp_operation_tool_aux_input_free            (AuxInput          *input);
+
+static void        gimp_operation_tool_unlink_chains             (GimpOperationTool *op_tool);
+static void        gimp_operation_tool_relink_chains             (GimpOperationTool *op_tool);
 
 
 G_DEFINE_TYPE (GimpOperationTool, gimp_operation_tool,
@@ -130,7 +135,7 @@ gimp_operation_tool_register (GimpToolRegisterCallback  callback,
                 "gimp-operation-tool",
                 _("GEGL Operation"),
                 _("Operation Tool: Use an arbitrary GEGL operation"),
-                N_("_GEGL Operation..."), NULL,
+                NULL, NULL,
                 NULL, GIMP_HELP_TOOL_GEGL,
                 GIMP_ICON_GEGL,
                 data);
@@ -143,17 +148,17 @@ gimp_operation_tool_class_init (GimpOperationToolClass *klass)
   GimpToolClass       *tool_class        = GIMP_TOOL_CLASS (klass);
   GimpFilterToolClass *filter_tool_class = GIMP_FILTER_TOOL_CLASS (klass);
 
-  object_class->finalize           = gimp_operation_tool_finalize;
+  object_class->finalize            = gimp_operation_tool_finalize;
 
-  tool_class->initialize           = gimp_operation_tool_initialize;
-  tool_class->control              = gimp_operation_tool_control;
-  tool_class->options_notify       = gimp_operation_tool_options_notify;
+  tool_class->initialize            = gimp_operation_tool_initialize;
+  tool_class->control               = gimp_operation_tool_control;
 
-  filter_tool_class->get_operation = gimp_operation_tool_get_operation;
-  filter_tool_class->dialog        = gimp_operation_tool_dialog;
-  filter_tool_class->reset         = gimp_operation_tool_reset;
-  filter_tool_class->set_config    = gimp_operation_tool_set_config;
-  filter_tool_class->color_picked  = gimp_operation_tool_color_picked;
+  filter_tool_class->get_operation  = gimp_operation_tool_get_operation;
+  filter_tool_class->dialog         = gimp_operation_tool_dialog;
+  filter_tool_class->reset          = gimp_operation_tool_reset;
+  filter_tool_class->set_config     = gimp_operation_tool_set_config;
+  filter_tool_class->region_changed = gimp_operation_tool_region_changed;
+  filter_tool_class->color_picked   = gimp_operation_tool_color_picked;
 }
 
 static void
@@ -234,35 +239,6 @@ gimp_operation_tool_control (GimpTool       *tool,
   GIMP_TOOL_CLASS (parent_class)->control (tool, action, display);
 }
 
-static void
-gimp_operation_tool_options_notify (GimpTool         *tool,
-                                    GimpToolOptions  *options,
-                                    const GParamSpec *pspec)
-{
-  GimpOperationTool *op_tool = GIMP_OPERATION_TOOL (tool);
-
-  GIMP_TOOL_CLASS (parent_class)->options_notify (tool, options, pspec);
-
-  if (! strcmp (pspec->name, "region"))
-    {
-      GimpFilterTool *filter_tool = GIMP_FILTER_TOOL (tool);
-
-      /* when the region changes, do we want the operation's on-canvas
-       * controller to move to a new position, or the operation to
-       * change its properties to match the on-canvas controller?
-       *
-       * decided to leave the on-canvas controler where it is and
-       * pretend it has changed, so the operation is updated
-       * accordingly...
-       */
-      if (filter_tool->widget)
-        g_signal_emit_by_name (filter_tool->widget, "changed");
-
-      if (filter_tool->config && tool->drawable)
-        gimp_operation_tool_sync_op (op_tool, FALSE);
-    }
-}
-
 static gchar *
 gimp_operation_tool_get_operation (GimpFilterTool  *filter_tool,
                                    gchar          **description)
@@ -279,18 +255,39 @@ gimp_operation_tool_dialog (GimpFilterTool *filter_tool)
 {
   GimpOperationTool *op_tool = GIMP_OPERATION_TOOL (filter_tool);
   GtkWidget         *main_vbox;
+  GtkWidget         *options_sw;
   GtkWidget         *options_gui;
   GtkWidget         *options_box;
+  GtkWidget         *viewport;
 
   main_vbox = gimp_filter_tool_dialog_get_vbox (filter_tool);
 
+  /*  The options scrolled window  */
+  options_sw = gtk_scrolled_window_new (NULL, NULL);
+  g_weak_ref_set (&op_tool->options_sw_ref, options_sw);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (options_sw),
+                                       GTK_SHADOW_NONE);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (options_sw),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_box_pack_start (GTK_BOX (main_vbox), options_sw,
+                      TRUE, TRUE, 0);
+  gtk_widget_show (options_sw);
+
+  viewport = gtk_viewport_new (NULL, NULL);
+  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
+  gtk_container_add (GTK_CONTAINER (options_sw), viewport);
+  gtk_widget_show (viewport);
+
   /*  The options vbox  */
   options_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
   g_weak_ref_set (&op_tool->options_box_ref, options_box);
-  gtk_box_pack_start (GTK_BOX (main_vbox), options_box,
-                      TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (viewport), options_box);
   gtk_widget_show (options_box);
 
+  g_signal_connect (options_box, "size-allocate",
+                    G_CALLBACK (gimp_operation_tool_options_box_size_allocate),
+                    op_tool);
+
   options_gui = g_weak_ref_get (&op_tool->options_gui_ref);
   if (options_gui)
     {
@@ -304,10 +301,14 @@ gimp_operation_tool_reset (GimpFilterTool *filter_tool)
 {
   GimpOperationTool *op_tool = GIMP_OPERATION_TOOL (filter_tool);
 
+  gimp_operation_tool_unlink_chains (op_tool);
+
   GIMP_FILTER_TOOL_CLASS (parent_class)->reset (filter_tool);
 
   if (filter_tool->config && GIMP_TOOL (op_tool)->drawable)
     gimp_operation_tool_sync_op (op_tool, TRUE);
+
+  gimp_operation_tool_relink_chains (op_tool);
 }
 
 static void
@@ -316,10 +317,33 @@ gimp_operation_tool_set_config (GimpFilterTool *filter_tool,
 {
   GimpOperationTool *op_tool = GIMP_OPERATION_TOOL (filter_tool);
 
+  gimp_operation_tool_unlink_chains (op_tool);
+
   GIMP_FILTER_TOOL_CLASS (parent_class)->set_config (filter_tool, config);
 
   if (filter_tool->config && GIMP_TOOL (op_tool)->drawable)
     gimp_operation_tool_sync_op (op_tool, FALSE);
+
+  gimp_operation_tool_relink_chains (op_tool);
+}
+
+static void
+gimp_operation_tool_region_changed (GimpFilterTool *filter_tool)
+{
+  GimpOperationTool *op_tool = GIMP_OPERATION_TOOL (filter_tool);
+
+  /* when the region changes, do we want the operation's on-canvas
+   * controller to move to a new position, or the operation to
+   * change its properties to match the on-canvas controller?
+   *
+   * decided to leave the on-canvas controller where it is and
+   * pretend it has changed, so the operation is updated
+   * accordingly...
+   */
+  if (filter_tool->widget)
+    g_signal_emit_by_name (filter_tool->widget, "changed");
+
+  gimp_operation_tool_sync_op (op_tool, FALSE);
 }
 
 static void
@@ -412,6 +436,54 @@ gimp_operation_tool_color_picked (GimpFilterTool  *filter_tool,
   g_strfreev (pspecs);
 }
 
+static void
+gimp_operation_tool_options_box_size_allocate (GtkWidget         *options_box,
+                                               GdkRectangle      *allocation,
+                                               GimpOperationTool *op_tool)
+{
+  GimpTool       *tool = GIMP_TOOL (op_tool);
+  GtkWidget      *shell;
+  GtkWidget      *options_sw;
+  GdkRectangle    workarea;
+  GtkRequisition  minimum;
+  gint            maximum_height;
+
+  shell      = GTK_WIDGET (gimp_display_get_shell (tool->display));
+  options_sw = g_weak_ref_get (&op_tool->options_sw_ref);
+
+  g_return_if_fail (options_sw != NULL);
+
+  gdk_screen_get_monitor_workarea (gtk_widget_get_screen (shell),
+                                   gimp_widget_get_monitor (shell), &workarea);
+
+  maximum_height = workarea.height / 2;
+
+  gtk_widget_size_request (options_box, &minimum);
+
+  if (minimum.height > maximum_height)
+    {
+      GtkWidget *scrollbar;
+
+      minimum.height = maximum_height;
+
+      scrollbar = gtk_scrolled_window_get_vscrollbar (
+        GTK_SCROLLED_WINDOW (options_sw));
+
+      if (scrollbar)
+        {
+          GtkRequisition req;
+
+          gtk_widget_size_request (scrollbar, &req);
+
+          minimum.width += req.width;
+        }
+    }
+
+  gtk_widget_set_size_request (options_sw, minimum.width, minimum.height);
+
+  g_object_unref (options_sw);
+}
+
 static void
 gimp_operation_tool_halt (GimpOperationTool *op_tool)
 {
@@ -685,6 +757,88 @@ gimp_operation_tool_aux_input_free (AuxInput *input)
   g_slice_free (AuxInput, input);
 }
 
+static void
+gimp_operation_tool_unlink_chains (GimpOperationTool *op_tool)
+{
+  GObject *options_gui = g_weak_ref_get (&op_tool->options_gui_ref);
+  GList   *chains;
+
+  g_return_if_fail (options_gui != NULL);
+
+  chains = g_object_get_data (options_gui, "chains");
+
+  while (chains)
+    {
+      GimpChainButton *chain = chains->data;
+      gboolean         active;
+
+      active = gimp_chain_button_get_active (chain);
+
+      g_object_set_data (G_OBJECT (chain), "was-active",
+                         GINT_TO_POINTER (active));
+
+      if (active)
+        {
+          gimp_chain_button_set_active (chain, FALSE);
+        }
+
+      chains = chains->next;
+    }
+
+  g_object_unref (options_gui);
+}
+
+static void
+gimp_operation_tool_relink_chains (GimpOperationTool *op_tool)
+{
+  GimpFilterTool *filter_tool = GIMP_FILTER_TOOL (op_tool);
+  GObject        *options_gui = g_weak_ref_get (&op_tool->options_gui_ref);
+  GList          *chains;
+
+  g_return_if_fail (options_gui != NULL);
+
+  chains = g_object_get_data (options_gui, "chains");
+
+  while (chains)
+    {
+      GimpChainButton *chain = chains->data;
+
+      if (g_object_get_data (G_OBJECT (chain), "was-active"))
+        {
+          const gchar *name_x    = g_object_get_data (chains->data, "x-property");
+          const gchar *name_y    = g_object_get_data (chains->data, "y-property");
+          const gchar *names[2]  = { name_x, name_y };
+          GValue       values[2] = { G_VALUE_INIT, G_VALUE_INIT };
+          GValue       double_x  = G_VALUE_INIT;
+          GValue       double_y  = G_VALUE_INIT;
+
+          g_object_getv (filter_tool->config, 2, names, values);
+
+          g_value_init (&double_x, G_TYPE_DOUBLE);
+          g_value_init (&double_y, G_TYPE_DOUBLE);
+
+          if (g_value_transform (&values[0], &double_x) &&
+              g_value_transform (&values[1], &double_y) &&
+              g_value_get_double (&double_x) ==
+              g_value_get_double (&double_y))
+            {
+              gimp_chain_button_set_active (chain, TRUE);
+            }
+
+          g_value_unset (&double_x);
+          g_value_unset (&double_y);
+          g_value_unset (&values[0]);
+          g_value_unset (&values[1]);
+
+          g_object_set_data (G_OBJECT (chain), "was-active", NULL);
+        }
+
+      chains = chains->next;
+    }
+
+  g_object_unref (options_gui);
+}
+
 
 /*  public functions  */
 
