@@ -502,7 +502,7 @@ gimp_text_tool_button_press (GimpTool            *tool,
         }
     }
 
-  if (gimp_image_coords_in_active_pickable (image, coords, FALSE, FALSE))
+  if (gimp_image_coords_in_active_pickable (image, coords, FALSE, FALSE, FALSE))
     {
       GimpDrawable *drawable = gimp_image_get_active_drawable (image);
       GimpItem     *item     = GIMP_ITEM (drawable);
@@ -837,8 +837,8 @@ gimp_text_tool_get_popup (GimpTool         *tool,
                                            "<TextTool>",
                                            text_tool, FALSE);
 
-          im_menu = gtk_ui_manager_get_widget (GTK_UI_MANAGER (text_tool->ui_manager),
-                                               "/text-tool-popup/text-tool-input-methods-menu");
+          im_menu = gimp_ui_manager_get_widget (text_tool->ui_manager,
+                                                "/text-tool-popup/text-tool-input-methods-menu");
 
           if (GTK_IS_MENU_ITEM (im_menu))
             im_menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (im_menu));
