@@ -49,6 +49,9 @@ enum
   PROP_DIRECTION,
   PROP_DIRECTION_LINKED,
   PROP_SHOW_PREVIEW,
+  PROP_COMPOSITED_PREVIEW,
+  PROP_PREVIEW_LINKED,
+  PROP_SYNCHRONOUS_PREVIEW,
   PROP_PREVIEW_OPACITY,
   PROP_GRID_TYPE,
   PROP_GRID_SIZE,
@@ -110,6 +113,27 @@ gimp_transform_grid_options_class_init (GimpTransformGridOptionsClass *klass)
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_COMPOSITED_PREVIEW,
+                            "composited-preview",
+                            _("Composited preview"),
+                            _("Show preview as part of the image composition"),
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PREVIEW_LINKED,
+                            "preview-linked",
+                            _("Preview linked items"),
+                            _("Include linked items in the preview"),
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SYNCHRONOUS_PREVIEW,
+                            "synchronous-preview",
+                            _("Synchronous preview"),
+                            _("Render the preview synchronously"),
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_PREVIEW_OPACITY,
                            "preview-opacity",
                            _("Image opacity"),
@@ -227,6 +251,15 @@ gimp_transform_grid_options_set_property (GObject      *object,
     case PROP_SHOW_PREVIEW:
       options->show_preview = g_value_get_boolean (value);
       break;
+    case PROP_COMPOSITED_PREVIEW:
+      options->composited_preview = g_value_get_boolean (value);
+      break;
+    case PROP_PREVIEW_LINKED:
+      options->preview_linked = g_value_get_boolean (value);
+      break;
+    case PROP_SYNCHRONOUS_PREVIEW:
+      options->synchronous_preview = g_value_get_boolean (value);
+      break;
     case PROP_PREVIEW_OPACITY:
       options->preview_opacity = g_value_get_double (value);
       break;
@@ -292,6 +325,15 @@ gimp_transform_grid_options_get_property (GObject    *object,
     case PROP_SHOW_PREVIEW:
       g_value_set_boolean (value, options->show_preview);
       break;
+    case PROP_COMPOSITED_PREVIEW:
+      g_value_set_boolean (value, options->composited_preview);
+      break;
+    case PROP_PREVIEW_LINKED:
+      g_value_set_boolean (value, options->preview_linked);
+      break;
+    case PROP_SYNCHRONOUS_PREVIEW:
+      g_value_set_boolean (value, options->synchronous_preview);
+      break;
     case PROP_PREVIEW_OPACITY:
       g_value_set_double (value, options->preview_opacity);
       break;
@@ -351,6 +393,9 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
   GObject                    *config = G_OBJECT (tool_options);
   GimpTransformGridToolClass *tg_class;
   GtkWidget                  *vbox;
+  GtkWidget                  *vbox2;
+  GtkWidget                  *vbox3;
+  GtkWidget                  *button;
   GtkWidget                  *frame;
   GtkWidget                  *combo;
   GtkWidget                  *scale;
@@ -366,9 +411,7 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
   if (tg_class->matrix_to_info)
     {
       GimpTransformOptions *tr_options = GIMP_TRANSFORM_OPTIONS (tool_options);
-      GtkWidget            *vbox2;
       GtkWidget            *hbox;
-      GtkWidget            *button;
 
       vbox2 = gtk_bin_get_child (GTK_BIN (tr_options->direction_frame));
       g_object_ref (vbox2);
@@ -396,11 +439,36 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
   g_type_class_unref (tg_class);
 
   /*  the preview frame  */
+  vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+
+  vbox3 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+
+  button = gimp_prop_check_button_new (config, "preview-linked", NULL);
+  gtk_box_pack_start (GTK_BOX (vbox3), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  button = gimp_prop_check_button_new (config, "synchronous-preview", NULL);
+  gtk_box_pack_start (GTK_BOX (vbox3), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  frame = gimp_prop_expanding_frame_new (config, "composited-preview", NULL,
+                                         vbox3, NULL);
+  gtk_box_pack_start (GTK_BOX (vbox2), frame, FALSE, FALSE, 0);
+  gtk_widget_show (frame);
+
   scale = gimp_prop_spin_scale_new (config, "preview-opacity", NULL,
                                     0.01, 0.1, 0);
   gimp_prop_widget_set_factor (scale, 100.0, 0.0, 0.0, 1);
+  gtk_box_pack_start (GTK_BOX (vbox2), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
+
+  g_object_bind_property (config, "composited-preview",
+                          scale,  "sensitive",
+                          G_BINDING_SYNC_CREATE |
+                          G_BINDING_INVERT_BOOLEAN);
+
   frame = gimp_prop_expanding_frame_new (config, "show-preview", NULL,
-                                         scale, NULL);
+                                         vbox2, NULL);
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
@@ -431,8 +499,7 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
 
   if (tool_options->tool_info->tool_type == GIMP_TYPE_ROTATE_TOOL)
     {
-      GtkWidget *button;
-      gchar     *label;
+      gchar *label;
 
       label = g_strdup_printf (_("15 degrees (%s)"),
                                gimp_get_mod_string (extend_mask));
@@ -448,8 +515,7 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
     }
   else if (tool_options->tool_info->tool_type == GIMP_TYPE_SCALE_TOOL)
     {
-      GtkWidget *button;
-      gchar     *label;
+      gchar *label;
 
       label = g_strdup_printf (_("Keep aspect (%s)"),
                                gimp_get_mod_string (extend_mask));
@@ -477,8 +543,7 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
     }
   else if (tool_options->tool_info->tool_type == GIMP_TYPE_PERSPECTIVE_TOOL)
     {
-      GtkWidget *button;
-      gchar     *label;
+      gchar *label;
 
       label = g_strdup_printf (_("Constrain handles (%s)"),
                                gimp_get_mod_string (extend_mask));
@@ -546,9 +611,8 @@ gimp_transform_grid_options_gui (GimpToolOptions *tool_options)
           N_("Lock pivot position to canvas") },
       };
 
-      GtkWidget *button;
-      gchar     *label;
-      gint       i;
+      gchar *label;
+      gint   i;
 
       frame = NULL;
 
