@@ -26,10 +26,21 @@
 
 #include "tools-types.h"
 
+#include "gegl/gimpapplicator.h"
+#include "gegl/gimp-gegl-nodes.h"
+#include "gegl/gimp-gegl-utils.h"
+
 #include "core/gimp.h"
+#include "core/gimp-transform-resize.h"
+#include "core/gimp-transform-utils.h"
 #include "core/gimpboundary.h"
+#include "core/gimpcontainer.h"
+#include "core/gimpdrawablefilter.h"
 #include "core/gimperror.h"
+#include "core/gimpfilter.h"
+#include "core/gimpgrouplayer.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-item-list.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
 #include "core/gimplayer.h"
@@ -37,6 +48,7 @@
 #include "core/gimppickable.h"
 #include "core/gimpprojection.h"
 #include "core/gimptoolinfo.h"
+#include "core/gimpviewable.h"
 
 #include "vectors/gimpvectors.h"
 #include "vectors/gimpstroke.h"
@@ -61,12 +73,37 @@
 #define EPSILON 1e-6
 
 
-#define RESPONSE_RESET    1
-#define RESPONSE_READJUST 2
+#define RESPONSE_RESET     1
+#define RESPONSE_READJUST  2
+
+#define UNDO_COMPRESS_TIME (0.5 * G_TIME_SPAN_SECOND)
 
 
 typedef struct
 {
+  GimpTransformGridTool *tg_tool;
+
+  GimpDrawable          *drawable;
+  GimpDrawableFilter    *filter;
+
+  GimpDrawable          *root_drawable;
+
+  GeglNode              *transform_node;
+  GeglNode              *crop_node;
+
+  GimpMatrix3            transform;
+  GeglRectangle          bounds;
+} Filter;
+
+typedef struct
+{
+  GimpTransformGridTool *tg_tool;
+  GimpDrawable          *root_drawable;
+} AddFilterData;
+
+typedef struct
+{
+  gint64                 time;
   GimpTransformDirection direction;
   TransInfo              trans_infos[2];
 } UndoInfo;
@@ -151,6 +188,13 @@ static void      gimp_transform_grid_tool_widget_response    (GimpToolWidget
                                                               gint                    response_id,
                                                               GimpTransformGridTool  *tg_tool);
 
+static void      gimp_transform_grid_tool_filter_flush       (GimpDrawableFilter     *filter,
+                                                              GimpTransformGridTool  *tg_tool);
+
+static void      gimp_transform_grid_tool_image_linked_items_changed
+                                                             (GimpImage              *image,
+                                                              GimpTransformGridTool  *tg_tool);
+
 static void      gimp_transform_grid_tool_halt               (GimpTransformGridTool  *tg_tool);
 static void      gimp_transform_grid_tool_commit             (GimpTransformGridTool  *tg_tool);
 
@@ -165,12 +209,29 @@ static void      gimp_transform_grid_tool_response           (GimpToolGui
                                                               gint                    response_id,
                                                               GimpTransformGridTool  *tg_tool);
 
+static gboolean  gimp_transform_grid_tool_composited_preview (GimpTransformGridTool  *tg_tool);
 static void      gimp_transform_grid_tool_update_sensitivity (GimpTransformGridTool  *tg_tool);
 static void      gimp_transform_grid_tool_update_preview     (GimpTransformGridTool  *tg_tool);
+static void      gimp_transform_grid_tool_update_filters     (GimpTransformGridTool  *tg_tool);
 static void      gimp_transform_grid_tool_hide_active_object (GimpTransformGridTool  *tg_tool,
                                                               GimpObject             *object);
 static void      gimp_transform_grid_tool_show_active_object (GimpTransformGridTool  *tg_tool);
 
+static void      gimp_transform_grid_tool_add_filter         (GimpDrawable           *drawable,
+                                                              AddFilterData          *data);
+static void      gimp_transform_grid_tool_remove_filter      (GimpDrawable           *drawable,
+                                                              GimpTransformGridTool  *tg_tool);
+
+static void      gimp_transform_grid_tool_effective_mode_changed
+                                                             (GimpLayer              *layer,
+                                                              GimpTransformGridTool  *tg_tool);
+
+static Filter  * filter_new                                  (GimpTransformGridTool  *tg_tool,
+                                                              GimpDrawable           *drawable,
+                                                              GimpDrawable           *root_drawable,
+                                                              gboolean                add_filter);
+static void      filter_free                                 (Filter                 *filter);
+
 static UndoInfo * undo_info_new  (void);
 static void       undo_info_free (UndoInfo *info);
 
@@ -238,9 +299,10 @@ gimp_transform_grid_tool_init (GimpTransformGridTool *tg_tool)
   gimp_tool_control_set_scroll_lock      (tool->control, TRUE);
   gimp_tool_control_set_preserve         (tool->control, FALSE);
   gimp_tool_control_set_dirty_mask       (tool->control,
-                                          GIMP_DIRTY_IMAGE_SIZE |
-                                          GIMP_DIRTY_DRAWABLE   |
-                                          GIMP_DIRTY_SELECTION  |
+                                          GIMP_DIRTY_IMAGE_SIZE      |
+                                          GIMP_DIRTY_IMAGE_STRUCTURE |
+                                          GIMP_DIRTY_DRAWABLE        |
+                                          GIMP_DIRTY_SELECTION       |
                                           GIMP_DIRTY_ACTIVE_DRAWABLE);
   gimp_tool_control_set_active_modifiers (tool->control,
                                           GIMP_TOOL_ACTIVE_MODIFIERS_SAME);
@@ -287,6 +349,9 @@ gimp_transform_grid_tool_initialize (GimpTool     *tool,
 
   tr_tool->object = object;
 
+  if (GIMP_IS_DRAWABLE (object))
+    gimp_viewable_preview_freeze (GIMP_VIEWABLE (object));
+
   /*  Initialize the transform_grid tool dialog  */
   if (! tg_tool->gui)
     gimp_transform_grid_tool_dialog (tg_tool);
@@ -319,6 +384,11 @@ gimp_transform_grid_tool_initialize (GimpTool     *tool,
   memcpy (undo_info->trans_infos, tg_tool->trans_infos,
           sizeof (tg_tool->trans_infos));
 
+  g_signal_connect (
+    image, "linked-items-changed",
+    G_CALLBACK (gimp_transform_grid_tool_image_linked_items_changed),
+    tg_tool);
+
   return TRUE;
 }
 
@@ -399,7 +469,7 @@ gimp_transform_grid_tool_button_release (GimpTool              *tool,
   if (release_type != GIMP_BUTTON_RELEASE_CANCEL)
     {
       /* We're done with an interaction, save it on the undo list */
-      gimp_transform_grid_tool_push_internal_undo (tg_tool);
+      gimp_transform_grid_tool_push_internal_undo (tg_tool, FALSE);
     }
   else
     {
@@ -611,31 +681,45 @@ gimp_transform_grid_tool_options_notify (GimpTool         *tool,
       /*  recalculate the tool's transformation matrix  */
       gimp_transform_tool_recalc_matrix (tr_tool, tool->display);
     }
-  else if (! strcmp (pspec->name, "show-preview"))
+  else if (! strcmp (pspec->name, "show-preview") ||
+           ! strcmp (pspec->name, "composited-preview"))
     {
       if (tg_tool->preview)
         {
           GimpDisplay *display;
           GimpObject  *object;
-          gboolean     show_preview;
-
-          show_preview = gimp_transform_grid_options_show_preview (tg_options) &&
-                         tr_tool->transform_valid;
-
-          gimp_canvas_item_set_visible (tg_tool->preview, show_preview);
 
           display = tool->display;
           object  = gimp_transform_tool_get_active_object (tr_tool, display);
 
           if (object)
             {
-              if (gimp_transform_grid_options_show_preview (tg_options))
-                gimp_transform_grid_tool_hide_active_object (tg_tool, object);
+              if (tg_options->show_preview &&
+                  ! gimp_transform_grid_tool_composited_preview (tg_tool))
+                {
+                  gimp_transform_grid_tool_hide_active_object (tg_tool, object);
+                }
               else
-                gimp_transform_grid_tool_show_active_object (tg_tool);
+                {
+                  gimp_transform_grid_tool_show_active_object (tg_tool);
+                }
             }
+
+          gimp_transform_grid_tool_update_preview (tg_tool);
         }
     }
+  else if (! strcmp (pspec->name, "preview-linked") &&
+           tg_tool->filters)
+    {
+      gimp_transform_grid_tool_update_filters (tg_tool);
+      gimp_transform_grid_tool_update_preview (tg_tool);
+    }
+  else if (! strcmp (pspec->name, "interpolation") ||
+           ! strcmp (pspec->name, "clip")          ||
+           ! strcmp (pspec->name, "preview-opacity"))
+    {
+      gimp_transform_grid_tool_update_preview (tg_tool);
+    }
   else if (g_str_has_prefix (pspec->name, "constrain-") ||
            g_str_has_prefix (pspec->name, "frompivot-") ||
            ! strcmp (pspec->name, "fixedpivot") ||
@@ -661,10 +745,10 @@ gimp_transform_grid_tool_draw (GimpDrawTool *draw_tool)
   if (tr_options->direction == GIMP_TRANSFORM_BACKWARD)
     gimp_matrix3_invert (&matrix);
 
-  if (tg_tool->widget)
+  if (tr_options->type == GIMP_TRANSFORM_TYPE_LAYER ||
+      tr_options->type == GIMP_TRANSFORM_TYPE_IMAGE)
     {
       GimpPickable *pickable;
-      gboolean      show_preview;
 
       if (tr_options->type == GIMP_TRANSFORM_TYPE_IMAGE)
         {
@@ -678,9 +762,6 @@ gimp_transform_grid_tool_draw (GimpDrawTool *draw_tool)
           pickable = GIMP_PICKABLE (tool->drawable);
         }
 
-      show_preview = gimp_transform_grid_options_show_preview (options) &&
-                     tr_tool->transform_valid;
-
       tg_tool->preview =
         gimp_draw_tool_add_transform_preview (draw_tool,
                                               pickable,
@@ -691,15 +772,6 @@ gimp_transform_grid_tool_draw (GimpDrawTool *draw_tool)
                                               tr_tool->y2);
       g_object_add_weak_pointer (G_OBJECT (tg_tool->preview),
                                  (gpointer) &tg_tool->preview);
-
-      gimp_canvas_item_set_visible (tg_tool->preview, show_preview);
-
-      g_object_bind_property (G_OBJECT (options),          "preview-opacity",
-                              G_OBJECT (tg_tool->preview), "opacity",
-                              G_BINDING_SYNC_CREATE |
-                              G_BINDING_BIDIRECTIONAL);
-
-      GIMP_DRAW_TOOL_CLASS (parent_class)->draw (draw_tool);
     }
 
   if (tr_options->type == GIMP_TRANSFORM_TYPE_SELECTION)
@@ -778,6 +850,10 @@ gimp_transform_grid_tool_draw (GimpDrawTool *draw_tool)
             }
         }
     }
+
+  GIMP_DRAW_TOOL_CLASS (parent_class)->draw (draw_tool);
+
+  gimp_transform_grid_tool_update_preview (tg_tool);
 }
 
 static void
@@ -1049,18 +1125,52 @@ gimp_transform_grid_tool_widget_response (GimpToolWidget        *widget,
     }
 }
 
+static void
+gimp_transform_grid_tool_filter_flush (GimpDrawableFilter    *filter,
+                                       GimpTransformGridTool *tg_tool)
+{
+  GimpTool  *tool = GIMP_TOOL (tg_tool);
+  GimpImage *image = gimp_display_get_image (tool->display);
+
+  gimp_projection_flush (gimp_image_get_projection (image));
+}
+
+static void
+gimp_transform_grid_tool_image_linked_items_changed (GimpImage             *image,
+                                                     GimpTransformGridTool *tg_tool)
+{
+  if (tg_tool->filters)
+    {
+      gimp_transform_grid_tool_update_filters (tg_tool);
+      gimp_transform_grid_tool_update_preview (tg_tool);
+    }
+}
+
 static void
 gimp_transform_grid_tool_halt (GimpTransformGridTool *tg_tool)
 {
   GimpTool          *tool    = GIMP_TOOL (tg_tool);
   GimpTransformTool *tr_tool = GIMP_TRANSFORM_TOOL (tg_tool);
 
+  if (tool->display)
+    {
+      GimpImage *image = gimp_display_get_image (tool->display);
+
+      g_signal_handlers_disconnect_by_func (
+        image,
+        gimp_transform_grid_tool_image_linked_items_changed,
+        tg_tool);
+    }
+
   if (gimp_draw_tool_is_active (GIMP_DRAW_TOOL (tg_tool)))
     gimp_draw_tool_stop (GIMP_DRAW_TOOL (tg_tool));
 
   gimp_draw_tool_set_widget (GIMP_DRAW_TOOL (tg_tool), NULL);
   g_clear_object (&tg_tool->widget);
 
+  g_clear_pointer (&tg_tool->filters, g_hash_table_unref);
+  g_clear_pointer (&tg_tool->preview_drawables, g_list_free);
+
   if (tg_tool->gui)
     gimp_tool_gui_hide (tg_tool->gui);
 
@@ -1081,7 +1191,13 @@ gimp_transform_grid_tool_halt (GimpTransformGridTool *tg_tool)
   tool->display   = NULL;
   tool->drawable  = NULL;
 
-  tr_tool->object = NULL;
+  if (tr_tool->object)
+    {
+      if (GIMP_IS_DRAWABLE (tr_tool->object))
+        gimp_viewable_preview_thaw (GIMP_VIEWABLE (tr_tool->object));
+
+      tr_tool->object = NULL;
+    }
 }
 
 static void
@@ -1290,7 +1406,7 @@ gimp_transform_grid_tool_response (GimpToolGui           *gui,
         tg_options->direction_linked = direction_linked;
 
         /*  push the restored info to the undo stack  */
-        gimp_transform_grid_tool_push_internal_undo (tg_tool);
+        gimp_transform_grid_tool_push_internal_undo (tg_tool, FALSE);
       }
       break;
 
@@ -1356,7 +1472,7 @@ gimp_transform_grid_tool_response (GimpToolGui           *gui,
           if (transform_valid)
             {
               /*  push the new info to the undo stack  */
-              gimp_transform_grid_tool_push_internal_undo (tg_tool);
+              gimp_transform_grid_tool_push_internal_undo (tg_tool, FALSE);
             }
           else
             {
@@ -1391,6 +1507,19 @@ gimp_transform_grid_tool_response (GimpToolGui           *gui,
     }
 }
 
+static gboolean
+gimp_transform_grid_tool_composited_preview (GimpTransformGridTool *tg_tool)
+{
+  GimpTool                 *tool       = GIMP_TOOL (tg_tool);
+  GimpTransformOptions     *tr_options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tg_tool);
+  GimpTransformGridOptions *tg_options = GIMP_TRANSFORM_GRID_TOOL_GET_OPTIONS (tg_tool);
+  GimpImage                *image      = gimp_display_get_image (tool->display);
+
+  return tg_options->composited_preview                &&
+         tr_options->type == GIMP_TRANSFORM_TYPE_LAYER &&
+         gimp_channel_is_empty (gimp_image_get_mask (image));
+}
+
 static void
 gimp_transform_grid_tool_update_sensitivity (GimpTransformGridTool *tg_tool)
 {
@@ -1418,21 +1547,165 @@ gimp_transform_grid_tool_update_sensitivity (GimpTransformGridTool *tg_tool)
 static void
 gimp_transform_grid_tool_update_preview (GimpTransformGridTool *tg_tool)
 {
-  GimpTransformTool        *tr_tool = GIMP_TRANSFORM_TOOL (tg_tool);
-  GimpTransformGridOptions *options = GIMP_TRANSFORM_GRID_TOOL_GET_OPTIONS (tg_tool);
+  GimpTool                 *tool       = GIMP_TOOL (tg_tool);
+  GimpTransformTool        *tr_tool    = GIMP_TRANSFORM_TOOL (tg_tool);
+  GimpTransformOptions     *tr_options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tg_tool);
+  GimpTransformGridOptions *tg_options = GIMP_TRANSFORM_GRID_TOOL_GET_OPTIONS (tg_tool);
   gint                      i;
 
+  if (! tool->display)
+    return;
+
+  if (tg_options->show_preview                              &&
+      gimp_transform_grid_tool_composited_preview (tg_tool) &&
+      tr_tool->transform_valid)
+    {
+      GHashTableIter  iter;
+      GimpDrawable   *drawable;
+      Filter         *filter;
+      gboolean        flush = FALSE;
+
+      if (! tg_tool->filters)
+        {
+          tg_tool->filters = g_hash_table_new_full (
+            g_direct_hash, g_direct_equal,
+            NULL, (GDestroyNotify) filter_free);
+
+          gimp_transform_grid_tool_update_filters (tg_tool);
+        }
+
+      g_hash_table_iter_init (&iter, tg_tool->filters);
+
+      while (g_hash_table_iter_next (&iter,
+                                     (gpointer *) &drawable,
+                                     (gpointer *) &filter))
+        {
+          GimpMatrix3   transform;
+          GeglRectangle bounds;
+          gint          offset_x;
+          gint          offset_y;
+          gint          width;
+          gint          height;
+          gint          x1, y1;
+          gint          x2, y2;
+          gboolean      update = FALSE;
+
+          if (! filter->filter)
+            continue;
+
+          gimp_item_get_offset (GIMP_ITEM (drawable), &offset_x, &offset_y);
+
+          width  = gimp_item_get_width  (GIMP_ITEM (drawable));
+          height = gimp_item_get_height (GIMP_ITEM (drawable));
+
+          gimp_matrix3_identity (&transform);
+          gimp_matrix3_translate (&transform, +offset_x, +offset_y);
+          gimp_matrix3_mult (&tr_tool->transform, &transform);
+          gimp_matrix3_translate (&transform, -offset_x, -offset_y);
+
+          gimp_transform_resize_boundary (&tr_tool->transform,
+                                          gimp_item_get_clip (
+                                            GIMP_ITEM (filter->root_drawable),
+                                            tr_options->clip),
+                                          offset_x,         offset_y,
+                                          offset_x + width, offset_y + height,
+                                          &x1,              &y1,
+                                          &x2,              &y2);
+
+          bounds.x      = x1 - offset_x;
+          bounds.y      = y1 - offset_y;
+          bounds.width  = x2 - x1;
+          bounds.height = y2 - y1;
+
+          if (! gimp_matrix3_equal (&transform, &filter->transform))
+            {
+              filter->transform = transform;
+
+              gimp_gegl_node_set_matrix (filter->transform_node, &transform);
+
+              update = TRUE;
+            }
+
+          if (! gegl_rectangle_equal (&bounds, &filter->bounds))
+            {
+              filter->bounds = bounds;
+
+              gegl_node_set (filter->crop_node,
+                             "x",      (gdouble) bounds.x,
+                             "y",      (gdouble) bounds.y,
+                             "width",  (gdouble) bounds.width,
+                             "height", (gdouble) bounds.height,
+                             NULL);
+
+              update = TRUE;
+            }
+
+          if (GIMP_IS_LAYER (drawable))
+            {
+              gimp_drawable_filter_set_add_alpha (
+                filter->filter,
+                tr_options->interpolation != GIMP_INTERPOLATION_NONE);
+            }
+
+          if (update)
+            {
+              if (tg_options->synchronous_preview)
+                {
+                  g_signal_handlers_block_by_func (
+                    filter->filter,
+                    G_CALLBACK (gimp_transform_grid_tool_filter_flush),
+                    tg_tool);
+                }
+
+              gimp_drawable_filter_apply (filter->filter, NULL);
+
+              if (tg_options->synchronous_preview)
+                {
+                  g_signal_handlers_unblock_by_func (
+                    filter->filter,
+                    G_CALLBACK (gimp_transform_grid_tool_filter_flush),
+                    tg_tool);
+
+                  flush = TRUE;
+                }
+            }
+        }
+
+      if (flush)
+        {
+          GimpImage *image = gimp_display_get_image (tool->display);
+
+          gimp_projection_flush_now (gimp_image_get_projection (image), TRUE);
+          gimp_display_flush_now (tool->display);
+        }
+    }
+  else
+    {
+      g_clear_pointer (&tg_tool->filters, g_hash_table_unref);
+      g_clear_pointer (&tg_tool->preview_drawables, g_list_free);
+    }
+
   if (tg_tool->preview)
     {
-      gboolean show_preview = gimp_transform_grid_options_show_preview (options) &&
-                              tr_tool->transform_valid;
-
-      gimp_canvas_item_begin_change (tg_tool->preview);
-      gimp_canvas_item_set_visible (tg_tool->preview, show_preview);
-      g_object_set (tg_tool->preview,
-                    "transform", &tr_tool->transform,
-                    NULL);
-      gimp_canvas_item_end_change (tg_tool->preview);
+      if (tg_options->show_preview                                &&
+          ! gimp_transform_grid_tool_composited_preview (tg_tool) &&
+          tr_tool->transform_valid)
+        {
+          gimp_canvas_item_begin_change (tg_tool->preview);
+          gimp_canvas_item_set_visible (tg_tool->preview, TRUE);
+          g_object_set (
+            tg_tool->preview,
+            "transform", &tr_tool->transform,
+            "clip",      gimp_item_get_clip (GIMP_ITEM (tool->drawable),
+                                             tr_options->clip),
+            "opacity",   tg_options->preview_opacity,
+            NULL);
+          gimp_canvas_item_end_change (tg_tool->preview);
+        }
+      else
+        {
+          gimp_canvas_item_set_visible (tg_tool->preview, FALSE);
+        }
     }
 
   if (tg_tool->boundary_in)
@@ -1470,6 +1743,68 @@ gimp_transform_grid_tool_update_preview (GimpTransformGridTool *tg_tool)
     }
 }
 
+static void
+gimp_transform_grid_tool_update_filters (GimpTransformGridTool *tg_tool)
+{
+  GimpTool                 *tool    = GIMP_TOOL (tg_tool);
+  GimpTransformGridOptions *options = GIMP_TRANSFORM_GRID_TOOL_GET_OPTIONS (tg_tool);
+  GHashTable               *new_drawables;
+  GList                    *drawables;
+  GList                    *iter;
+  GimpDrawable             *drawable;
+  GHashTableIter            hash_iter;
+
+  if (! tg_tool->filters)
+    return;
+
+  if (options->preview_linked &&
+      gimp_item_get_linked (GIMP_ITEM (tool->drawable)))
+    {
+      GimpImage *image = gimp_display_get_image (tool->display);
+
+      drawables = gimp_image_item_list_get_list (image,
+                                                 GIMP_ITEM_TYPE_LAYERS |
+                                                 GIMP_ITEM_TYPE_CHANNELS,
+                                                 GIMP_ITEM_SET_LINKED);
+
+      drawables = gimp_image_item_list_filter (drawables);
+    }
+  else
+    {
+      drawables = g_list_prepend (NULL, tool->drawable);
+    }
+
+  new_drawables = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+  for (iter = drawables; iter; iter = g_list_next (iter))
+    g_hash_table_add (new_drawables, iter->data);
+
+  for (iter = tg_tool->preview_drawables; iter; iter = g_list_next (iter))
+    {
+      drawable = iter->data;
+
+      if (! g_hash_table_remove (new_drawables, drawable))
+        gimp_transform_grid_tool_remove_filter (drawable, tg_tool);
+    }
+
+  g_hash_table_iter_init (&hash_iter, new_drawables);
+
+  while (g_hash_table_iter_next (&hash_iter, (gpointer *) &drawable, NULL))
+    {
+      AddFilterData data;
+
+      data.tg_tool       = tg_tool;
+      data.root_drawable = drawable;
+
+      gimp_transform_grid_tool_add_filter (drawable, &data);
+    }
+
+  g_hash_table_unref (new_drawables);
+
+  g_list_free (tg_tool->preview_drawables);
+  tg_tool->preview_drawables = drawables;
+}
+
 static void
 gimp_transform_grid_tool_hide_active_object (GimpTransformGridTool *tg_tool,
                                              GimpObject            *object)
@@ -1483,6 +1818,7 @@ gimp_transform_grid_tool_hide_active_object (GimpTransformGridTool *tg_tool,
     {
       /*  hide only complete layers and channels, not layer masks  */
       if (tr_options->type == GIMP_TRANSFORM_TYPE_LAYER &&
+          ! options->composited_preview                 &&
           GIMP_IS_DRAWABLE (object)                     &&
           ! GIMP_IS_LAYER_MASK (object)                 &&
           gimp_item_get_visible (GIMP_ITEM (object))    &&
@@ -1531,6 +1867,194 @@ gimp_transform_grid_tool_show_active_object (GimpTransformGridTool *tg_tool)
     }
 }
 
+static void
+gimp_transform_grid_tool_add_filter (GimpDrawable  *drawable,
+                                     AddFilterData *data)
+{
+  Filter        *filter;
+  GimpLayerMode  mode = GIMP_LAYER_MODE_NORMAL;
+
+  if (GIMP_IS_LAYER (drawable))
+    {
+      gimp_layer_get_effective_mode (GIMP_LAYER (drawable),
+                                     &mode, NULL, NULL, NULL);
+    }
+
+  if (mode != GIMP_LAYER_MODE_PASS_THROUGH)
+    {
+      filter = filter_new (data->tg_tool, drawable, data->root_drawable, TRUE);
+    }
+  else
+    {
+      GimpContainer *container;
+
+      filter = filter_new (data->tg_tool, drawable, data->root_drawable, FALSE);
+
+      container = gimp_viewable_get_children (GIMP_VIEWABLE (drawable));
+
+      gimp_container_foreach (container,
+                              (GFunc) gimp_transform_grid_tool_add_filter,
+                              data);
+    }
+
+  g_hash_table_insert (data->tg_tool->filters, drawable, filter);
+
+  if (GIMP_IS_LAYER (drawable))
+    {
+      GimpLayerMask *mask = gimp_layer_get_mask (GIMP_LAYER (drawable));
+
+      if (mask)
+        gimp_transform_grid_tool_add_filter (GIMP_DRAWABLE (mask), data);
+    }
+}
+
+static void
+gimp_transform_grid_tool_remove_filter (GimpDrawable          *drawable,
+                                        GimpTransformGridTool *tg_tool)
+{
+  Filter *filter = g_hash_table_lookup (tg_tool->filters, drawable);
+
+  if (GIMP_IS_LAYER (drawable))
+    {
+      GimpLayerMask *mask = gimp_layer_get_mask (GIMP_LAYER (drawable));
+
+      if (mask)
+        gimp_transform_grid_tool_remove_filter (GIMP_DRAWABLE (mask), tg_tool);
+    }
+
+  if (! filter->filter)
+    {
+      GimpContainer *container;
+
+      container = gimp_viewable_get_children (GIMP_VIEWABLE (drawable));
+
+      gimp_container_foreach (container,
+                              (GFunc) gimp_transform_grid_tool_remove_filter,
+                              tg_tool);
+    }
+
+  g_hash_table_remove (tg_tool->filters, drawable);
+}
+
+static void
+gimp_transform_grid_tool_effective_mode_changed (GimpLayer             *layer,
+                                                 GimpTransformGridTool *tg_tool)
+{
+  Filter        *filter = g_hash_table_lookup (tg_tool->filters, layer);
+  GimpLayerMode  mode;
+  gboolean       old_pass_through;
+  gboolean       new_pass_through;
+
+  gimp_layer_get_effective_mode (layer, &mode, NULL, NULL, NULL);
+
+  old_pass_through = ! filter->filter;
+  new_pass_through = mode == GIMP_LAYER_MODE_PASS_THROUGH;
+
+  if (old_pass_through != new_pass_through)
+    {
+      AddFilterData data;
+
+      data.tg_tool       = tg_tool;
+      data.root_drawable = filter->root_drawable;
+
+      gimp_transform_grid_tool_remove_filter (GIMP_DRAWABLE (layer), tg_tool);
+      gimp_transform_grid_tool_add_filter    (GIMP_DRAWABLE (layer), &data);
+
+      gimp_transform_grid_tool_update_preview (tg_tool);
+    }
+}
+
+static Filter *
+filter_new (GimpTransformGridTool *tg_tool,
+            GimpDrawable          *drawable,
+            GimpDrawable          *root_drawable,
+            gboolean               add_filter)
+{
+  Filter   *filter = g_slice_new0 (Filter);
+  GeglNode *node;
+  GeglNode *input_node;
+  GeglNode *output_node;
+
+  filter->tg_tool       = tg_tool;
+  filter->drawable      = drawable;
+  filter->root_drawable = root_drawable;
+
+  if (add_filter)
+    {
+      node = gegl_node_new ();
+
+      input_node  = gegl_node_get_input_proxy (node, "input");
+      output_node = gegl_node_get_input_proxy (node, "output");
+
+      filter->transform_node = gegl_node_new_child (
+        node,
+        "operation", "gegl:transform",
+        "near-z",    GIMP_TRANSFORM_NEAR_Z,
+        "sampler",   GEGL_SAMPLER_NEAREST,
+        NULL);
+
+      filter->crop_node = gegl_node_new_child (
+        node,
+        "operation", "gegl:crop",
+        NULL);
+
+      gegl_node_link_many (input_node,
+                           filter->transform_node,
+                           filter->crop_node,
+                           output_node,
+                           NULL);
+
+      gimp_gegl_node_set_underlying_operation (node, filter->transform_node);
+
+      filter->filter = gimp_drawable_filter_new (
+        drawable,
+        GIMP_TRANSFORM_TOOL_GET_CLASS (tg_tool)->undo_desc,
+        node,
+        gimp_tool_get_icon_name (GIMP_TOOL (tg_tool)));
+
+      gimp_drawable_filter_set_clip (filter->filter, FALSE);
+      gimp_drawable_filter_set_override_constraints (filter->filter, TRUE);
+
+      g_signal_connect (
+        filter->filter, "flush",
+        G_CALLBACK (gimp_transform_grid_tool_filter_flush),
+        tg_tool);
+
+      g_object_unref (node);
+    }
+
+  if (GIMP_IS_GROUP_LAYER (drawable))
+    {
+      g_signal_connect (
+        drawable, "effective-mode-changed",
+        G_CALLBACK (gimp_transform_grid_tool_effective_mode_changed),
+        tg_tool);
+    }
+
+  return filter;
+}
+
+static void
+filter_free (Filter *filter)
+{
+  if (filter->filter)
+    {
+      gimp_drawable_filter_abort (filter->filter);
+
+      g_object_unref (filter->filter);
+    }
+
+  if (GIMP_IS_GROUP_LAYER (filter->drawable))
+    {
+      g_signal_handlers_disconnect_by_func (
+        filter->drawable,
+        gimp_transform_grid_tool_effective_mode_changed,
+        filter->tg_tool);
+    }
+
+  g_slice_free (Filter, filter);
+}
+
 static UndoInfo *
 undo_info_new (void)
 {
@@ -1599,7 +2123,8 @@ gimp_transform_grid_tool_matrix_to_info (GimpTransformGridTool *tg_tool,
 }
 
 void
-gimp_transform_grid_tool_push_internal_undo (GimpTransformGridTool *tg_tool)
+gimp_transform_grid_tool_push_internal_undo (GimpTransformGridTool *tg_tool,
+                                             gboolean               compress)
 {
   UndoInfo *undo_info;
 
@@ -1614,23 +2139,46 @@ gimp_transform_grid_tool_push_internal_undo (GimpTransformGridTool *tg_tool)
   if (! trans_infos_equal (undo_info->trans_infos, tg_tool->trans_infos))
     {
       GimpTransformOptions *tr_options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tg_tool);
+      gint64                time       = 0;
+      gboolean              flush = FALSE;
 
-      undo_info            = undo_info_new ();
+      if (tg_tool->undo_list->next == NULL)
+        flush = TRUE;
+
+      if (compress)
+        time = g_get_monotonic_time ();
+
+      if (! compress || time - undo_info->time >= UNDO_COMPRESS_TIME)
+        {
+          undo_info = undo_info_new ();
+
+          tg_tool->undo_list = g_list_prepend (tg_tool->undo_list, undo_info);
+        }
+
+      undo_info->time      = time;
       undo_info->direction = tr_options->direction;
       memcpy (undo_info->trans_infos, tg_tool->trans_infos,
               sizeof (tg_tool->trans_infos));
 
-      tg_tool->undo_list = g_list_prepend (tg_tool->undo_list, undo_info);
-
       /* If we undid anything and started interacting, we have to
        * discard the redo history
        */
-      g_list_free_full (tg_tool->redo_list, (GDestroyNotify) undo_info_free);
-      tg_tool->redo_list = NULL;
+      if (tg_tool->redo_list)
+        {
+          g_list_free_full (tg_tool->redo_list,
+                            (GDestroyNotify) undo_info_free);
+          tg_tool->redo_list = NULL;
+
+          flush = TRUE;
+        }
 
       gimp_transform_grid_tool_update_sensitivity (tg_tool);
 
       /*  update the undo actions / menu items  */
-      gimp_image_flush (gimp_display_get_image (GIMP_TOOL (tg_tool)->display));
+      if (flush)
+        {
+          gimp_image_flush (
+            gimp_display_get_image (GIMP_TOOL (tg_tool)->display));
+        }
     }
 }
