@@ -25,11 +25,14 @@
 
 #include "tools-types.h"
 
+#include "config/gimpguiconfig.h"
+
 #include "core/gimp.h"
 #include "core/gimpdrawable-transform.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-item-list.h"
+#include "core/gimpimage-transform.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpitem-linked.h"
 #include "core/gimplayer.h"
@@ -40,11 +43,14 @@
 #include "vectors/gimpvectors.h"
 
 #include "display/gimpdisplay.h"
+#include "display/gimpdisplayshell.h"
 
 #include "widgets/gimpmessagedialog.h"
 #include "widgets/gimpmessagebox.h"
+#include "widgets/gimpwidgets-utils.h"
 
 #include "gimptoolcontrol.h"
+#include "gimptools-utils.h"
 #include "gimptransformoptions.h"
 #include "gimptransformtool.h"
 
@@ -59,17 +65,25 @@
 
 /*  local function prototypes  */
 
-static GeglBuffer * gimp_transform_tool_real_transform (GimpTransformTool  *tr_tool,
-                                                        GimpItem           *item,
-                                                        GeglBuffer         *orig_buffer,
-                                                        gint                orig_offset_x,
-                                                        gint                orig_offset_y,
-                                                        GimpColorProfile  **buffer_profile,
-                                                        gint               *new_offset_x,
-                                                        gint               *new_offset_y);
+static void                     gimp_transform_tool_control            (GimpTool           *tool,
+                                                                        GimpToolAction      action,
+                                                                        GimpDisplay        *display);
 
-static gboolean     gimp_transform_tool_confirm        (GimpTransformTool  *tr_tool,
-                                                        GimpDisplay        *display);
+static gchar                  * gimp_transform_tool_real_get_undo_desc (GimpTransformTool  *tr_tool);
+static GimpTransformDirection   gimp_transform_tool_real_get_direction (GimpTransformTool  *tr_tool);
+static GeglBuffer             * gimp_transform_tool_real_transform     (GimpTransformTool  *tr_tool,
+                                                                        GimpObject         *object,
+                                                                        GeglBuffer         *orig_buffer,
+                                                                        gint                orig_offset_x,
+                                                                        gint                orig_offset_y,
+                                                                        GimpColorProfile  **buffer_profile,
+                                                                        gint               *new_offset_x,
+                                                                        gint               *new_offset_y);
+
+static void                     gimp_transform_tool_halt               (GimpTransformTool  *tr_tool);
+
+static gboolean                 gimp_transform_tool_confirm            (GimpTransformTool  *tr_tool,
+                                                                        GimpDisplay        *display);
 
 
 G_DEFINE_TYPE (GimpTransformTool, gimp_transform_tool, GIMP_TYPE_DRAW_TOOL)
@@ -83,10 +97,16 @@ G_DEFINE_TYPE (GimpTransformTool, gimp_transform_tool, GIMP_TYPE_DRAW_TOOL)
 static void
 gimp_transform_tool_class_init (GimpTransformToolClass *klass)
 {
+  GimpToolClass *tool_class = GIMP_TOOL_CLASS (klass);
+
+  tool_class->control  = gimp_transform_tool_control;
+
   klass->recalc_matrix = NULL;
-  klass->get_undo_desc = NULL;
+  klass->get_undo_desc = gimp_transform_tool_real_get_undo_desc;
+  klass->get_direction = gimp_transform_tool_real_get_direction;
   klass->transform     = gimp_transform_tool_real_transform;
 
+  klass->undo_desc     = _("Transform");
   klass->progress_text = _("Transforming");
 }
 
@@ -95,11 +115,53 @@ gimp_transform_tool_init (GimpTransformTool *tr_tool)
 {
   gimp_matrix3_identity (&tr_tool->transform);
   tr_tool->transform_valid = TRUE;
+
+  tr_tool->restore_type = FALSE;
+}
+
+static void
+gimp_transform_tool_control (GimpTool       *tool,
+                             GimpToolAction  action,
+                             GimpDisplay    *display)
+{
+  GimpTransformTool *tr_tool = GIMP_TRANSFORM_TOOL (tool);
+
+  switch (action)
+    {
+    case GIMP_TOOL_ACTION_PAUSE:
+      break;
+
+    case GIMP_TOOL_ACTION_RESUME:
+      break;
+
+    case GIMP_TOOL_ACTION_HALT:
+      gimp_transform_tool_halt (tr_tool);
+      break;
+
+    case GIMP_TOOL_ACTION_COMMIT:
+      break;
+    }
+
+  GIMP_TOOL_CLASS (parent_class)->control (tool, action, display);
+}
+
+static gchar *
+gimp_transform_tool_real_get_undo_desc (GimpTransformTool *tr_tool)
+{
+  return g_strdup (GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->undo_desc);
+}
+
+static GimpTransformDirection
+gimp_transform_tool_real_get_direction (GimpTransformTool *tr_tool)
+{
+  GimpTransformOptions *options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
+
+  return options->direction;
 }
 
 static GeglBuffer *
 gimp_transform_tool_real_transform (GimpTransformTool *tr_tool,
-                                    GimpItem          *active_item,
+                                    GimpObject        *object,
                                     GeglBuffer        *orig_buffer,
                                     gint               orig_offset_x,
                                     gint               orig_offset_y,
@@ -113,31 +175,29 @@ gimp_transform_tool_real_transform (GimpTransformTool *tr_tool,
   GimpContext            *context = GIMP_CONTEXT (options);
   GeglBuffer             *ret     = NULL;
   GimpTransformResize     clip    = options->clip;
+  GimpTransformDirection  direction;
   GimpProgress           *progress;
 
+  direction = GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->get_direction (tr_tool);
+
   progress = gimp_progress_start (GIMP_PROGRESS (tool), FALSE,
                                   "%s", klass->progress_text);
 
   if (orig_buffer)
     {
       /*  this happens when transforming a selection cut out of a
-       *  normal drawable, or the selection
+       *  normal drawable
        */
 
-      /*  always clip the selection and unfloated channels
-       *  so they keep their size
-       */
-      if (GIMP_IS_CHANNEL (active_item) &&
-          ! babl_format_has_alpha (gegl_buffer_get_format (orig_buffer)))
-        clip = GIMP_TRANSFORM_RESIZE_CLIP;
+      g_return_val_if_fail (GIMP_IS_DRAWABLE (object), NULL);
 
-      ret = gimp_drawable_transform_buffer_affine (GIMP_DRAWABLE (active_item),
+      ret = gimp_drawable_transform_buffer_affine (GIMP_DRAWABLE (object),
                                                    context,
                                                    orig_buffer,
                                                    orig_offset_x,
                                                    orig_offset_y,
                                                    &tr_tool->transform,
-                                                   options->direction,
+                                                   direction,
                                                    options->interpolation,
                                                    clip,
                                                    buffer_profile,
@@ -145,34 +205,46 @@ gimp_transform_tool_real_transform (GimpTransformTool *tr_tool,
                                                    new_offset_y,
                                                    progress);
     }
-  else
+  else if (GIMP_IS_ITEM (object))
     {
       /*  this happens for entire drawables, paths and layer groups  */
 
-      if (gimp_item_get_linked (active_item))
+      GimpItem *item = GIMP_ITEM (object);
+
+      if (gimp_item_get_linked (item))
         {
-          gimp_item_linked_transform (active_item, context,
+          gimp_item_linked_transform (item, context,
                                       &tr_tool->transform,
-                                      options->direction,
+                                      direction,
                                       options->interpolation,
                                       clip,
                                       progress);
         }
       else
         {
-          /*  always clip layer masks so they keep their size
-           */
-          if (GIMP_IS_CHANNEL (active_item))
-            clip = GIMP_TRANSFORM_RESIZE_CLIP;
+          clip = gimp_item_get_clip (item, clip);
 
-          gimp_item_transform (active_item, context,
+          gimp_item_transform (item, context,
                                &tr_tool->transform,
-                               options->direction,
+                               direction,
                                options->interpolation,
                                clip,
                                progress);
         }
     }
+  else
+    {
+      /*  this happens for images  */
+
+      g_return_val_if_fail (GIMP_IS_IMAGE (object), NULL);
+
+      gimp_image_transform (GIMP_IMAGE (object), context,
+                            &tr_tool->transform,
+                            direction,
+                            options->interpolation,
+                            clip,
+                            progress);
+    }
 
   if (progress)
     gimp_progress_end (progress);
@@ -180,33 +252,57 @@ gimp_transform_tool_real_transform (GimpTransformTool *tr_tool,
   return ret;
 }
 
+static void
+gimp_transform_tool_halt (GimpTransformTool *tr_tool)
+{
+  GimpTransformOptions *options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
+
+  tr_tool->x1 = 0;
+  tr_tool->y1 = 0;
+  tr_tool->x2 = 0;
+  tr_tool->y2 = 0;
+
+  if (tr_tool->restore_type)
+    {
+      g_object_set (options,
+                    "type", tr_tool->saved_type,
+                    NULL);
+
+      tr_tool->restore_type = FALSE;
+    }
+}
+
 static gboolean
 gimp_transform_tool_confirm (GimpTransformTool *tr_tool,
                              GimpDisplay       *display)
 {
-  GimpTransformOptions *options   = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
-  GimpDisplayShell     *shell     = gimp_display_get_shell (display);
-  GimpImage            *image     = gimp_display_get_image (display);
-  GimpItem             *active_item;
-  gdouble               max_ratio = 0.0;
+  GimpTransformOptions *options          = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
+  GimpDisplayShell     *shell            = gimp_display_get_shell (display);
+  GimpImage            *image            = gimp_display_get_image (display);
+  GimpObject           *active_object;
+  gdouble               max_ratio        = 0.0;
+  GimpObject           *max_ratio_object = NULL;
 
-  active_item = gimp_transform_tool_get_active_item (tr_tool, display);
+  active_object = gimp_transform_tool_get_active_object (tr_tool, display);
 
   if (GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->recalc_matrix)
     {
-      GimpMatrix3    transform;
-      GeglRectangle  selection_bounds;
-      gboolean       selection_empty = TRUE;
-      GList         *items;
-      GList         *iter;
+      GimpMatrix3             transform;
+      GimpTransformDirection  direction;
+      GeglRectangle           selection_bounds;
+      gboolean                selection_empty = TRUE;
+      GList                  *objects;
+      GList                  *iter;
 
       transform = tr_tool->transform;
+      direction = GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->get_direction (
+        tr_tool);
 
-      if (options->direction == GIMP_TRANSFORM_BACKWARD)
+      if (direction == GIMP_TRANSFORM_BACKWARD)
         gimp_matrix3_invert (&transform);
 
       if (options->type == GIMP_TRANSFORM_TYPE_LAYER &&
-          ! gimp_viewable_get_children (GIMP_VIEWABLE (active_item)))
+          ! gimp_viewable_get_children (GIMP_VIEWABLE (active_object)))
         {
           selection_empty = ! gimp_item_bounds (
             GIMP_ITEM (gimp_image_get_mask (image)),
@@ -214,48 +310,76 @@ gimp_transform_tool_confirm (GimpTransformTool *tr_tool,
             &selection_bounds.width, &selection_bounds.height);
         }
 
-      if (selection_empty && gimp_item_get_linked (active_item))
+      if (selection_empty              &&
+          GIMP_IS_ITEM (active_object) &&
+          gimp_item_get_linked (GIMP_ITEM (active_object)))
         {
-          items = gimp_image_item_list_get_list (image,
-                                                 GIMP_ITEM_TYPE_ALL,
-                                                 GIMP_ITEM_SET_LINKED);
+          objects = gimp_image_item_list_get_list (image,
+                                                   GIMP_ITEM_TYPE_ALL,
+                                                   GIMP_ITEM_SET_LINKED);
         }
       else
         {
-          items = g_list_append (NULL, active_item);
+          objects = g_list_append (NULL, active_object);
         }
 
-      for (iter = items; iter; iter = g_list_next (iter))
+      if (options->type == GIMP_TRANSFORM_TYPE_IMAGE)
         {
-          GimpItem            *item = iter->data;
-          GimpTransformResize  clip = options->clip;
+          objects = g_list_concat (
+            objects,
+            gimp_image_item_list_get_list (image,
+                                           GIMP_ITEM_TYPE_ALL,
+                                           GIMP_ITEM_SET_ALL));
+        }
+
+      for (iter = objects; iter; iter = g_list_next (iter))
+        {
+          GimpObject          *object = iter->data;
+          GimpTransformResize  clip   = options->clip;
           GeglRectangle        orig_bounds;
           GeglRectangle        new_bounds;
+          gdouble              ratio  = 0.0;
 
-          if (GIMP_IS_DRAWABLE (item))
+          if (GIMP_IS_DRAWABLE (object))
             {
               if (selection_empty)
                 {
+                  GimpItem *item = GIMP_ITEM (object);
+
                   gimp_item_get_offset (item, &orig_bounds.x, &orig_bounds.y);
 
                   orig_bounds.width  = gimp_item_get_width  (item);
                   orig_bounds.height = gimp_item_get_height (item);
+
+                  clip = gimp_item_get_clip (item, clip);
                 }
               else
                 {
                   orig_bounds = selection_bounds;
                 }
-
-              clip = gimp_drawable_transform_get_effective_clip (
-                GIMP_DRAWABLE (item), NULL, clip);
             }
-          else
+          else if (GIMP_IS_ITEM (object))
             {
+              GimpItem *item = GIMP_ITEM (object);
+
               gimp_item_bounds (item,
                                 &orig_bounds.x,     &orig_bounds.y,
                                 &orig_bounds.width, &orig_bounds.height);
 
-              clip = GIMP_TRANSFORM_RESIZE_ADJUST;
+              clip = gimp_item_get_clip (item, clip);
+            }
+          else
+            {
+              GimpImage *image;
+
+              g_return_val_if_fail (GIMP_IS_IMAGE (object), FALSE);
+
+              image = GIMP_IMAGE (object);
+
+              orig_bounds.x      = 0;
+              orig_bounds.y      = 0;
+              orig_bounds.width  = gimp_image_get_width  (image);
+              orig_bounds.height = gimp_image_get_height (image);
             }
 
           gimp_transform_resize_boundary (&transform, clip,
@@ -275,20 +399,26 @@ gimp_transform_tool_confirm (GimpTransformTool *tr_tool,
 
           if (new_bounds.width > orig_bounds.width)
             {
-              max_ratio = MAX (max_ratio,
-                               (gdouble) new_bounds.width /
-                               (gdouble) gimp_image_get_width (image));
+              ratio = MAX (ratio,
+                           (gdouble) new_bounds.width /
+                           (gdouble) gimp_image_get_width (image));
             }
 
           if (new_bounds.height > orig_bounds.height)
             {
-              max_ratio = MAX (max_ratio,
-                               (gdouble) new_bounds.height /
-                               (gdouble) gimp_image_get_height (image));
+              ratio = MAX (ratio,
+                           (gdouble) new_bounds.height /
+                           (gdouble) gimp_image_get_height (image));
+            }
+
+          if (ratio > max_ratio)
+            {
+              max_ratio        = ratio;
+              max_ratio_object = object;
             }
         }
 
-      g_list_free (items);
+      g_list_free (objects);
     }
 
   if (max_ratio > MIN_CONFIRMATION_RATIO)
@@ -313,15 +443,31 @@ gimp_transform_tool_confirm (GimpTransformTool *tr_tool,
                                                GTK_RESPONSE_CANCEL,
                                                -1);
 
-      gimp_message_box_set_primary_text (GIMP_MESSAGE_DIALOG (dialog)->box,
-                                         _("Transformation creates "
-                                           "a very large item."));
+      if (GIMP_IS_ITEM (max_ratio_object))
+        {
+          gimp_message_box_set_primary_text (GIMP_MESSAGE_DIALOG (dialog)->box,
+                                             _("Transformation creates "
+                                               "a very large item."));
 
-      gimp_message_box_set_text (GIMP_MESSAGE_DIALOG (dialog)->box,
-                                 _("Applying the transformation will result "
-                                   "in an item that is over %g times larger "
-                                   "than the image."),
-                                   floor (max_ratio));
+          gimp_message_box_set_text (
+            GIMP_MESSAGE_DIALOG (dialog)->box,
+            _("Applying the transformation will result "
+              "in an item that is over %g times larger "
+              "than the image."),
+              floor (max_ratio));
+        }
+      else
+        {
+          gimp_message_box_set_primary_text (GIMP_MESSAGE_DIALOG (dialog)->box,
+                                             _("Transformation creates "
+                                               "a very large image."));
+
+          gimp_message_box_set_text (
+            GIMP_MESSAGE_DIALOG (dialog)->box,
+            _("Applying the transformation will enlarge "
+              "the image by a factor of %g."),
+              floor (max_ratio));
+        }
 
       response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -343,6 +489,7 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
                             GimpDisplay       *display)
 {
   GimpTransformOptions *options;
+  GimpDisplayShell     *shell;
   GimpImage            *image;
   gboolean              non_empty = TRUE;
 
@@ -350,6 +497,7 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
 
   options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
   image   = gimp_display_get_image (display);
+  shell   = gimp_display_get_shell (display);
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), FALSE);
 
@@ -423,6 +571,27 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
       }
 
       break;
+
+    case GIMP_TRANSFORM_TYPE_IMAGE:
+      if (! shell->show_all)
+        {
+          tr_tool->x1 = 0;
+          tr_tool->y1 = 0;
+          tr_tool->x2 = gimp_image_get_width  (image);
+          tr_tool->y2 = gimp_image_get_height (image);
+        }
+      else
+        {
+          GeglRectangle bounding_box;
+
+          bounding_box = gimp_display_shell_get_bounding_box (shell);
+
+          tr_tool->x1 = bounding_box.x;
+          tr_tool->y1 = bounding_box.y;
+          tr_tool->x2 = bounding_box.x + bounding_box.width;
+          tr_tool->y2 = bounding_box.y + bounding_box.height;
+        }
+      break;
     }
 
   return non_empty;
@@ -435,19 +604,20 @@ gimp_transform_tool_recalc_matrix (GimpTransformTool *tr_tool,
   g_return_if_fail (GIMP_IS_TRANSFORM_TOOL (tr_tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  gimp_transform_tool_bounds (tr_tool, display);
+  if (tr_tool->x1 == tr_tool->x2 && tr_tool->y1 == tr_tool->y2)
+    gimp_transform_tool_bounds (tr_tool, display);
 
   if (GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->recalc_matrix)
     GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->recalc_matrix (tr_tool);
 }
 
-GimpItem *
-gimp_transform_tool_get_active_item (GimpTransformTool  *tr_tool,
-                                     GimpDisplay        *display)
+GimpObject *
+gimp_transform_tool_get_active_object (GimpTransformTool  *tr_tool,
+                                       GimpDisplay        *display)
 {
   GimpTransformOptions *options;
   GimpImage            *image;
-  GimpItem             *item = NULL;
+  GimpObject           *object = NULL;
 
   g_return_val_if_fail (GIMP_IS_TRANSFORM_TOOL (tr_tool), NULL);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), NULL);
@@ -458,39 +628,44 @@ gimp_transform_tool_get_active_item (GimpTransformTool  *tr_tool,
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
 
-  if (tr_tool->item)
-    return tr_tool->item;
+  if (tr_tool->object)
+    return tr_tool->object;
 
   switch (options->type)
     {
     case GIMP_TRANSFORM_TYPE_LAYER:
-      item = GIMP_ITEM (gimp_image_get_active_drawable (image));
+      object = GIMP_OBJECT (gimp_image_get_active_drawable (image));
       break;
 
     case GIMP_TRANSFORM_TYPE_SELECTION:
-      item = GIMP_ITEM (gimp_image_get_mask (image));
+      object = GIMP_OBJECT (gimp_image_get_mask (image));
 
-      if (gimp_channel_is_empty (GIMP_CHANNEL (item)))
-        item = NULL;
+      if (gimp_channel_is_empty (GIMP_CHANNEL (object)))
+        object = NULL;
       break;
 
     case GIMP_TRANSFORM_TYPE_PATH:
-      item = GIMP_ITEM (gimp_image_get_active_vectors (image));
+      object = GIMP_OBJECT (gimp_image_get_active_vectors (image));
+      break;
+
+    case GIMP_TRANSFORM_TYPE_IMAGE:
+      object = GIMP_OBJECT (image);
       break;
     }
 
-  return item;
+  return object;
 }
 
-GimpItem *
-gimp_transform_tool_check_active_item (GimpTransformTool  *tr_tool,
-                                       GimpDisplay        *display,
-                                       GError            **error)
+GimpObject *
+gimp_transform_tool_check_active_object (GimpTransformTool  *tr_tool,
+                                         GimpDisplay        *display,
+                                         GError            **error)
 {
   GimpTransformOptions *options;
-  GimpItem             *item;
+  GimpObject           *object;
   const gchar          *null_message   = NULL;
   const gchar          *locked_message = NULL;
+  GimpGuiConfig        *config         = GIMP_GUI_CONFIG (display->gimp->config);
 
   g_return_val_if_fail (GIMP_IS_TRANSFORM_TOOL (tr_tool), NULL);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), NULL);
@@ -498,22 +673,25 @@ gimp_transform_tool_check_active_item (GimpTransformTool  *tr_tool,
 
   options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
 
-  item = gimp_transform_tool_get_active_item (tr_tool, display);
+  object = gimp_transform_tool_get_active_object (tr_tool, display);
 
   switch (options->type)
     {
     case GIMP_TRANSFORM_TYPE_LAYER:
       null_message = _("There is no layer to transform.");
 
-      if (item)
+      if (object)
         {
+          GimpItem *item = GIMP_ITEM (object);
+
           if (gimp_item_is_content_locked (item))
             locked_message = _("The active layer's pixels are locked.");
           else if (gimp_item_is_position_locked (item))
             locked_message = _("The active layer's position and size are locked.");
 
           if (! gimp_item_is_visible (item) &&
-              item != tr_tool->item) /* see bug #759194 */
+              ! config->edit_non_visible &&
+              object != tr_tool->object) /* see bug #759194 */
             {
               g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
                                    _("The active layer is not visible."));
@@ -532,8 +710,10 @@ gimp_transform_tool_check_active_item (GimpTransformTool  *tr_tool,
     case GIMP_TRANSFORM_TYPE_SELECTION:
       null_message = _("There is no selection to transform.");
 
-      if (item)
+      if (object)
         {
+          GimpItem *item = GIMP_ITEM (object);
+
           /* cannot happen, so don't translate these messages */
           if (gimp_item_is_content_locked (item))
             locked_message = "The selection's pixels are locked.";
@@ -545,8 +725,10 @@ gimp_transform_tool_check_active_item (GimpTransformTool  *tr_tool,
     case GIMP_TRANSFORM_TYPE_PATH:
       null_message = _("There is no path to transform.");
 
-      if (item)
+      if (object)
         {
+          GimpItem *item = GIMP_ITEM (object);
+
           if (gimp_item_is_content_locked (item))
             locked_message = _("The active path's strokes are locked.");
           else if (gimp_item_is_position_locked (item))
@@ -555,21 +737,30 @@ gimp_transform_tool_check_active_item (GimpTransformTool  *tr_tool,
             locked_message = _("The active path has no strokes.");
         }
       break;
+
+    case GIMP_TRANSFORM_TYPE_IMAGE:
+      /* cannot happen, so don't translate this message */
+      null_message = "There is no image to transform.";
+      break;
     }
 
-  if (! item)
+  if (! object)
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED, null_message);
+      if (error)
+        gimp_widget_blink (options->type_box);
       return NULL;
     }
 
   if (locked_message)
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED, locked_message);
+      if (error)
+        gimp_tools_blink_lock_box (display->gimp, GIMP_ITEM (object));
       return NULL;
     }
 
-  return item;
+  return object;
 }
 
 gboolean
@@ -580,7 +771,7 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
   GimpTransformOptions *options;
   GimpContext          *context;
   GimpImage            *image;
-  GimpItem             *active_item;
+  GimpObject           *active_object;
   GeglBuffer           *orig_buffer   = NULL;
   gint                  orig_offset_x = 0;
   gint                  orig_offset_y = 0;
@@ -602,10 +793,10 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), FALSE);
 
-  active_item = gimp_transform_tool_check_active_item (tr_tool, display,
-                                                       &error);
+  active_object = gimp_transform_tool_check_active_object (tr_tool, display,
+                                                           &error);
 
-  if (! active_item)
+  if (! active_object)
     {
       gimp_tool_message_literal (tool, display, error->message);
       g_clear_error (&error);
@@ -643,35 +834,35 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
   switch (options->type)
     {
     case GIMP_TRANSFORM_TYPE_LAYER:
-      if (! gimp_viewable_get_children (GIMP_VIEWABLE (active_item)) &&
+      if (! gimp_viewable_get_children (GIMP_VIEWABLE (active_object)) &&
           ! gimp_channel_is_empty (gimp_image_get_mask (image)))
         {
-          orig_buffer = gimp_drawable_transform_cut (GIMP_DRAWABLE (active_item),
-                                                     context,
-                                                     &orig_offset_x,
-                                                     &orig_offset_y,
-                                                     &new_layer);
+          orig_buffer = gimp_drawable_transform_cut (
+            GIMP_DRAWABLE (active_object),
+            context,
+            &orig_offset_x,
+            &orig_offset_y,
+            &new_layer);
         }
       break;
 
     case GIMP_TRANSFORM_TYPE_SELECTION:
-      orig_buffer = g_object_ref (gimp_drawable_get_buffer (GIMP_DRAWABLE (active_item)));
-      break;
-
     case GIMP_TRANSFORM_TYPE_PATH:
+    case GIMP_TRANSFORM_TYPE_IMAGE:
       break;
     }
 
   /*  Send the request for the transformation to the tool...
    */
-  new_buffer = GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->transform (tr_tool,
-                                                                   active_item,
-                                                                   orig_buffer,
-                                                                   orig_offset_x,
-                                                                   orig_offset_y,
-                                                                   &buffer_profile,
-                                                                   &new_offset_x,
-                                                                   &new_offset_y);
+  new_buffer = GIMP_TRANSFORM_TOOL_GET_CLASS (tr_tool)->transform (
+    tr_tool,
+    active_object,
+    orig_buffer,
+    orig_offset_x,
+    orig_offset_y,
+    &buffer_profile,
+    &new_offset_x,
+    &new_offset_y);
 
   if (orig_buffer)
     g_object_unref (orig_buffer);
@@ -684,7 +875,7 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
           /*  paste the new transformed image to the image...also implement
            *  undo...
            */
-          gimp_drawable_transform_paste (GIMP_DRAWABLE (active_item),
+          gimp_drawable_transform_paste (GIMP_DRAWABLE (active_object),
                                          new_buffer, buffer_profile,
                                          new_offset_x, new_offset_y,
                                          new_layer);
@@ -692,18 +883,9 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
         }
       break;
 
-     case GIMP_TRANSFORM_TYPE_SELECTION:
-      if (new_buffer)
-        {
-          gimp_channel_push_undo (GIMP_CHANNEL (active_item), NULL);
-
-          gimp_drawable_set_buffer (GIMP_DRAWABLE (active_item),
-                                    FALSE, NULL, new_buffer);
-          g_object_unref (new_buffer);
-        }
-      break;
-
+    case GIMP_TRANSFORM_TYPE_SELECTION:
     case GIMP_TRANSFORM_TYPE_PATH:
+    case GIMP_TRANSFORM_TYPE_IMAGE:
       /*  Nothing to be done  */
       break;
     }
@@ -721,3 +903,25 @@ gimp_transform_tool_transform (GimpTransformTool *tr_tool,
 
   return TRUE;
 }
+
+void
+gimp_transform_tool_set_type (GimpTransformTool *tr_tool,
+                              GimpTransformType  type)
+{
+  GimpTransformOptions *options;
+
+  g_return_if_fail (GIMP_IS_TRANSFORM_TOOL (tr_tool));
+
+  options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
+
+  if (! tr_tool->restore_type)
+    tr_tool->saved_type = options->type;
+
+  tr_tool->restore_type = FALSE;
+
+  g_object_set (options,
+                "type", type,
+                NULL);
+
+  tr_tool->restore_type = TRUE;
+}
