@@ -28,6 +28,8 @@
 
 #include "widgets-types.h"
 
+#include "gegl/gimp-babl-compat.h"
+
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpbrush.h"
@@ -258,11 +260,18 @@ gimp_brush_select_run_callback (GimpPdbDialog  *dialog,
 {
   GimpBrush      *brush = GIMP_BRUSH (object);
   GimpTempBuf    *mask  = gimp_brush_get_mask (brush);
+  const Babl     *format;
+  gpointer        data;
   GimpArray      *array;
   GimpValueArray *return_vals;
 
-  array = gimp_array_new (gimp_temp_buf_get_data (mask),
-                          gimp_temp_buf_get_data_size (mask),
+  format = gimp_babl_compat_u8_mask_format (gimp_temp_buf_get_format (mask));
+  data   = gimp_temp_buf_lock (mask, format, GEGL_ACCESS_READ);
+
+  array = gimp_array_new (data,
+                          gimp_temp_buf_get_width         (mask) *
+                          gimp_temp_buf_get_height        (mask) *
+                          babl_format_get_bytes_per_pixel (format),
                           TRUE);
 
   return_vals =
@@ -283,6 +292,8 @@ gimp_brush_select_run_callback (GimpPdbDialog  *dialog,
 
   gimp_array_free (array);
 
+  gimp_temp_buf_unlock (mask, data);
+
   return return_vals;
 }
 
