@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -58,7 +58,7 @@ gimp_color_selector_palette_class_init (GimpColorSelectorPaletteClass *klass)
 
   selector_class->name       = _("Palette");
   selector_class->help_id    = "gimp-colorselector-palette";
-  selector_class->icon_name  = GIMP_STOCK_PALETTE;
+  selector_class->icon_name  = GIMP_ICON_PALETTE;
   selector_class->set_color  = gimp_color_selector_palette_set_color;
   selector_class->set_config = gimp_color_selector_palette_set_config;
 }
@@ -127,8 +127,7 @@ gimp_color_selector_palette_set_config (GimpColorSelector *selector,
       gimp_view_renderer_set_context (GIMP_VIEW (select->view)->renderer,
                                       NULL);
 
-      g_object_unref (select->context);
-      select->context = NULL;
+      g_clear_object (&select->context);
     }
 
   if (config)
