@@ -261,7 +261,7 @@ gimp_palette_editor_init (GimpPaletteEditor *editor)
 
   editor->columns_adj = (GtkAdjustment *)
     gtk_adjustment_new (0, 0, 64, 1, 4, 0);
-  spinbutton = gtk_spin_button_new (editor->columns_adj, 1.0, 0);
+  spinbutton = gimp_spin_button_new (editor->columns_adj, 1.0, 0);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
   gtk_widget_show (spinbutton);
@@ -498,6 +498,13 @@ gimp_palette_editor_edit_color (GimpPaletteEditor *editor)
                                          data_editor->context);
       gimp_color_dialog_set_color (GIMP_COLOR_DIALOG (editor->color_dialog),
                                    &editor->color->color);
+
+      if (! gtk_widget_get_visible (editor->color_dialog))
+        gimp_dialog_factory_position_dialog (gimp_dialog_factory_get_singleton (),
+                                             "gimp-palette-editor-color-dialog",
+                                             editor->color_dialog,
+                                             gtk_widget_get_screen (GTK_WIDGET (editor)),
+                                             gimp_widget_get_monitor (GTK_WIDGET (editor)));
     }
 
   gtk_window_present (GTK_WINDOW (editor->color_dialog));
