@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -40,6 +40,7 @@
 #include "core/gimpviewable.h"
 
 #include "gimpcolorpanel.h"
+#include "gimpcompressioncombobox.h"
 #include "gimpdial.h"
 #include "gimpdnd.h"
 #include "gimpiconpicker.h"
@@ -142,6 +143,144 @@ gimp_prop_expanding_frame_new (GObject      *config,
 }
 
 
+/**********************/
+/*  boolean icon box  */
+/**********************/
+
+static void  gimp_prop_radio_button_callback (GtkWidget  *widget,
+                                              GObject    *config);
+static void  gimp_prop_radio_button_notify   (GObject    *config,
+                                              GParamSpec *param_spec,
+                                              GtkWidget  *button);
+
+GtkWidget *
+gimp_prop_boolean_icon_box_new (GObject     *config,
+                                const gchar *property_name,
+                                const gchar *true_icon,
+                                const gchar *false_icon,
+                                const gchar *true_tooltip,
+                                const gchar *false_tooltip)
+{
+  GParamSpec *param_spec;
+  GtkWidget  *box;
+  GtkWidget  *button;
+  GtkWidget  *first_button;
+  GtkWidget  *image;
+  GSList     *group = NULL;
+  gboolean    value;
+
+  g_return_val_if_fail (G_IS_OBJECT (config), NULL);
+  g_return_val_if_fail (property_name != NULL, NULL);
+
+  param_spec = check_param_spec_w (config, property_name,
+                                   G_TYPE_PARAM_BOOLEAN, G_STRFUNC);
+  if (! param_spec)
+    return NULL;
+
+  g_object_get (config,
+                property_name, &value,
+                NULL);
+
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+
+  button = first_button = gtk_radio_button_new (group);
+  group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
+  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  image = gtk_image_new_from_icon_name (true_icon, GTK_ICON_SIZE_MENU);
+  if (image)
+    {
+      gtk_container_add (GTK_CONTAINER (button), image);
+      gtk_widget_show (image);
+    }
+
+  gimp_help_set_help_data (button, true_tooltip, NULL);
+
+
+  g_object_set_data (G_OBJECT (button), "gimp-item-data",
+                     GINT_TO_POINTER (TRUE));
+
+  set_param_spec (G_OBJECT (button), NULL, param_spec);
+
+  g_signal_connect (button, "toggled",
+                    G_CALLBACK (gimp_prop_radio_button_callback),
+                    config);
+
+  button = gtk_radio_button_new (group);
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
+  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  image = gtk_image_new_from_icon_name (false_icon, GTK_ICON_SIZE_MENU);
+  if (image)
+    {
+      gtk_container_add (GTK_CONTAINER (button), image);
+      gtk_widget_show (image);
+    }
+
+  gimp_help_set_help_data (button, false_tooltip, NULL);
+
+  g_object_set_data (G_OBJECT (button), "gimp-item-data",
+                     GINT_TO_POINTER (FALSE));
+
+  set_param_spec (G_OBJECT (button), NULL, param_spec);
+
+  g_signal_connect (button, "toggled",
+                    G_CALLBACK (gimp_prop_radio_button_callback),
+                    config);
+
+  gimp_int_radio_group_set_active (GTK_RADIO_BUTTON (first_button), value);
+
+  connect_notify (config, property_name,
+                  G_CALLBACK (gimp_prop_radio_button_notify),
+                  button);
+
+  return box;
+}
+
+static void
+gimp_prop_radio_button_callback (GtkWidget *widget,
+                                 GObject   *config)
+{
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+    {
+      GParamSpec *param_spec;
+      gint        value;
+      gint        v;
+
+      param_spec = get_param_spec (G_OBJECT (widget));
+      if (! param_spec)
+        return;
+
+      value = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget),
+                                                  "gimp-item-data"));
+
+      g_object_get (config, param_spec->name, &v, NULL);
+
+      if (v != value)
+        g_object_set (config, param_spec->name, value, NULL);
+    }
+}
+
+static void
+gimp_prop_radio_button_notify (GObject    *config,
+                               GParamSpec *param_spec,
+                               GtkWidget  *button)
+{
+  gint value;
+
+  g_object_get (config,
+                param_spec->name, &value,
+                NULL);
+
+  gimp_int_radio_group_set_active (GTK_RADIO_BUTTON (button), value);
+}
+
+
 /********************/
 /*  layer mode box  */
 /********************/
@@ -367,22 +506,26 @@ gimp_prop_scale_button_callback (GtkWidget *button,
                                  GObject   *config)
 {
   GParamSpec *param_spec;
+  gdouble     v;
 
   param_spec = get_param_spec (G_OBJECT (button));
   if (! param_spec)
     return;
 
-  g_signal_handlers_block_by_func (config,
-                                   gimp_prop_scale_button_notify,
-                                   button);
+  g_object_get (config, param_spec->name, &v, NULL);
 
-  g_object_set (config,
-                param_spec->name, value,
-                NULL);
+  if (v != value)
+    {
+      g_signal_handlers_block_by_func (config,
+                                       gimp_prop_scale_button_notify,
+                                       button);
 
-  g_signal_handlers_unblock_by_func (config,
-                                     gimp_prop_scale_button_notify,
-                                     button);
+      g_object_set (config, param_spec->name, value, NULL);
+
+      g_signal_handlers_unblock_by_func (config,
+                                         gimp_prop_scale_button_notify,
+                                         button);
+    }
 }
 
 static void
@@ -899,7 +1042,7 @@ gimp_prop_angle_range_dial_new (GObject     *config,
                                NULL, NULL);
 
   g_object_bind_property (config, clockwise_property_name,
-                          dial,   "clockwise",
+                          dial,   "clockwise-delta",
                           G_BINDING_BIDIRECTIONAL |
                           G_BINDING_SYNC_CREATE);
 
@@ -1536,7 +1679,9 @@ GtkWidget *
 gimp_prop_profile_combo_box_new (GObject      *config,
                                  const gchar  *property_name,
                                  GtkListStore *profile_store,
-                                 const gchar  *dialog_title)
+                                 const gchar  *dialog_title,
+                                 GObject      *profile_path_config,
+                                 const gchar  *profile_path_property_name)
 {
   GParamSpec *param_spec;
   GtkWidget  *dialog;
@@ -1564,6 +1709,11 @@ gimp_prop_profile_combo_box_new (GObject      *config,
   dialog = gimp_color_profile_chooser_dialog_new (dialog_title, NULL,
                                                   GTK_FILE_CHOOSER_ACTION_OPEN);
 
+  if (profile_path_config && profile_path_property_name)
+    gimp_color_profile_chooser_dialog_connect_path (dialog,
+                                                    profile_path_config,
+                                                    profile_path_property_name);
+
   if (G_IS_PARAM_SPEC_STRING (param_spec))
     {
       gchar *path;
@@ -1707,6 +1857,107 @@ gimp_prop_profile_combo_notify (GObject                  *config,
 }
 
 
+/***************************/
+/*  compression combo box  */
+/***************************/
+
+static void   gimp_prop_compression_combo_box_callback (GtkWidget  *combo,
+                                                        GObject    *config);
+static void   gimp_prop_compression_combo_box_notify   (GObject    *config,
+                                                        GParamSpec *param_spec,
+                                                        GtkWidget  *combo);
+
+GtkWidget *
+gimp_prop_compression_combo_box_new (GObject     *config,
+                                     const gchar *property_name)
+{
+  GParamSpec *param_spec;
+  GtkWidget  *combo;
+  gchar      *value;
+
+  param_spec = check_param_spec_w (config, property_name,
+                                   G_TYPE_PARAM_STRING, G_STRFUNC);
+  if (! param_spec)
+    return NULL;
+
+  combo = gimp_compression_combo_box_new ();
+
+  g_object_get (config,
+                property_name, &value,
+                NULL);
+
+  gimp_compression_combo_box_set_compression (
+    GIMP_COMPRESSION_COMBO_BOX (combo), value);
+  g_free (value);
+
+  set_param_spec (G_OBJECT (combo), combo, param_spec);
+
+  g_signal_connect (combo, "changed",
+                    G_CALLBACK (gimp_prop_compression_combo_box_callback),
+                    config);
+
+  connect_notify (config, property_name,
+                  G_CALLBACK (gimp_prop_compression_combo_box_notify),
+                  combo);
+
+  return combo;
+}
+
+static void
+gimp_prop_compression_combo_box_callback (GtkWidget *combo,
+                                          GObject   *config)
+{
+  GParamSpec *param_spec;
+  gchar      *compression;
+
+  param_spec = get_param_spec (G_OBJECT (combo));
+  if (! param_spec)
+    return;
+
+  compression = gimp_compression_combo_box_get_compression (
+    GIMP_COMPRESSION_COMBO_BOX (combo));
+
+  g_signal_handlers_block_by_func (config,
+                                   gimp_prop_compression_combo_box_notify,
+                                   combo);
+
+  g_object_set (config,
+                param_spec->name, compression,
+                NULL);
+
+  g_signal_handlers_unblock_by_func (config,
+                                     gimp_prop_compression_combo_box_notify,
+                                     combo);
+
+  g_free (compression);
+}
+
+static void
+gimp_prop_compression_combo_box_notify (GObject    *config,
+                                        GParamSpec *param_spec,
+                                        GtkWidget  *combo)
+{
+  gchar *value;
+
+  g_object_get (config,
+                param_spec->name, &value,
+                NULL);
+
+  g_signal_handlers_block_by_func (combo,
+                                   gimp_prop_compression_combo_box_callback,
+                                   config);
+
+  gimp_compression_combo_box_set_compression (
+    GIMP_COMPRESSION_COMBO_BOX (combo), value);
+
+  g_signal_handlers_unblock_by_func (combo,
+                                     gimp_prop_compression_combo_box_callback,
+                                     config);
+
+  g_free (value);
+}
+
+
 /*****************/
 /*  icon picker  */
 /*****************/
