@@ -35,6 +35,9 @@
 #include "gimp-intl.h"
 
 
+#define RELATIVE_CHANGE_SPEED 0.1
+
+
 enum
 {
   PROP_0,
@@ -54,6 +57,8 @@ typedef struct _GimpSpinScalePrivate GimpSpinScalePrivate;
 
 struct _GimpSpinScalePrivate
 {
+  gboolean         compact;
+
   gchar           *label;
   gchar           *label_text;
   gchar           *label_pattern;
@@ -163,6 +168,12 @@ gimp_spin_scale_class_init (GimpSpinScaleClass *klass)
                                    g_param_spec_string ("label", NULL, NULL,
                                                         NULL,
                                                         GIMP_PARAM_READWRITE));
+
+  gtk_widget_class_install_style_property (widget_class,
+                                           g_param_spec_boolean ("compact",
+                                                                 NULL, NULL,
+                                                                 FALSE,
+                                                                 GIMP_PARAM_READABLE));
 }
 
 static void
@@ -261,17 +272,21 @@ gimp_spin_scale_size_request (GtkWidget      *widget,
   GtkStyle             *style   = gtk_widget_get_style (widget);
   PangoContext         *context = gtk_widget_get_pango_context (widget);
   PangoFontMetrics     *metrics;
-  gint                  height;
 
   GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
   metrics = pango_context_get_metrics (context, style->font_desc,
                                        pango_context_get_language (context));
 
-  height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
-                         pango_font_metrics_get_descent (metrics));
+  if (! private->compact)
+    {
+      gint height;
 
-  requisition->height += height;
+      height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
+                             pango_font_metrics_get_descent (metrics));
+
+      requisition->height += height;
+    }
 
   if (private->label)
     {
@@ -299,6 +314,10 @@ gimp_spin_scale_style_set (GtkWidget *widget,
   GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
 
   g_clear_object (&private->layout);
+
+  gtk_widget_style_get (widget,
+                        "compact", &private->compact,
+                        NULL);
 }
 
 static PangoAttrList *
@@ -362,7 +381,8 @@ gimp_spin_scale_expose (GtkWidget      *widget,
   h = gdk_window_get_height (event->window);
 
   /* upper/lower halves highlight */
-  if (event->window == gtk_entry_get_text_window (GTK_ENTRY (widget)) &&
+  if (! private->compact                                              &&
+      event->window == gtk_entry_get_text_window (GTK_ENTRY (widget)) &&
       gtk_widget_get_sensitive (widget)                               &&
       (private->target == TARGET_UPPER || private->target == TARGET_LOWER))
     {
@@ -554,43 +574,71 @@ gimp_spin_scale_expose (GtkWidget      *widget,
 }
 
 static SpinScaleTarget
-gimp_spin_scale_get_target (GtkWidget *widget,
-                            gdouble    x,
-                            gdouble    y)
+gimp_spin_scale_get_target (GtkWidget      *widget,
+                            gdouble         x,
+                            gdouble         y,
+                            GdkEventButton *event)
 {
-  GtkAllocation  allocation;
-  PangoRectangle logical;
-  gint           layout_x;
-  gint           layout_y;
+  GimpSpinScalePrivate *private = GET_PRIVATE (widget);
+  GtkAllocation         allocation;
+  PangoRectangle        logical;
+  gint                  layout_x;
+  gint                  layout_y;
+
+  if (private->compact && ! event)
+    return TARGET_UPPER;
 
   gtk_widget_get_allocation (widget, &allocation);
   gtk_entry_get_layout_offsets (GTK_ENTRY (widget), &layout_x, &layout_y);
   pango_layout_get_pixel_extents (gtk_entry_get_layout (GTK_ENTRY (widget)),
                                   NULL, &logical);
 
-  if (x >= layout_x && x < layout_x + logical.width &&
-      y >= layout_y && y < layout_y + logical.height)
+  if (x >= layout_x && x < layout_x + logical.width  &&
+      y >= layout_y && y < layout_y + logical.height &&
+      (! private->compact                            ||
+       (gtk_widget_has_focus (widget)                &&
+        gdk_event_triggers_context_menu ((GdkEvent *) event))))
     {
       return TARGET_NUMBER;
     }
-  else if (y >= allocation.height / 2)
-    {
-      return TARGET_LOWER;
-    }
 
-  return TARGET_UPPER;
+  if (private->compact)
+    {
+      switch (event->button)
+        {
+        case 1:
+          if (event->state & GDK_SHIFT_MASK)
+            return TARGET_LOWER;
+          else
+            return TARGET_UPPER;
+
+        case 3:
+          return TARGET_LOWER;
+
+        default:
+          return TARGET_NUMBER;
+        }
+    }
+   else
+    {
+      if (y >= allocation.height / 2)
+        return TARGET_LOWER;
+      else
+        return TARGET_UPPER;
+    }
 }
 
 static void
-gimp_spin_scale_update_target (GtkWidget *widget,
-                               GdkWindow *window,
-                               gdouble    x,
-                               gdouble    y)
+gimp_spin_scale_update_target (GtkWidget      *widget,
+                               GdkWindow      *window,
+                               gdouble         x,
+                               gdouble         y,
+                               GdkEventButton *event)
 {
   GimpSpinScalePrivate *private = GET_PRIVATE (widget);
   SpinScaleTarget       target;
 
-  target = gimp_spin_scale_get_target (widget, x, y);
+  target = gimp_spin_scale_get_target (widget, x, y, event);
 
   if (target != private->target)
     {
@@ -666,7 +714,8 @@ gimp_spin_scale_get_limits (GimpSpinScale *scale,
 
 static void
 gimp_spin_scale_change_value (GtkWidget *widget,
-                              gdouble    x)
+                              gdouble    x,
+                              guint      state)
 {
   GimpSpinScalePrivate *private     = GET_PRIVATE (widget);
   GtkSpinButton        *spin_button = GTK_SPIN_BUTTON (widget);
@@ -688,17 +737,23 @@ gimp_spin_scale_change_value (GtkWidget *widget,
 
   if (private->relative_change)
     {
-      gdouble diff;
       gdouble step;
 
-      step = (upper - lower) / width / 10.0;
+      step = (upper - lower) / width * RELATIVE_CHANGE_SPEED;
 
       if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-        diff = x - (width - private->start_x);
+        step *= x - (width - private->start_x);
       else
-        diff = x - private->start_x;
+        step *= x - private->start_x;
 
-      value = (private->start_value + diff * step);
+      if (state & GDK_CONTROL_MASK)
+        {
+          gdouble page_inc = gtk_adjustment_get_page_increment (adjustment);
+
+          step = RINT (step / page_inc) * page_inc;
+        }
+
+      value = private->start_value + step;
     }
   else
     {
@@ -709,6 +764,13 @@ gimp_spin_scale_change_value (GtkWidget *widget,
         fraction = pow (fraction, private->gamma);
 
       value = fraction * (upper - lower) + lower;
+
+      if (state & GDK_CONTROL_MASK)
+        {
+          gdouble page_inc = gtk_adjustment_get_page_increment (adjustment);
+
+          value = RINT (value / page_inc) * page_inc;
+        }
     }
 
   digits = gtk_spin_button_get_digits (spin_button);
@@ -742,7 +804,7 @@ gimp_spin_scale_button_press (GtkWidget      *widget,
   if (event->window == gtk_entry_get_text_window (GTK_ENTRY (widget)))
     {
       gimp_spin_scale_update_target (widget, event->window,
-                                     event->x, event->y);
+                                     event->x, event->y, event);
 
       gtk_widget_queue_draw (widget);
 
@@ -753,7 +815,7 @@ gimp_spin_scale_button_press (GtkWidget      *widget,
 
           gtk_widget_grab_focus (widget);
 
-          gimp_spin_scale_change_value (widget, event->x);
+          gimp_spin_scale_change_value (widget, event->x, event->state);
 
           return TRUE;
 
@@ -795,7 +857,7 @@ gimp_spin_scale_button_release (GtkWidget      *widget,
        * gimp_spin_scale_motion_notify().
        */
       if (! private->pointer_warp)
-        gimp_spin_scale_change_value (widget, event->x);
+        gimp_spin_scale_change_value (widget, event->x, event->state);
 
       if (private->relative_change)
         {
@@ -807,7 +869,7 @@ gimp_spin_scale_button_release (GtkWidget      *widget,
 
       if (private->hover)
         gimp_spin_scale_update_target (widget, event->window,
-                                       event->x, event->y);
+                                       event->x, event->y, NULL);
       else
         gimp_spin_scale_clear_target (widget, event->window);
 
@@ -885,7 +947,7 @@ gimp_spin_scale_motion_notify (GtkWidget      *widget,
             private->start_x = private->pointer_warp_start_x;
         }
 
-      gimp_spin_scale_change_value (widget, event->x);
+      gimp_spin_scale_change_value (widget, event->x, event->state);
 
       if (private->relative_change)
         {
@@ -945,7 +1007,7 @@ gimp_spin_scale_motion_notify (GtkWidget      *widget,
       private->hover)
     {
       gimp_spin_scale_update_target (widget, event->window,
-                                     event->x, event->y);
+                                     event->x, event->y, NULL);
     }
 
   return FALSE;
