@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __APP_GIMP_WIDGETS_UTILS_H__
@@ -83,7 +83,7 @@ guint32           gimp_window_get_native_id        (GtkWindow            *window
 void              gimp_window_set_transient_for    (GtkWindow            *window,
                                                     guint32               parent_ID);
 void              gimp_widget_set_accel_help       (GtkWidget            *widget,
-                                                    GtkAction            *action);
+                                                    GimpAction           *action);
 
 const gchar     * gimp_get_message_icon_name       (GimpMessageSeverity   severity);
 gboolean          gimp_get_color_tag_color         (GimpColorTag          color_tag,
@@ -105,6 +105,9 @@ GtkWidget       * gimp_dock_with_window_new        (GimpDialogFactory    *factor
 GtkWidget       * gimp_tools_get_tool_options_gui  (GimpToolOptions      *tool_options);
 void              gimp_tools_set_tool_options_gui  (GimpToolOptions      *tool_options,
                                                     GtkWidget            *widget);
+void              gimp_tools_set_tool_options_gui_func
+                                                   (GimpToolOptions      *tool_options,
+                                                    GimpToolOptionsGUIFunc func);
 
 void              gimp_widget_flush_expose         (GtkWidget            *widget);
 
@@ -112,6 +115,8 @@ gboolean          gimp_widget_get_fully_opaque     (GtkWidget            *widget
 void              gimp_widget_set_fully_opaque     (GtkWidget            *widget,
                                                     gboolean              fully_opaque);
 
+void              gimp_gtk_container_clear         (GtkContainer         *container);
+
 const gchar     * gimp_print_event                 (const GdkEvent       *event);
 
 gboolean          gimp_color_profile_store_add_defaults
