@@ -175,6 +175,7 @@ xcf_load_image (Gimp     *gimp,
   gint                n_broken_layers         = 0;
   gint                n_broken_channels       = 0;
   GList              *broken_paths            = NULL;
+  GList              *group_layers            = NULL;
   GList              *syms;
   GList              *iter;
 
@@ -513,6 +514,16 @@ xcf_load_image (Gimp     *gimp,
 
       xcf_progress_update (info);
 
+      /* suspend layer-group size updates */
+      if (GIMP_IS_GROUP_LAYER (layer))
+        {
+          GimpGroupLayer *group = GIMP_GROUP_LAYER (layer);
+
+          group_layers = g_list_prepend (group_layers, group);
+
+          gimp_group_layer_suspend_resize (group, FALSE);
+        }
+
       /* add the layer to the image if its not the floating selection */
       if (layer != info->floating_sel)
         {
@@ -566,6 +577,15 @@ xcf_load_image (Gimp     *gimp,
         goto error;
     }
 
+  /* resume layer-group size updates, in reverse order */
+  for (iter = group_layers; iter; iter = g_list_next (iter))
+    {
+      GimpGroupLayer *group = iter->data;
+
+      gimp_group_layer_resume_resize (group, FALSE);
+    }
+  g_clear_pointer (&group_layers, g_list_free);
+
   if (broken_paths)
     {
       g_list_free_full (broken_paths, (GDestroyNotify) g_list_free);
@@ -650,12 +670,17 @@ xcf_load_image (Gimp     *gimp,
   return image;
 
  error:
-  if (broken_paths)
-    g_list_free_full (broken_paths, (GDestroyNotify) g_list_free);
-
   if (num_successful_elements == 0)
     goto hard_error;
 
+  g_clear_pointer (&group_layers, g_list_free);
+
+  if (broken_paths)
+    {
+      g_list_free_full (broken_paths, (GDestroyNotify) g_list_free);
+      broken_paths = NULL;
+    }
+
   gimp_message_literal (gimp, G_OBJECT (info->progress), GIMP_MESSAGE_WARNING,
                         _("This XCF file is corrupt!  I have loaded as much "
                           "of it as I can, but it is incomplete."));
@@ -667,12 +692,19 @@ xcf_load_image (Gimp     *gimp,
   return image;
 
  hard_error:
+  g_clear_pointer (&group_layers, g_list_free);
+
+  if (broken_paths)
+    {
+      g_list_free_full (broken_paths, (GDestroyNotify) g_list_free);
+      broken_paths = NULL;
+    }
+
   g_set_error_literal (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                        _("This XCF file is corrupt!  I could not even "
                          "salvage any partial image data from it."));
 
-  if (image)
-    g_object_unref (image);
+  g_clear_object (&image);
 
   return NULL;
 }
@@ -2008,7 +2040,17 @@ xcf_load_layer (XcfInfo    *info,
   return layer;
 
  error:
+  if (info->active_layer == layer)
+    info->active_layer = NULL;
+
+  if (info->floating_sel == layer)
+    info->floating_sel = NULL;
+
+  if (info->floating_sel_drawable == GIMP_DRAWABLE (layer))
+    info->floating_sel_drawable = NULL;
+
   g_object_unref (layer);
+
   return NULL;
 }
 
@@ -2070,7 +2112,15 @@ xcf_load_channel (XcfInfo   *info,
  error:
   /* don't unref the selection of a partially loaded XCF */
   if (channel != gimp_image_get_mask (image))
-    g_object_unref (channel);
+    {
+      if (info->active_channel == channel)
+        info->active_channel = NULL;
+
+      if (info->floating_sel_drawable == GIMP_DRAWABLE (channel))
+        info->floating_sel_drawable = NULL;
+
+      g_object_unref (channel);
+    }
 
   return NULL;
 }
@@ -2134,7 +2184,14 @@ xcf_load_layer_mask (XcfInfo   *info,
   return layer_mask;
 
  error:
+  if (info->active_channel == GIMP_CHANNEL (layer_mask))
+    info->active_channel = NULL;
+
+  if (info->floating_sel_drawable == GIMP_DRAWABLE (layer_mask))
+    info->floating_sel_drawable = NULL;
+
   g_object_unref (layer_mask);
+
   return NULL;
 }
 
