@@ -15,9 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
+#if !defined (__GIMP_H_INSIDE__) && !defined (GIMP_COMPILATION)
+#error "Only <libgimp/gimp.h> can be included directly."
+#endif
+
 #ifndef __GIMP_IMAGE_H__
 #define __GIMP_IMAGE_H__
 
@@ -37,54 +41,75 @@ guchar       * gimp_image_get_thumbnail_data      (gint32              image_ID,
                                                    gint               *height,
                                                    gint               *bpp);
 
-#ifndef GIMP_DISABLE_DEPRECATED
+GimpMetadata * gimp_image_get_metadata            (gint32              image_ID);
+gboolean       gimp_image_set_metadata            (gint32              image_ID,
+                                                   GimpMetadata       *metadata);
+
+GIMP_DEPRECATED_FOR(gimp_image_get_colormap)
 guchar       * gimp_image_get_cmap                (gint32              image_ID,
                                                    gint               *num_colors);
+GIMP_DEPRECATED_FOR(gimp_image_set_colormap)
 gboolean       gimp_image_set_cmap                (gint32              image_ID,
                                                    const guchar       *cmap,
                                                    gint                num_colors);
+GIMP_DEPRECATED_FOR(gimp_image_get_item_position)
 gint           gimp_image_get_layer_position      (gint32              image_ID,
                                                    gint32              layer_ID);
+GIMP_DEPRECATED_FOR(gimp_image_raise_item)
 gboolean       gimp_image_raise_layer             (gint32              image_ID,
                                                    gint32              layer_ID);
+GIMP_DEPRECATED_FOR(gimp_image_lower_item)
 gboolean       gimp_image_lower_layer             (gint32              image_ID,
                                                    gint32              layer_ID);
+GIMP_DEPRECATED_FOR(gimp_image_raise_item_to_top)
 gboolean       gimp_image_raise_layer_to_top      (gint32              image_ID,
                                                    gint32              layer_ID);
+GIMP_DEPRECATED_FOR(gimp_image_lower_item_to_bottom)
 gboolean       gimp_image_lower_layer_to_bottom   (gint32              image_ID,
                                                    gint32              layer_ID);
+GIMP_DEPRECATED_FOR(gimp_image_get_item_position)
 gint           gimp_image_get_channel_position    (gint32              image_ID,
                                                    gint32              channel_ID);
+GIMP_DEPRECATED_FOR(gimp_image_raise_item)
 gboolean       gimp_image_raise_channel           (gint32              image_ID,
                                                    gint32              channel_ID);
+GIMP_DEPRECATED_FOR(gimp_image_lower_item)
 gboolean       gimp_image_lower_channel           (gint32              image_ID,
                                                    gint32              channel_ID);
+GIMP_DEPRECATED_FOR(gimp_image_get_item_position)
 gint           gimp_image_get_vectors_position    (gint32              image_ID,
                                                    gint32              vectors_ID);
+GIMP_DEPRECATED_FOR(gimp_image_raise_item)
 gboolean       gimp_image_raise_vectors           (gint32              image_ID,
                                                    gint32              vectors_ID);
+GIMP_DEPRECATED_FOR(gimp_image_lower_item)
 gboolean       gimp_image_lower_vectors           (gint32              image_ID,
                                                    gint32              vectors_ID);
+GIMP_DEPRECATED_FOR(gimp_image_raise_item_to_top)
 gboolean       gimp_image_raise_vectors_to_top    (gint32              image_ID,
                                                    gint32              vectors_ID);
+GIMP_DEPRECATED_FOR(gimp_image_lower_item_to_bottom)
 gboolean       gimp_image_lower_vectors_to_bottom (gint32              image_ID,
                                                    gint32              vectors_ID);
+GIMP_DEPRECATED_FOR(gimp_image_get_parasite)
 GimpParasite * gimp_image_parasite_find           (gint32              image_ID,
                                                    const gchar        *name);
+GIMP_DEPRECATED_FOR(gimp_image_attach_parasite)
 gboolean       gimp_image_parasite_attach         (gint32              image_ID,
                                                    const GimpParasite *parasite);
+GIMP_DEPRECATED_FOR(gimp_image_detach_parasite)
 gboolean       gimp_image_parasite_detach         (gint32              image_ID,
                                                    const gchar        *name);
+GIMP_DEPRECATED_FOR(gimp_image_get_parasite_list)
 gboolean       gimp_image_parasite_list           (gint32              image_ID,
                                                    gint               *num_parasites,
                                                    gchar            ***parasites);
+GIMP_DEPRECATED_FOR(gimp_image_attach_parasite)
 gboolean       gimp_image_attach_new_parasite     (gint32              image_ID,
                                                    const gchar        *name,
                                                    gint                flags,
                                                    gint                size,
                                                    gconstpointer       data);
-#endif /* GIMP_DISABLE_DEPRECATED */
-
 
 G_END_DECLS
 
