@@ -28,7 +28,6 @@
 #include <lcms2.h>
 
 #include <gio/gio.h>
-#define GEGL_ITERATOR2_API
 #include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -205,7 +204,7 @@ gimp_color_transform_new (GimpColorProfile         *src_profile,
                                                           &error);
   if (! priv->src_space_format)
     {
-      g_printerr ("%s: error making format: %s\n",
+      g_printerr ("%s: error making src format: %s\n",
                   G_STRFUNC, error->message);
       g_clear_error (&error);
     }
@@ -216,7 +215,7 @@ gimp_color_transform_new (GimpColorProfile         *src_profile,
                                                            &error);
   if (! priv->dest_space_format)
     {
-      g_printerr ("%s: error making format: %s\n",
+      g_printerr ("%s: error making dest format: %s\n",
                   G_STRFUNC, error->message);
       g_clear_error (&error);
     }
