@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #if !defined (__GIMP_WIDGETS_H_INSIDE__) && !defined (GIMP_WIDGETS_COMPILATION)
@@ -101,6 +101,11 @@ void          gimp_int_combo_box_set_label       (GimpIntComboBox *combo_box,
                                                   const gchar     *label);
 const gchar * gimp_int_combo_box_get_label       (GimpIntComboBox *combo_box);
 
+void          gimp_int_combo_box_set_layout      (GimpIntComboBox       *combo_box,
+                                                  GimpIntComboBoxLayout  layout);
+GimpIntComboBoxLayout
+              gimp_int_combo_box_get_layout      (GimpIntComboBox       *combo_box);
+
 void          gimp_int_combo_box_set_sensitivity (GimpIntComboBox        *combo_box,
                                                   GimpIntSensitivityFunc  func,
                                                   gpointer                data,
