@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -140,7 +140,8 @@ gimp_table_attach_aligned (GtkTable    *table,
       GtkWidget *mnemonic_widget;
 
       label = gtk_label_new_with_mnemonic (label_text);
-      gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
+      gtk_label_set_xalign (GTK_LABEL (label), xalign);
+      gtk_label_set_yalign (GTK_LABEL (label), yalign);
       gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
       gtk_table_attach (table, label,
                         column, column + 1,
@@ -469,25 +470,27 @@ gimp_widget_track_monitor (GtkWidget *widget,
     track_monitor_hierarchy_changed (widget, NULL, track_data);
 }
 
+/**
+ * gimp_screen_get_color_profile:
+ * @screen:  a #GdkScreen
+ * @monitor: the monitor number
+ *
+ * This function returns the #GimpColorProfile of monitor number @monitor
+ * of @screen, or %NULL if there is no profile configured.
+ *
+ * Since: 2.10
+ *
+ * Return value: the monitor's #GimpColorProfile, or %NULL.
+ **/
 GimpColorProfile *
-gimp_widget_get_color_profile (GtkWidget *widget)
+gimp_screen_get_color_profile (GdkScreen *screen,
+                               gint       monitor)
 {
   GimpColorProfile *profile = NULL;
-  GdkScreen        *screen;
-  gint              monitor;
 
-  g_return_val_if_fail (widget == NULL || GTK_IS_WIDGET (widget), NULL);
-
-  if (widget)
-    {
-      screen  = gtk_widget_get_screen (widget);
-      monitor = gimp_widget_get_monitor (widget);
-    }
-  else
-    {
-      screen  = gdk_screen_get_default ();
-      monitor = 0;
-    }
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  g_return_val_if_fail (monitor >= 0, NULL);
+  g_return_val_if_fail (monitor < gdk_screen_get_n_monitors (screen), NULL);
 
 #if defined GDK_WINDOWING_X11
   {
@@ -578,6 +581,28 @@ gimp_widget_get_color_profile (GtkWidget *widget)
   return profile;
 }
 
+GimpColorProfile *
+gimp_widget_get_color_profile (GtkWidget *widget)
+{
+  GdkScreen *screen;
+  gint       monitor;
+
+  g_return_val_if_fail (widget == NULL || GTK_IS_WIDGET (widget), NULL);
+
+  if (widget)
+    {
+      screen  = gtk_widget_get_screen (widget);
+      monitor = gimp_widget_get_monitor (widget);
+    }
+  else
+    {
+      screen  = gdk_screen_get_default ();
+      monitor = 0;
+    }
+
+  return gimp_screen_get_color_profile (screen, monitor);
+}
+
 static GimpColorProfile *
 get_display_profile (GtkWidget       *widget,
                      GimpColorConfig *config)
