@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Comment:
  *  CML is the abbreviation of Coupled-Map Lattice that is a model of
@@ -461,7 +461,7 @@ query (void)
                           "Make an image of Coupled-Map Lattice (CML). CML is "
                           "a kind of Cellula Automata on continuous (value) "
                           "domain. In GIMP_RUN_NONINTERACTIVE, the name of a "
-                          "prameter file is passed as the 4th arg. You can "
+                          "parameter file is passed as the 4th arg. You can "
                           "control CML_explorer via parameter file.",
                           /*  Or do you want to call me with over 50 args? */
                           "Shuji Narazaki (narazaki@InetQ.or.jp); "
@@ -492,6 +492,7 @@ run (const gchar      *name,
   drawable_id = param[2].data.d_drawable;
 
   INIT_I18N ();
+  gegl_init (NULL, NULL);
 
   *nreturn_vals = 1;
   *return_vals = values;
@@ -519,7 +520,6 @@ run (const gchar      *name,
       break;
     }
 
-  gimp_tile_cache_ntiles (TILE_CACHE_SIZE);
   status = CML_main_function (FALSE);
 
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
@@ -538,28 +538,27 @@ run (const gchar      *name,
 static GimpPDBStatusType
 CML_main_function (gboolean preview_p)
 {
-  GimpDrawable *drawable = NULL;
-  GimpPixelRgn  dest_rgn, src_rgn;
-  guchar    *dest_buffer = NULL;
-  guchar    *src_buffer  = NULL;
-  gint       x, y;
-  gint       dx, dy;
-  gboolean   dest_has_alpha = FALSE;
-  gboolean   dest_is_gray   = FALSE;
-  gboolean   src_has_alpha  = FALSE;
-  gboolean   src_is_gray    = FALSE;
-  gint       total, processed = 0;
-  gint       keep_height = 1;
-  gint       cell_num, width_by_pixel, height_by_pixel;
-  gint       index;
-  gint       src_bpp, src_bpl;
-  gint       dest_bpp, dest_bpl;
-  gdouble   *hues, *sats, *vals;
-  gdouble   *newh, *news, *newv;
-  gdouble   *haux, *saux, *vaux;
-
-  /* open THE drawable */
-  drawable = gimp_drawable_get (drawable_id);
+  GeglBuffer *src_buffer;
+  GeglBuffer *dest_buffer;
+  const Babl *src_format;
+  const Babl *dest_format;
+  guchar     *dest_buf = NULL;
+  guchar     *src_buf  = NULL;
+  gint        x, y;
+  gint        dx, dy;
+  gboolean    dest_has_alpha = FALSE;
+  gboolean    dest_is_gray   = FALSE;
+  gboolean    src_has_alpha  = FALSE;
+  gboolean    src_is_gray    = FALSE;
+  gint        total, processed = 0;
+  gint        keep_height = 1;
+  gint        cell_num, width_by_pixel, height_by_pixel;
+  gint        index;
+  gint        src_bpp, src_bpl;
+  gint        dest_bpp, dest_bpl;
+  gdouble    *hues, *sats, *vals;
+  gdouble    *newh, *news, *newv;
+  gdouble    *haux, *saux, *vaux;
 
   if (! gimp_drawable_mask_intersect (drawable_id,
                                       &x, &y,
@@ -567,11 +566,31 @@ CML_main_function (gboolean preview_p)
     return GIMP_PDB_SUCCESS;
 
   src_has_alpha = dest_has_alpha = gimp_drawable_has_alpha (drawable_id);
-  src_is_gray = dest_is_gray = gimp_drawable_is_gray (drawable_id);
-  src_bpp = dest_bpp = (src_is_gray ? 1 : 3) + (src_has_alpha ? 1 : 0);
+  src_is_gray   = dest_is_gray   = gimp_drawable_is_gray (drawable_id);
+
+  if (src_is_gray)
+    {
+      if (src_has_alpha)
+        src_format = babl_format ("Y'A u8");
+      else
+        src_format = babl_format ("Y' u8");
+    }
+  else
+    {
+      if (src_has_alpha)
+        src_format = babl_format ("R'G'B'A u8");
+      else
+        src_format = babl_format ("R'G'B' u8");
+    }
+
+  dest_format = src_format;
+
+  src_bpp = dest_bpp = babl_format_get_bytes_per_pixel (src_format);
 
   if (preview_p)
     {
+      dest_format = babl_format ("R'G'B' u8");
+
       dest_has_alpha = FALSE;
       dest_bpp       = 3;
 
@@ -580,12 +599,15 @@ CML_main_function (gboolean preview_p)
       if (height_by_pixel > PREVIEW_HEIGHT)
         height_by_pixel = PREVIEW_HEIGHT;
     }
+
   dest_bpl = width_by_pixel * dest_bpp;
   src_bpl = width_by_pixel * src_bpp;
   cell_num = (width_by_pixel - 1)/ VALS.scale + 1;
   total = height_by_pixel * width_by_pixel;
+
   if (total < 1)
     return GIMP_PDB_EXECUTION_ERROR;
+
   keep_height = VALS.scale;
 
   /* configure reusable memories */
@@ -595,6 +617,7 @@ CML_main_function (gboolean preview_p)
       mem_chank0_size = 9 * cell_num * sizeof (gdouble);
       mem_chank0 = (gdouble *) g_malloc (mem_chank0_size);
     }
+
   hues = mem_chank0;
   sats = mem_chank0 + cell_num;
   vals = mem_chank0 + 2 * cell_num;
@@ -611,7 +634,7 @@ CML_main_function (gboolean preview_p)
       mem_chank1_size = src_bpl * keep_height;
       mem_chank1 = (guchar *) g_malloc (mem_chank1_size);
     }
-  src_buffer = mem_chank1;
+  src_buf = mem_chank1;
 
   if (mem_chank2_size < dest_bpl * keep_height)
     {
@@ -619,16 +642,12 @@ CML_main_function (gboolean preview_p)
       mem_chank2_size = dest_bpl * keep_height;
       mem_chank2 = (guchar *) g_malloc (mem_chank2_size);
     }
-  dest_buffer = mem_chank2;
+  dest_buf = mem_chank2;
 
   if (! preview_p)
-    gimp_pixel_rgn_init (&dest_rgn, drawable, x, y,
-                         width_by_pixel, height_by_pixel,
-                         TRUE, TRUE);
+    dest_buffer = gimp_drawable_get_shadow_buffer (drawable_id);
 
-  gimp_pixel_rgn_init (&src_rgn, drawable, x, y,
-                       width_by_pixel, height_by_pixel,
-                       FALSE, FALSE);
+  src_buffer = gimp_drawable_get_buffer (drawable_id);
 
   gr = g_rand_new ();
   if (VALS.initial_value == CML_INITIAL_RANDOM_FROM_SEED)
@@ -660,6 +679,7 @@ CML_main_function (gboolean preview_p)
           haux [index] = VALS.hue.power;
           break;
         }
+
       switch (VALS.sat.arrange)
         {
         case RAND_POWER0:
@@ -684,6 +704,7 @@ CML_main_function (gboolean preview_p)
           saux [index] = VALS.sat.power;
           break;
         }
+
       switch (VALS.val.arrange)
         {
         case RAND_POWER0:
@@ -708,6 +729,7 @@ CML_main_function (gboolean preview_p)
           vaux [index] = VALS.val.power;
           break;
         }
+
       switch (VALS.initial_value)
         {
         case 0:
@@ -750,8 +772,10 @@ CML_main_function (gboolean preview_p)
           int   rgbi[3];
           int   i;
 
-          gimp_pixel_rgn_get_pixel (&src_rgn, buffer,
-                                    x + (index * VALS.scale), y);
+          gegl_buffer_sample (src_buffer, x + (index * VALS.scale), y, NULL,
+                              buffer, src_format,
+                              GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
+
           for (i = 0; i < 3; i++) rgbi[i] = buffer[i];
           gimp_rgb_to_hsv_int (rgbi, rgbi + 1, rgbi + 2);
           hues[index] = (gdouble) rgbi[0] / (gdouble) 255;
@@ -780,8 +804,13 @@ CML_main_function (gboolean preview_p)
       if ((VALS.hue.function == CML_KEEP_VALUES) ||
           (VALS.sat.function == CML_KEEP_VALUES) ||
           (VALS.val.function == CML_KEEP_VALUES))
-        gimp_pixel_rgn_get_rect (&src_rgn, src_buffer,
-                                 x, y + dy, width_by_pixel, keep_height);
+        {
+          gegl_buffer_get (src_buffer,
+                           GEGL_RECTANGLE (x, y + dy,
+                                           width_by_pixel, keep_height), 1.0,
+                           src_format, src_buf,
+                           GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+        }
 
       CML_compute_next_step (cell_num,
                              &hues, &sats, &vals,
@@ -813,8 +842,8 @@ CML_main_function (gboolean preview_p)
                     int i;
 
                     for (i = 0; i < src_bpp; i++)
-                      rgbi[i] = src_buffer[offset_y * src_bpl
-                                           + (dx * VALS.scale + offset_x) * src_bpp + i];
+                      rgbi[i] = src_buf[offset_y * src_bpl
+                                        + (dx * VALS.scale + offset_x) * src_bpp + i];
                     if (src_is_gray && (VALS.val.function == CML_KEEP_VALUES))
                       {
                         b = rgbi[0];
@@ -835,21 +864,21 @@ CML_main_function (gboolean preview_p)
 
                 if (dest_is_gray)
                   {
-                    dest_buffer[dest_offset++] = b;
+                    dest_buf[dest_offset++] = b;
                     if (preview_p)
                       {
-                        dest_buffer[dest_offset++] = b;
-                        dest_buffer[dest_offset++] = b;
+                        dest_buf[dest_offset++] = b;
+                        dest_buf[dest_offset++] = b;
                       }
                   }
                 else
                   {
-                    dest_buffer[dest_offset++] = r;
-                    dest_buffer[dest_offset++] = g;
-                    dest_buffer[dest_offset++] = b;
+                    dest_buf[dest_offset++] = r;
+                    dest_buf[dest_offset++] = g;
+                    dest_buf[dest_offset++] = b;
                   }
                 if (dest_has_alpha)
-                  dest_buffer[dest_offset] = 255;
+                  dest_buf[dest_offset] = 255;
 
                 if ((!preview_p) &&
                     (++processed % (total / PROGRESS_UPDATE_NUM + 1)) == 0)
@@ -858,16 +887,26 @@ CML_main_function (gboolean preview_p)
         }
 
       if (preview_p)
-        gimp_preview_area_draw (GIMP_PREVIEW_AREA (preview),
-                                0, dy,
-                                width_by_pixel, keep_height,
-                                GIMP_RGB_IMAGE,
-                                dest_buffer,
-                                dest_bpl);
+        {
+          gimp_preview_area_draw (GIMP_PREVIEW_AREA (preview),
+                                  0, dy,
+                                  width_by_pixel, keep_height,
+                                  GIMP_RGB_IMAGE,
+                                  dest_buf,
+                                  dest_bpl);
+        }
       else
-        gimp_pixel_rgn_set_rect (&dest_rgn, dest_buffer, x, y + dy,
-                                 width_by_pixel, keep_height);
+        {
+          gegl_buffer_set (dest_buffer,
+                           GEGL_RECTANGLE (x, y + dy,
+                                           width_by_pixel, keep_height), 0,
+                           dest_format, dest_buf,
+                           GEGL_AUTO_ROWSTRIDE);
+        }
     }
+
+  g_object_unref (src_buffer);
+
   if (preview_p)
     {
       gtk_widget_queue_draw (preview);
@@ -875,11 +914,12 @@ CML_main_function (gboolean preview_p)
   else
     {
       gimp_progress_update (1.0);
-      gimp_drawable_flush (drawable);
-      gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
-      gimp_drawable_update (drawable->drawable_id,
+
+      g_object_unref (dest_buffer);
+
+      gimp_drawable_merge_shadow (drawable_id, TRUE);
+      gimp_drawable_update (drawable_id,
                             x, y, width_by_pixel, height_by_pixel);
-      gimp_drawable_detach (drawable);
     }
 
   g_rand_free (gr);
@@ -1186,8 +1226,8 @@ CML_explorer_dialog (void)
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                            _("_Cancel"), GTK_RESPONSE_CANCEL,
+                            _("_OK"),     GTK_RESPONSE_OK,
 
                             NULL);
 
@@ -1266,7 +1306,7 @@ CML_explorer_dialog (void)
   gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
   gtk_widget_show (bbox);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_OPEN);
+  button = gtk_button_new_with_mnemonic (_("_Open"));
   gtk_container_add (GTK_CONTAINER (bbox), button);
   gtk_widget_show (button);
 
@@ -1274,7 +1314,7 @@ CML_explorer_dialog (void)
                     G_CALLBACK (CML_load_from_file_callback),
                     &VALS);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
+  button = gtk_button_new_with_mnemonic (_("_Save"));
   gtk_container_add (GTK_CONTAINER (bbox), button);
   gtk_widget_show (button);
 
@@ -1853,7 +1893,7 @@ function_graph_new (GtkWidget *widget,
                             gtk_widget_get_toplevel (widget), 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
-                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                            _("_Close"), GTK_RESPONSE_CLOSE,
 
                             NULL);
 
@@ -1983,8 +2023,8 @@ CML_save_to_file_callback (GtkWidget *widget,
                                      GTK_WINDOW (gtk_widget_get_toplevel (widget)),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
 
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                     GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
+                                     _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("_Save"),   GTK_RESPONSE_OK,
 
                                      NULL);
 
@@ -2116,8 +2156,8 @@ CML_load_from_file_callback (GtkWidget *widget,
                                      GTK_WINDOW (gtk_widget_get_toplevel (widget)),
                                      GTK_FILE_CHOOSER_ACTION_OPEN,
 
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                     GTK_STOCK_OPEN,   GTK_RESPONSE_OK,
+                                     _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("_Open"),   GTK_RESPONSE_OK,
 
                                      NULL);
 
