@@ -228,6 +228,9 @@ static gboolean          gui_single                 (void);
 
 static gboolean          gui_multi                  (void);
 
+static void              reverse_order_toggled      (GtkToggleButton *reverse_order,
+                                                     GtkButton       *layers_as_pages);
+
 static void              choose_file_call           (GtkWidget       *browse_button,
                                                      gpointer         file_entry);
 
@@ -257,6 +260,15 @@ static void              drawText                   (gint32           text_id,
                                                      gdouble          x_res,
                                                      gdouble          y_res);
 
+static gboolean          draw_layer                 (gint32          *layers,
+                                                     gint             n_layers,
+                                                     gint             j,
+                                                     cairo_t         *cr,
+                                                     gdouble          x_res,
+                                                     gdouble          y_res,
+                                                     const gchar     *name,
+                                                     GError         **error);
+
 static gboolean     dnd_remove = TRUE;
 static PdfMultiPage multi_page;
 
@@ -310,8 +322,8 @@ query (void)
     { GIMP_PDB_INT32,    "vectorize",       "Convert bitmaps to vector graphics where possible. TRUE or FALSE" },
     { GIMP_PDB_INT32,    "ignore-hidden",   "Omit hidden layers and layers with zero opacity. TRUE or FALSE" },
     { GIMP_PDB_INT32,    "apply-masks",     "Apply layer masks before saving. TRUE or FALSE (Keeping them will not change the output)" },
-    { GIMP_PDB_INT32,    "layers-as-pages", "Layers as pages. TRUE or FALSE" },
-    { GIMP_PDB_INT32,    "reverse-order",   "Reverse the pages order. TRUE or FALSE" }
+    { GIMP_PDB_INT32,    "layers-as-pages", "Layers as pages (bottom layers first). TRUE or FALSE" },
+    { GIMP_PDB_INT32,    "reverse-order",   "Reverse the pages order (top layers first). TRUE or FALSE" }
   };
 
   static GimpParamDef save_multi_args[] =
@@ -577,123 +589,24 @@ run (const gchar      *name,
       /* Now, we should loop over the layers of each image */
       for (j = 0; j < n_layers; j++)
         {
-          gint32           layer_ID;
-          gint32           mask_ID    = -1;
-          cairo_surface_t *mask_image = NULL;
-          gdouble          opacity;
-          gboolean         single_color;
-          gint             x, y;
-
-          if (optimize.reverse_order && optimize.layers_as_pages)
-            layer_ID = layers [j];
-          else
-            layer_ID = layers [n_layers - j - 1];
-
-          opacity = gimp_layer_get_opacity (layer_ID) / 100.0;
-
-          if ((gimp_item_get_visible (layer_ID) && opacity > 0.0) ||
-              ! optimize.ignore_hidden)
+          if (! draw_layer (layers, n_layers, j, cr, x_res, y_res, name, &error))
             {
-              mask_ID = gimp_layer_get_mask (layer_ID);
-              if (mask_ID != -1)
-                {
-                  mask_image = get_cairo_surface (mask_ID, TRUE,
-                                                  &error);
-                  if (error != NULL)
-                    {
-                      *nreturn_vals = 2;
+              *nreturn_vals = 2;
 
-                      /* free the resources */
-                      cairo_surface_destroy (pdf_file);
-                      cairo_destroy (cr);
-                      fclose (fp);
+              /* free the resources */
+              g_free (layers);
+              cairo_surface_destroy (pdf_file);
+              cairo_destroy (cr);
+              fclose (fp);
 
-                      values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+              values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
-                      values[1].type          = GIMP_PDB_STRING;
-                      values[1].data.d_string = error->message;
-                      return;
-                    }
-                }
-              gimp_drawable_offsets (layer_ID, &x, &y);
-
-              if (! gimp_item_is_text_layer (layer_ID))
-                {
-                  /* For raster layers */
-
-                  GimpRGB layer_color;
-
-                  layer_color = get_layer_color (layer_ID, &single_color);
-
-                  cairo_rectangle (cr, x, y,
-                                   gimp_drawable_width (layer_ID),
-                                   gimp_drawable_height (layer_ID));
-
-                  if (optimize.vectorize && single_color)
-                    {
-                      cairo_set_source_rgba (cr,
-                                             layer_color.r,
-                                             layer_color.g,
-                                             layer_color.b,
-                                             layer_color.a * opacity);
-                      if (mask_ID != -1)
-                        cairo_mask_surface (cr, mask_image, x, y);
-                      else
-                        cairo_fill (cr);
-                    }
-                  else
-                    {
-                      cairo_surface_t *layer_image;
-
-                      layer_image = get_cairo_surface (layer_ID, FALSE,
-                                                       &error);
-                      if (error != NULL)
-                        {
-                          *nreturn_vals = 2;
-
-                          /* free the resources */
-                          cairo_surface_destroy (pdf_file);
-                          cairo_destroy (cr);
-                          fclose (fp);
-
-                          values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
-
-                          values[1].type          = GIMP_PDB_STRING;
-                          values[1].data.d_string = error->message;
-                          return;
-                        }
-
-                      cairo_clip (cr);
-
-                      cairo_set_source_surface (cr, layer_image, x, y);
-                      cairo_push_group (cr);
-                      cairo_paint_with_alpha (cr, opacity);
-                      cairo_pop_group_to_source (cr);
-
-                      if (mask_ID != -1)
-                        cairo_mask_surface (cr, mask_image, x, y);
-                      else
-                        cairo_paint (cr);
-
-                      cairo_reset_clip (cr);
-
-                      cairo_surface_destroy (layer_image);
-                    }
-                }
-              else
-                {
-                  /* For text layers */
-                  drawText (layer_ID, opacity, cr, x_res, y_res);
-                }
-              /* draw new page if "layers as pages" option is checked */
-              if (optimize.layers_as_pages &&
-                  g_strcmp0 (name, SAVE2_PROC) == 0)
-                cairo_show_page (cr);
+              values[1].type          = GIMP_PDB_STRING;
+              values[1].data.d_string = error->message;
+              return;
             }
-          /* We are done with the layer - time to free some resources */
-          if (mask_ID != -1)
-            cairo_surface_destroy (mask_image);
         }
+      g_free (layers);
 
       /* We are done with this image - Show it!
        * Unless that's a multi-page to avoid blank page at the end
@@ -892,6 +805,8 @@ gui_single (void)
   GtkWidget *apply_c;
   GtkWidget *layers_as_pages_c;
   GtkWidget *reverse_order_c;
+  GtkWidget *frame;
+  gchar     *text;
   gboolean   run;
   gint32     n_layers;
 
@@ -905,43 +820,54 @@ gui_single (void)
 
   gtk_container_set_border_width (GTK_CONTAINER (window), 12);
 
-  ignore_hidden_c = gtk_check_button_new_with_label (_("Omit hidden layers and layers with zero opacity"));
+  ignore_hidden_c = gtk_check_button_new_with_mnemonic (_("_Omit hidden layers and layers with zero opacity"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ignore_hidden_c),
                                 optimize.ignore_hidden);
   gtk_box_pack_end (GTK_BOX (vbox), ignore_hidden_c, TRUE, TRUE, 0);
 
-  vectorize_c = gtk_check_button_new_with_label (_("Convert bitmaps to vector graphics where possible"));
+  vectorize_c = gtk_check_button_new_with_mnemonic (_("Convert _bitmaps to vector graphics where possible"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vectorize_c),
                                 optimize.vectorize);
   gtk_box_pack_end (GTK_BOX (vbox), vectorize_c, TRUE, TRUE, 0);
 
-  apply_c = gtk_check_button_new_with_label (_("Apply layer masks before saving"));
+  apply_c = gtk_check_button_new_with_mnemonic (_("_Apply layer masks before saving"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (apply_c),
                                 optimize.apply_masks);
   gtk_box_pack_end (GTK_BOX (vbox), apply_c, TRUE, TRUE, 0);
   gimp_help_set_help_data (apply_c, _("Keeping the masks will not change the output"), NULL);
 
-  reverse_order_c = gtk_check_button_new_with_label (_("Reverse the pages order"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reverse_order_c),
-                                optimize.reverse_order);
-  gtk_box_pack_end (GTK_BOX (vbox), reverse_order_c, TRUE, TRUE, 0);
+  /* Frame for multi-page from layers. */
+  frame = gtk_frame_new (NULL);
+  gtk_box_pack_end (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
-  layers_as_pages_c = gtk_check_button_new_with_label (_("Layers as pages"));
+  text = g_strdup_printf (_("_Layers as pages (%s)"),
+                          optimize.reverse_order ?
+                          _("top layers first") : _("bottom layers first"));
+  layers_as_pages_c = gtk_check_button_new_with_mnemonic (text);
+  g_free (text);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (layers_as_pages_c),
                                 optimize.layers_as_pages);
-  gimp_image_get_layers (multi_page.images[0], &n_layers);
-  gtk_box_pack_end (GTK_BOX (vbox), layers_as_pages_c, TRUE, TRUE, 0);
+  gtk_frame_set_label_widget (GTK_FRAME (frame), layers_as_pages_c);
+  g_free (gimp_image_get_layers (multi_page.images[0], &n_layers));
+
+  reverse_order_c = gtk_check_button_new_with_mnemonic (_("_Reverse the pages order"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reverse_order_c),
+                                optimize.reverse_order);
+  gtk_container_add (GTK_CONTAINER (frame), reverse_order_c);
 
   if (n_layers <= 1)
-  {
-    gtk_widget_set_sensitive(layers_as_pages_c, FALSE);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (layers_as_pages_c),
-                                  FALSE);
-  }
+    {
+      gtk_widget_set_sensitive (layers_as_pages_c, FALSE);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (layers_as_pages_c),
+                                    FALSE);
+    }
 
   g_object_bind_property (layers_as_pages_c, "active",
                           reverse_order_c,  "sensitive",
                           G_BINDING_SYNC_CREATE);
+  g_signal_connect (G_OBJECT (reverse_order_c), "toggled",
+                    G_CALLBACK (reverse_order_toggled),
+                    layers_as_pages_c);
 
   gtk_widget_show_all (window);
 
@@ -1054,17 +980,17 @@ gui_multi (void)
 
   gtk_box_pack_start (GTK_BOX (vbox), h_box, FALSE, FALSE, 0);
 
-  ignore_hidden_c = gtk_check_button_new_with_label (_("Omit hidden layers and layers with zero opacity"));
+  ignore_hidden_c = gtk_check_button_new_with_mnemonic (_("_Omit hidden layers and layers with zero opacity"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ignore_hidden_c),
                                 optimize.ignore_hidden);
   gtk_box_pack_end (GTK_BOX (vbox), ignore_hidden_c, FALSE, FALSE, 0);
 
-  vectorize_c = gtk_check_button_new_with_label (_("Convert bitmaps to vector graphics where possible"));
+  vectorize_c = gtk_check_button_new_with_mnemonic (_("Convert _bitmaps to vector graphics where possible"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vectorize_c),
                                 optimize.vectorize);
   gtk_box_pack_end (GTK_BOX (vbox), vectorize_c, FALSE, FALSE, 0);
 
-  apply_c = gtk_check_button_new_with_label (_("Apply layer masks before saving"));
+  apply_c = gtk_check_button_new_with_mnemonic (_("_Apply layer masks before saving"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (apply_c),
                                 optimize.apply_masks);
   gtk_box_pack_end (GTK_BOX (vbox), apply_c, FALSE, FALSE, 0);
@@ -1107,6 +1033,19 @@ gui_multi (void)
   return run;
 }
 
+static void
+reverse_order_toggled (GtkToggleButton *reverse_order,
+                       GtkButton       *layers_as_pages)
+{
+  gchar *text;
+
+  text = g_strdup_printf (_("Layers as pages (%s)"),
+                          gtk_toggle_button_get_active (reverse_order) ?
+                          _("top layers first") : _("bottom layers first"));
+  gtk_button_set_label (layers_as_pages, text);
+  g_free (text);
+}
+
 /* A function that is called when the button for browsing for file
  * locations was clicked
  */
@@ -1723,3 +1662,130 @@ drawText (gint32    text_id,
 
   cairo_restore (cr);
 }
+
+static gboolean
+draw_layer (gint32       *layers,
+            gint          n_layers,
+            gint          j,
+            cairo_t      *cr,
+            gdouble       x_res,
+            gdouble       y_res,
+            const gchar  *name,
+            GError      **error)
+{
+  gint32 layer_ID;
+
+  if (optimize.reverse_order && optimize.layers_as_pages)
+    layer_ID = layers [j];
+  else
+    layer_ID = layers [n_layers - j - 1];
+
+  if (gimp_item_is_group (layer_ID))
+    {
+      gint *children;
+      gint  children_num;
+      gint  i;
+
+      children = gimp_item_get_children (layer_ID, &children_num);
+      for (i = 0; i < children_num; i++)
+        {
+          if (! draw_layer (children, children_num, i,
+                            cr, x_res, y_res, name, error))
+            {
+              g_free (children);
+              return FALSE;
+            }
+        }
+      g_free (children);
+    }
+  else
+    {
+      cairo_surface_t *mask_image = NULL;
+      gint32           mask_ID    = -1;
+      gdouble          opacity;
+
+      opacity = gimp_layer_get_opacity (layer_ID) / 100.0;
+
+      if ((gimp_item_get_visible (layer_ID) && opacity > 0.0) ||
+          ! optimize.ignore_hidden)
+        {
+          gint x, y;
+
+          mask_ID = gimp_layer_get_mask (layer_ID);
+          if (mask_ID != -1)
+            {
+              mask_image = get_cairo_surface (mask_ID, TRUE, error);
+
+              if (*error)
+                return FALSE;
+            }
+          gimp_drawable_offsets (layer_ID, &x, &y);
+
+          if (! gimp_item_is_text_layer (layer_ID))
+            {
+              /* For raster layers */
+
+              GimpRGB  layer_color;
+              gboolean single_color = FALSE;
+
+              layer_color = get_layer_color (layer_ID, &single_color);
+
+              cairo_rectangle (cr, x, y,
+                               gimp_drawable_width (layer_ID),
+                               gimp_drawable_height (layer_ID));
+
+              if (optimize.vectorize && single_color)
+                {
+                  cairo_set_source_rgba (cr,
+                                         layer_color.r,
+                                         layer_color.g,
+                                         layer_color.b,
+                                         layer_color.a * opacity);
+                  if (mask_ID != -1)
+                    cairo_mask_surface (cr, mask_image, x, y);
+                  else
+                    cairo_fill (cr);
+                }
+              else
+                {
+                  cairo_surface_t *layer_image;
+
+                  layer_image = get_cairo_surface (layer_ID, FALSE, error);
+
+                  if (*error)
+                    return FALSE;
+
+                  cairo_clip (cr);
+
+                  cairo_set_source_surface (cr, layer_image, x, y);
+                  cairo_push_group (cr);
+                  cairo_paint_with_alpha (cr, opacity);
+                  cairo_pop_group_to_source (cr);
+
+                  if (mask_ID != -1)
+                    cairo_mask_surface (cr, mask_image, x, y);
+                  else
+                    cairo_paint (cr);
+
+                  cairo_reset_clip (cr);
+
+                  cairo_surface_destroy (layer_image);
+                }
+            }
+          else
+            {
+              /* For text layers */
+              drawText (layer_ID, opacity, cr, x_res, y_res);
+            }
+          /* draw new page if "layers as pages" option is checked */
+          if (optimize.layers_as_pages &&
+              g_strcmp0 (name, SAVE2_PROC) == 0)
+            cairo_show_page (cr);
+        }
+      /* We are done with the layer - time to free some resources */
+      if (mask_ID != -1)
+        cairo_surface_destroy (mask_image);
+    }
+
+  return TRUE;
+}
