@@ -38,7 +38,6 @@
  *   see ChangeLog
  */
 
-#define GEGL_ITERATOR2_API
 #include "config.h"
 
 #include <stdlib.h>
@@ -105,28 +104,30 @@ typedef struct
   gboolean  save_xmp;
   gboolean  save_iptc;
   gboolean  save_thumbnail;
+  gboolean  save_profile;
   PngExportFormat export_format;
 }
 PngSaveVals;
 
 typedef struct
 {
-  gboolean   run;
+  gboolean       run;
 
-  GtkWidget *interlaced;
-  GtkWidget *bkgd;
-  GtkWidget *gama;
-  GtkWidget *offs;
-  GtkWidget *phys;
-  GtkWidget *time;
-  GtkWidget *comment;
-  GtkWidget *pixelformat;
-  GtkWidget *save_transp_pixels;
+  GtkWidget     *interlaced;
+  GtkWidget     *bkgd;
+  GtkWidget     *gama;
+  GtkWidget     *offs;
+  GtkWidget     *phys;
+  GtkWidget     *time;
+  GtkWidget     *comment;
+  GtkWidget     *pixelformat;
+  GtkWidget     *save_transp_pixels;
   GtkAdjustment *compression_level;
-  GtkWidget *save_exif;
-  GtkWidget *save_xmp;
-  GtkWidget *save_iptc;
-  GtkWidget *save_thumbnail;
+  GtkWidget     *save_exif;
+  GtkWidget     *save_xmp;
+  GtkWidget     *save_iptc;
+  GtkWidget     *save_thumbnail;
+  GtkWidget     *save_profile;
 }
 PngSaveGui;
 
@@ -157,6 +158,7 @@ static void      run                       (const gchar      *name,
 static gint32    load_image                (const gchar      *filename,
                                             gboolean          interactive,
                                             gboolean         *resolution_loaded,
+                                            gboolean         *profile_loaded,
                                             GError          **error);
 static gboolean  save_image                (const gchar      *filename,
                                             gint32            image_ID,
@@ -451,6 +453,7 @@ run (const gchar      *name,
     {
       gboolean interactive;
       gboolean resolution_loaded = FALSE;
+      gboolean profile_loaded    = FALSE;
 
       switch (run_mode)
         {
@@ -467,6 +470,7 @@ run (const gchar      *name,
       image_ID = load_image (param[1].data.d_string,
                              interactive,
                              &resolution_loaded,
+                             &profile_loaded,
                              &error);
 
       if (image_ID != -1)
@@ -484,6 +488,9 @@ run (const gchar      *name,
               if (resolution_loaded)
                 flags &= ~GIMP_METADATA_LOAD_RESOLUTION;
 
+              if (profile_loaded)
+                flags &= ~GIMP_METADATA_LOAD_COLORSPACE;
+
               gimp_image_metadata_load_finish (image_ID, "image/png",
                                                metadata, flags,
                                                interactive);
@@ -552,6 +559,7 @@ run (const gchar      *name,
       pngvals.save_xmp       = (metadata_flags & GIMP_METADATA_SAVE_XMP) != 0;
       pngvals.save_iptc      = (metadata_flags & GIMP_METADATA_SAVE_IPTC) != 0;
       pngvals.save_thumbnail = (metadata_flags & GIMP_METADATA_SAVE_THUMBNAIL) != 0;
+      pngvals.save_profile   = (metadata_flags & GIMP_METADATA_SAVE_COLOR_PROFILE) != 0;
 
       /* Override preferences from PNG export defaults (if saved). */
       load_parasite ();
@@ -657,6 +665,11 @@ run (const gchar      *name,
                   else
                     metadata_flags &= ~GIMP_METADATA_SAVE_THUMBNAIL;
 
+                  if (pngvals.save_profile)
+                    metadata_flags |= GIMP_METADATA_SAVE_COLOR_PROFILE;
+                  else
+                    metadata_flags &= ~GIMP_METADATA_SAVE_COLOR_PROFILE;
+
                   file = g_file_new_for_path (param[3].data.d_string);
                   gimp_image_metadata_save_finish (orig_image_ID,
                                                    "image/png",
@@ -842,6 +855,7 @@ static gint32
 load_image (const gchar  *filename,
             gboolean      interactive,
             gboolean     *resolution_loaded,
+            gboolean     *profile_loaded,
             GError      **error)
 {
   gint              i;                    /* Looping var */
@@ -849,7 +863,6 @@ load_image (const gchar  *filename,
   gint              bpp;                  /* Bytes per pixel */
   gint              width;                /* image width */
   gint              height;               /* image height */
-  gint              empty;                /* Number of fully transparent indices */
   gint              num_passes;           /* Number of interlace passes in file */
   gint              pass;                 /* Current pass in file */
   gint              tile_height;          /* Height of tile in GIMP */
@@ -949,7 +962,11 @@ load_image (const gchar  *filename,
   profile = load_color_profile (pp, info, &profile_name);
 
   if (profile)
-    linear = gimp_color_profile_is_linear (profile);
+    {
+      *profile_loaded = TRUE;
+
+      linear = gimp_color_profile_is_linear (profile);
+    }
 
   /*
    * Get image precision and color model
@@ -1180,30 +1197,14 @@ load_image (const gchar  *filename,
    * Load the colormap as necessary...
    */
 
-  empty = 0; /* by default assume no full transparent palette entries */
-
   if (png_get_color_type (pp, info) & PNG_COLOR_MASK_PALETTE)
     {
       png_colorp palette;
       int num_palette;
 
       png_get_PLTE (pp, info, &palette, &num_palette);
-      if (png_get_valid (pp, info, PNG_INFO_tRNS))
-        {
-          for (empty = 0; empty < 256 && alpha[empty] == 0; ++empty)
-            /* Calculates number of fully transparent "empty" entries */;
-
-          /*  keep at least one entry  */
-          empty = MIN (empty, num_palette - 1);
-
-          gimp_image_set_colormap (image, (guchar *) (palette + empty),
-                                   num_palette - empty);
-        }
-      else
-        {
-          gimp_image_set_colormap (image, (guchar *) palette,
-                                   num_palette);
-        }
+      gimp_image_set_colormap (image, (guchar *) palette,
+                               num_palette);
     }
 
   bpp = babl_format_get_bytes_per_pixel (file_format);
@@ -1373,7 +1374,6 @@ load_image (const gchar  *filename,
           while (length--)
             {
               data[1] = alpha[data[0]];
-              data[0] -= empty;
 
               data += n_components;
             }
@@ -1474,7 +1474,7 @@ save_image (const gchar  *filename,
   gint              num;              /* Number of rows to load */
   FILE             *fp;               /* File pointer */
   GimpColorProfile *profile = NULL;   /* Color profile */
-  gboolean          linear;           /* Save linear RGB */
+  gboolean          out_linear;       /* Save linear RGB */
   GeglBuffer       *buffer;           /* GEGL buffer for layer */
   const Babl       *file_format;      /* BABL format of file */
   png_structp       pp;               /* PNG read pointer */
@@ -1489,46 +1489,77 @@ save_image (const gchar  *filename,
   time_t            cutime;           /* Time since epoch */
   struct tm        *gmt;              /* GMT broken down */
   gint              color_type;       /* PNG color type */
-  gint              bit_depth = 16;   /* Default to bit bepth 16 */
+  gint              bit_depth;        /* Default to bit depth 16 */
 
   guchar            remap[256];       /* Re-mapping for the palette */
 
   png_textp         text = NULL;
 
+  out_linear = FALSE;
 #if defined(PNG_iCCP_SUPPORTED)
-  profile = gimp_image_get_color_profile (orig_image_ID);
+  /* If no profile is written: export as sRGB.
+   * If manually assigned profile written: follow its TRC.
+   * If default profile written:
+   *   - when export as auto or 16-bit: follow the storage TRC.
+   *   - when export from 8-bit storage: follow the storage TRC.
+   *   - when converting high bit depth to 8-bit: export as sRGB.
+   */
+  if (pngvals.save_profile)
+    {
+      profile = gimp_image_get_color_profile (orig_image_ID);
+
+      if (profile                                     ||
+          pngvals.export_format == PNG_FORMAT_AUTO    ||
+          pngvals.export_format == PNG_FORMAT_RGB16   ||
+          pngvals.export_format == PNG_FORMAT_RGBA16  ||
+          pngvals.export_format == PNG_FORMAT_GRAY16  ||
+          pngvals.export_format == PNG_FORMAT_GRAYA16 ||
+          gimp_image_get_precision (image_ID) == GIMP_PRECISION_U8_LINEAR ||
+          gimp_image_get_precision (image_ID) == GIMP_PRECISION_U8_GAMMA)
+        {
+          if (! profile)
+            profile = gimp_image_get_effective_color_profile (orig_image_ID);
+          out_linear = (gimp_color_profile_is_linear (profile));
+        }
+      else
+        {
+          /* When converting higher bit depth work image into 8-bit,
+           * with no manually assigned profile, make sure the result is
+           * sRGB.
+           */
+          profile = gimp_image_get_effective_color_profile (orig_image_ID);
+
+          if (gimp_color_profile_is_linear (profile))
+            {
+              GimpColorProfile *saved_profile;
+
+              saved_profile = gimp_color_profile_new_srgb_trc_from_color_profile (profile);
+              g_object_unref (profile);
+              profile = saved_profile;
+            }
+        }
+    }
 #endif
 
+  /* We save as 8-bit PNG only if:
+   * (1) Work image is 8-bit linear with linear profile to be saved.
+   * (2) Work image is 8-bit non-linear or perceptual with or without
+   * profile.
+   */
+  bit_depth = 16;
   switch (gimp_image_get_precision (image_ID))
     {
     case GIMP_PRECISION_U8_LINEAR:
-      /* only keep 8 bit linear RGB if we also save a profile */
-      if (profile)
+      if (out_linear)
         bit_depth = 8;
-
-    case GIMP_PRECISION_U16_LINEAR:
-    case GIMP_PRECISION_U32_LINEAR:
-    case GIMP_PRECISION_HALF_LINEAR:
-    case GIMP_PRECISION_FLOAT_LINEAR:
-    case GIMP_PRECISION_DOUBLE_LINEAR:
-      /* save linear RGB only if we save a profile, or a loader won't
-       * do the right thing
-       */
-      if (profile)
-        linear = TRUE;
-      else
-        linear = FALSE;
       break;
 
     case GIMP_PRECISION_U8_GAMMA:
-      bit_depth = 8;
+      if (! out_linear)
+        bit_depth = 8;
+      break;
 
-    case GIMP_PRECISION_U16_GAMMA:
-    case GIMP_PRECISION_U32_GAMMA:
-    case GIMP_PRECISION_HALF_GAMMA:
-    case GIMP_PRECISION_FLOAT_GAMMA:
-    case GIMP_PRECISION_DOUBLE_GAMMA:
-      linear = FALSE;
+    default:
       break;
     }
 
@@ -1614,14 +1645,14 @@ save_image (const gchar  *filename,
         color_type = PNG_COLOR_TYPE_RGB;
         if (bit_depth == 8)
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("RGB u8");
             else
               file_format = babl_format ("R'G'B' u8");
           }
         else
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("RGB u16");
             else
               file_format = babl_format ("R'G'B' u16");
@@ -1632,14 +1663,14 @@ save_image (const gchar  *filename,
         color_type = PNG_COLOR_TYPE_RGB_ALPHA;
         if (bit_depth == 8)
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("RGBA u8");
             else
               file_format = babl_format ("R'G'B'A u8");
           }
         else
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("RGBA u16");
             else
               file_format = babl_format ("R'G'B'A u16");
@@ -1650,14 +1681,14 @@ save_image (const gchar  *filename,
         color_type = PNG_COLOR_TYPE_GRAY;
         if (bit_depth == 8)
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("Y u8");
             else
               file_format = babl_format ("Y' u8");
           }
         else
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("Y u16");
             else
               file_format = babl_format ("Y' u16");
@@ -1668,14 +1699,14 @@ save_image (const gchar  *filename,
         color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
         if (bit_depth == 8)
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("YA u8");
             else
               file_format = babl_format ("Y'A u8");
           }
         else
           {
-            if (linear)
+            if (out_linear)
               file_format = babl_format ("YA u16");
             else
               file_format = babl_format ("Y'A u16");
@@ -1709,45 +1740,70 @@ save_image (const gchar  *filename,
       {
         case PNG_FORMAT_RGB8:
           color_type = PNG_COLOR_TYPE_RGB;
-          file_format = babl_format ("R'G'B' u8");
+          if (out_linear)
+            file_format = babl_format ("RGB u8");
+          else
+            file_format = babl_format ("R'G'B' u8");
           bit_depth = 8;
           break;
         case PNG_FORMAT_GRAY8:
           color_type = PNG_COLOR_TYPE_GRAY;
-          file_format = babl_format ("Y' u8");
+          if (out_linear)
+            file_format = babl_format ("Y u8");
+          else
+            file_format = babl_format ("Y' u8");
           bit_depth = 8;
           break;
-        case PNG_FORMAT_AUTO: // shut up gcc
         case PNG_FORMAT_RGBA8:
           color_type = PNG_COLOR_TYPE_RGB_ALPHA;
-          file_format = babl_format ("R'G'B'A u8");
+          if (out_linear)
+            file_format = babl_format ("RGBA u8");
+          else
+            file_format = babl_format ("R'G'B'A u8");
           bit_depth = 8;
           break;
         case PNG_FORMAT_GRAYA8:
           color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
-          file_format = babl_format ("Y'A u8");
+          if (out_linear)
+            file_format = babl_format ("YA u8");
+          else
+            file_format = babl_format ("Y'A u8");
           bit_depth = 8;
           break;
         case PNG_FORMAT_RGB16:
           color_type = PNG_COLOR_TYPE_RGB;
-          file_format = babl_format ("R'G'B' u16");
+          if (out_linear)
+            file_format = babl_format ("RGB u16");
+          else
+            file_format = babl_format ("R'G'B' u16");
           bit_depth = 16;
           break;
         case PNG_FORMAT_GRAY16:
           color_type = PNG_COLOR_TYPE_GRAY;
-          file_format = babl_format ("Y' u16");
+          if (out_linear)
+            file_format = babl_format ("Y u16");
+          else
+            file_format = babl_format ("Y' u16");
           bit_depth = 16;
           break;
         case PNG_FORMAT_RGBA16:
           color_type = PNG_COLOR_TYPE_RGB_ALPHA;
-          file_format = babl_format ("R'G'B'A u16");
+          if (out_linear)
+            file_format = babl_format ("RGBA u16");
+          else
+            file_format = babl_format ("R'G'B'A u16");
           bit_depth = 16;
           break;
         case PNG_FORMAT_GRAYA16:
           color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
-          file_format = babl_format ("Y'A u16");
+          if (out_linear)
+            file_format = babl_format ("YA u16");
+          else
+            file_format = babl_format ("Y'A u16");
           bit_depth = 16;
           break;
+        case PNG_FORMAT_AUTO:
+          g_return_val_if_reached (FALSE);
       }
     }
 
@@ -1833,7 +1889,7 @@ save_image (const gchar  *filename,
     }
 
 #if defined(PNG_iCCP_SUPPORTED)
-  if (profile)
+  if (pngvals.save_profile)
     {
       GimpParasite *parasite;
       gchar        *profile_name = NULL;
@@ -1857,7 +1913,6 @@ save_image (const gchar  *filename,
                     icc_length);
 
       g_free (profile_name);
-
       g_object_unref (profile);
     }
 #endif
@@ -2390,18 +2445,25 @@ save_dialog (gint32    image_ID,
   pg.time = toggle_button_init (builder, "save-creation-time",
                                 pngvals.time,
                                 &pngvals.time);
-  pg.save_exif = toggle_button_init (builder, "sv_exif",
+  pg.save_exif = toggle_button_init (builder, "save-exif",
                                      pngvals.save_exif,
                                      &pngvals.save_exif);
-  pg.save_xmp = toggle_button_init (builder, "sv_xmp",
+  pg.save_xmp = toggle_button_init (builder, "save-xmp",
                                     pngvals.save_xmp,
                                     &pngvals.save_xmp);
-  pg.save_iptc = toggle_button_init (builder, "sv_iptc",
+  pg.save_iptc = toggle_button_init (builder, "save-iptc",
                                      pngvals.save_iptc,
                                      &pngvals.save_iptc);
-  pg.save_thumbnail = toggle_button_init (builder, "sv_thumbnail",
+  pg.save_thumbnail = toggle_button_init (builder, "save-thumbnail",
                                           pngvals.save_thumbnail,
                                           &pngvals.save_thumbnail);
+  pg.save_profile = toggle_button_init (builder, "save-color-profile",
+                                        pngvals.save_profile,
+                                        &pngvals.save_profile);
+
+#if !defined(PNG_iCCP_SUPPORTED)
+  gtk_widget_hide (pg.save_profile);
+#endif
 
   /* Comment toggle */
   parasite = gimp_image_get_parasite (image_ID, "gimp-comment");
@@ -2500,7 +2562,7 @@ load_parasite (void)
 
       gimp_parasite_free (parasite);
 
-      num_fields = sscanf (def_str, "%d %d %d %d %d %d %d %d %d %d %d %d %d",
+      num_fields = sscanf (def_str, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d",
                            &tmpvals.interlaced,
                            &tmpvals.bkgd,
                            &tmpvals.gama,
@@ -2513,11 +2575,12 @@ load_parasite (void)
                            &tmpvals.save_exif,
                            &tmpvals.save_xmp,
                            &tmpvals.save_iptc,
-                           &tmpvals.save_thumbnail);
+                           &tmpvals.save_thumbnail,
+                           &tmpvals.save_profile);
 
       g_free (def_str);
 
-      if (num_fields == 9 || num_fields == 13)
+      if (num_fields == 9 || num_fields == 13 || num_fields == 14)
         pngvals = tmpvals;
     }
 }
@@ -2528,7 +2591,7 @@ save_parasite (void)
   GimpParasite *parasite;
   gchar        *def_str;
 
-  def_str = g_strdup_printf ("%d %d %d %d %d %d %d %d %d %d %d %d %d",
+  def_str = g_strdup_printf ("%d %d %d %d %d %d %d %d %d %d %d %d %d %d",
                              pngvals.interlaced,
                              pngvals.bkgd,
                              pngvals.gama,
@@ -2541,7 +2604,8 @@ save_parasite (void)
                              pngvals.save_exif,
                              pngvals.save_xmp,
                              pngvals.save_iptc,
-                             pngvals.save_thumbnail);
+                             pngvals.save_thumbnail,
+                             pngvals.save_profile);
 
   parasite = gimp_parasite_new (PNG_DEFAULTS_PARASITE,
                                 GIMP_PARASITE_PERSISTENT,
@@ -2577,6 +2641,7 @@ load_gui_defaults (PngSaveGui *pg)
   SET_ACTIVE (save_xmp);
   SET_ACTIVE (save_iptc);
   SET_ACTIVE (save_thumbnail);
+  SET_ACTIVE (save_profile);
 
 #undef SET_ACTIVE
 
