@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -237,6 +237,7 @@ run (const gchar      *name,
   *retvals  = rvals;
 
   INIT_I18N ();
+  gegl_init (NULL, NULL);
 
   memset (&args, 0, sizeof (args));
   args.mode = -1;
@@ -306,28 +307,36 @@ run (const gchar      *name,
 static gboolean
 pluginCore (piArgs *argp)
 {
-  GimpDrawable *drw, *ndrw = NULL;
-  GimpPixelRgn  srcPr, dstPr;
-  gboolean      success = TRUE;
-  gint          nl      = 0;
-  gint          y, i;
-  gint          Y, I, Q;
-  gint          width, height, bpp;
-  gint          sel_x1, sel_x2, sel_y1, sel_y2;
-  gint          prog_interval;
-  guchar       *src, *s, *dst, *d;
-  guchar        r, prev_r=0, new_r=0;
-  guchar        g, prev_g=0, new_g=0;
-  guchar        b, prev_b=0, new_b=0;
-  gdouble       fy, fc, t, scale;
-  gdouble       pr, pg, pb;
-  gdouble       py;
+  GeglBuffer *src_buffer;
+  GeglBuffer *dest_buffer;
+  const Babl *src_format;
+  const Babl *dest_format;
+  gint        src_bpp;
+  gint        dest_bpp;
+  gboolean    success = TRUE;
+  gint        nl      = 0;
+  gint        y, i;
+  gint        Y, I, Q;
+  gint        width, height;
+  gint        sel_x1, sel_x2, sel_y1, sel_y2;
+  gint        prog_interval;
+  guchar     *src, *s, *dst, *d;
+  guchar      r, prev_r=0, new_r=0;
+  guchar      g, prev_g=0, new_g=0;
+  guchar      b, prev_b=0, new_b=0;
+  gdouble     fy, fc, t, scale;
+  gdouble     pr, pg, pb;
+  gdouble     py;
 
-  drw = gimp_drawable_get (argp->drawable);
+  width  = gimp_drawable_width  (argp->drawable);
+  height = gimp_drawable_height (argp->drawable);
 
-  width  = drw->width;
-  height = drw->height;
-  bpp    = drw->bpp;
+  if (gimp_drawable_has_alpha (argp->drawable))
+    src_format = babl_format ("R'G'B'A u8");
+  else
+    src_format = babl_format ("R'G'B' u8");
+
+  dest_format = src_format;
 
   if (argp->new_layerp)
     {
@@ -352,35 +361,41 @@ pluginCore (piArgs *argp)
                            GIMP_RGBA_IMAGE,
                            100,
                            gimp_image_get_default_new_layer_mode (argp->image));
-      ndrw = gimp_drawable_get (nl);
+
       gimp_drawable_fill (nl, GIMP_FILL_TRANSPARENT);
       gimp_image_insert_layer (argp->image, nl, -1, 0);
+
+      dest_format = babl_format ("R'G'B'A u8");
     }
 
-  if (! gimp_drawable_mask_intersect (drw->drawable_id,
+  if (! gimp_drawable_mask_intersect (argp->drawable,
                                       &sel_x1, &sel_y1, &width, &height))
     return success;
 
+  src_bpp  = babl_format_get_bytes_per_pixel (src_format);
+  dest_bpp = babl_format_get_bytes_per_pixel (dest_format);
+
   sel_x2 = sel_x1 + width;
   sel_y2 = sel_y1 + height;
 
-  src = g_new (guchar, width * height * bpp);
-  dst = g_new (guchar, width * height * 4);
-  gimp_pixel_rgn_init (&srcPr, drw, sel_x1, sel_y1, width, height,
-                       FALSE, FALSE);
+  src = g_new (guchar, width * height * src_bpp);
+  dst = g_new (guchar, width * height * dest_bpp);
+
+  src_buffer = gimp_drawable_get_buffer (argp->drawable);
 
   if (argp->new_layerp)
     {
-      gimp_pixel_rgn_init (&dstPr, ndrw, sel_x1, sel_y1, width, height,
-                           FALSE, FALSE);
+      dest_buffer = gimp_drawable_get_buffer (nl);
     }
   else
     {
-      gimp_pixel_rgn_init (&dstPr, drw, sel_x1, sel_y1, width, height,
-                           TRUE, TRUE);
+      dest_buffer = gimp_drawable_get_shadow_buffer (argp->drawable);
     }
 
-  gimp_pixel_rgn_get_rect (&srcPr, src, sel_x1, sel_y1, width, height);
+  gegl_buffer_get (src_buffer,
+                   GEGL_RECTANGLE (sel_x1, sel_y1, width, height), 1.0,
+                   src_format, src,
+                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   s = src;
   d = dst;
@@ -406,7 +421,7 @@ pluginCore (piArgs *argp)
                   for (i = 0; i < 3; i++)
                     *d++ = 0;
                   s += 3;
-                  if (bpp == 4)
+                  if (src_bpp == 4)
                     *d++ = *s++;
                   else if (argp->new_layerp)
                     *d++ = 255;
@@ -422,7 +437,7 @@ pluginCore (piArgs *argp)
                       *d++ = new_g;
                       *d++ = new_b;
                       s += 3;
-                      if (bpp == 4)
+                      if (src_bpp == 4)
                         *d++ = *s++;
                       else if (argp->new_layerp)
                         *d++ = 255;
@@ -529,7 +544,7 @@ pluginCore (piArgs *argp)
 
                       s += 3;
 
-                      if (bpp == 4)
+                      if (src_bpp == 4)
                         *d++ = *s++;
                       else if (argp->new_layerp)
                         *d++ = 255;
@@ -538,36 +553,41 @@ pluginCore (piArgs *argp)
             }
           else
             {
-              if (!argp->new_layerp)
+              if (! argp->new_layerp)
                 {
-                  for (i = 0; i < bpp; i++)
+                  for (i = 0; i < src_bpp; i++)
                     *d++ = *s++;
                 }
               else
                 {
-                  s += bpp;
-                  d += 4;
+                  s += src_bpp;
+                  d += dest_bpp;
                 }
             }
         }
     }
-  gimp_progress_update (1.0);
 
-  gimp_pixel_rgn_set_rect (&dstPr, dst, sel_x1, sel_y1, width, height);
+  gegl_buffer_set (dest_buffer,
+                   GEGL_RECTANGLE (sel_x1, sel_y1, width, height), 0,
+                   dest_format, dst,
+                   GEGL_AUTO_ROWSTRIDE);
+
+  gimp_progress_update (1.0);
 
   g_free (src);
   g_free (dst);
 
+  g_object_unref (src_buffer);
+  g_object_unref (dest_buffer);
+
   if (argp->new_layerp)
     {
-      gimp_drawable_flush (ndrw);
       gimp_drawable_update (nl, sel_x1, sel_y1, width, height);
     }
   else
     {
-      gimp_drawable_flush (drw);
-      gimp_drawable_merge_shadow (drw->drawable_id, TRUE);
-      gimp_drawable_update (drw->drawable_id, sel_x1, sel_y1, width, height);
+      gimp_drawable_merge_shadow (argp->drawable, TRUE);
+      gimp_drawable_update (argp->drawable, sel_x1, sel_y1, width, height);
     }
 
   gimp_displays_flush ();
