@@ -1,29 +1,27 @@
 %plugins = (
     'align-layers' => { ui => 1 },
-    'animation-optimize' => {},
+    'animation-optimize' => { gegl => 1},
     'animation-play' => { ui => 1, gegl => 1 },
-    'blinds' => { ui => 1 },
+    'blinds' => { ui => 1, gegl => 1 },
     'blur' => {},
     'border-average' => { ui => 1, gegl => 1 },
     'busy-dialog' => { ui => 1, gegl => 1 },
     'cartoon' => { ui => 1 },
-    'checkerboard' => { ui => 1 },
-    'cml-explorer' => { ui => 1 },
+    'checkerboard' => { ui => 1, gegl => 1 },
+    'cml-explorer' => { ui => 1, gegl => 1 },
     'color-cube-analyze' => { ui => 1 },
     'color-enhance' => { ui => 1 },
     'colorify' => { ui => 1 },
     'colormap-remap' => { ui => 1, gegl => 1 },
     'compose' => { ui => 1, gegl => 1 },
-    'contrast-normalize' => {},
-    'contrast-retinex' => { ui => 1 },
+    'contrast-retinex' => { ui => 1, gegl => 1 },
     'crop-zealous' => { gegl => 1 },
-    'curve-bend' => { ui => 1 },
+    'curve-bend' => { ui => 1, gegl => 1 },
     'decompose' => { ui => 1, gegl => 1 },
-    'depth-merge' => { ui => 1 },
-    'despeckle' => { ui => 1 },
-    'destripe' => { ui => 1 },
+    'depth-merge' => { ui => 1, gegl => 1 },
+    'despeckle' => { ui => 1, gegl => 1 },
+    'destripe' => { ui => 1, gegl => 1 },
     'edge-dog' => { ui => 1 },
-    'edge-neon' => { ui => 1 },
     'emboss' => { ui => 1 },
     'file-aa' => { ui => 1, gegl => 1, optional => 1, libs => 'AA_LIBS' },
     'file-cel' => { ui => 1, gegl => 1 },
@@ -47,7 +45,7 @@
     'file-pix' => { ui => 1, gegl => 1 },
     'file-png' => { ui => 1, gegl => 1, libs => 'PNG_LIBS', cflags => 'PNG_CFLAGS' },
     'file-pnm' => { ui => 1, gegl => 1 },
-    'file-pdf-load' => { ui => 1, libs => 'POPPLER_LIBS', cflags => 'POPPLER_CFLAGS' },
+    'file-pdf-load' => { ui => 1, gegl => 1, libs => 'POPPLER_LIBS', cflags => 'POPPLER_CFLAGS' },
     'file-pdf-save' => { ui => 1, gegl => 1, optional => 1, libs => 'CAIRO_PDF_LIBS', cflags => 'CAIRO_PDF_CFLAGS' },
     'file-ps' => { ui => 1, gegl => 1, optional => 1, libs => 'GS_LIBS' },
     'file-psp' => { ui => 1, gegl => 1, libs => 'Z_LIBS' },
@@ -60,35 +58,33 @@
     'file-xmc' => { ui => 1, gegl => 1, optional => 1, libs => 'XMC_LIBS' },
     'file-xpm' => { ui => 1, gegl => 1, optional => 1, libs => 'XPM_LIBS' },
     'file-xwd' => {  ui => 1, gegl => 1 },
-    'film' => { ui => 1 },
+    'film' => { ui => 1, gegl => 1 },
     'filter-pack' => { ui => 1 },
     'fractal-trace' => { ui => 1 },
     'goat-exercise' => { gegl => 1 },
     'gradient-map' => { gegl => 1 },
-    'grid' => { ui => 1 },
+    'grid' => { ui => 1, gegl => 1 },
     'guillotine' => {},
-    'hot' => { ui => 1 },
-    'jigsaw' => { ui => 1 },
+    'hot' => { ui => 1, gegl => 1 },
+    'jigsaw' => { ui => 1, gegl => 1 },
     'mail' => { ui => 1, optional => 1 },
     'max-rgb' => { ui => 1 },
-    'newsprint' => { ui => 1 },
-    'nl-filter' => { ui => 1 },
-    'oilify' => { ui => 1 },
+    'nl-filter' => { ui => 1, gegl => 1 },
     'photocopy' => { ui => 1 },
     'plugin-browser' => { ui => 1 },
     'procedure-browser' => { ui => 1 },
     'qbist' => { ui => 1, gegl => 1 },
-    'sample-colorize' => { ui => 1 },
+    'sample-colorize' => { ui => 1, gegl => 1 },
     'sharpen' => { ui => 1 },
     'smooth-palette' => { ui => 1, gegl => 1 },
     'softglow' => { ui => 1 },
-    'sparkle' => { ui => 1 },
-    'sphere-designer' => { ui => 1 },
+    'sparkle' => { ui => 1, gegl => 1 },
+    'sphere-designer' => { ui => 1, gegl => 1 },
     'tile' => { ui => 1, gegl => 1 },
-    'tile-small' => { ui => 1 },
+    'tile-small' => { ui => 1, gegl => 1 },
     'unit-editor' => { ui => 1 },
-    'van-gogh-lic' => { ui => 1 },
-    'warp' => { ui => 1 },
+    'van-gogh-lic' => { ui => 1, gegl => 1 },
+    'warp' => { ui => 1, gegl => 1 },
     'wavelet-decompose' => { ui => 1, gegl => 1 },
     'web-browser' => { ui => 1, ldflags => '$(framework_cocoa)', cppflags => '$(AM_CPPFLAGS) $(xobjective_c)' },
     'web-page' => { ui => 1, optional => 1, libs => 'WEBKIT_LIBS', cflags => 'WEBKIT_CFLAGS' }
