@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -504,7 +504,7 @@ gfig_brush_changed_callback (GimpBrushSelectButton *button,
                              const gchar           *brush_name,
                              gdouble                opacity,
                              gint                   spacing,
-                             GimpLayerModeEffects   paint_mode,
+                             GimpLayerMode          paint_mode,
                              gint                   width,
                              gint                   height,
                              const guchar          *mask_data,
@@ -521,6 +521,7 @@ gfig_brush_changed_callback (GimpBrushSelectButton *button,
   gfig_context->bdesc.width = width;
   gfig_context->bdesc.height = height;
   gimp_context_set_brush (brush_name);
+  gimp_context_set_brush_default_size ();
 
   gfig_paint_callback ();
 }
@@ -577,7 +578,7 @@ gfig_style_copy (Style       *style1,
   if (name)
     style1->name = g_strdup (name);
   else
-    g_message ("Eror: name is NULL in gfig_style_copy.");
+    g_message ("Error: name is NULL in gfig_style_copy.");
 
   if (gfig_context->debug_styles)
     g_printerr ("Copying style %s as style %s\n", style0->name, name);
@@ -611,10 +612,12 @@ gfig_style_apply (Style *style)
 
   gimp_context_set_background (&style->background);
 
-  if (!gimp_context_set_brush (style->brush_name))
+  if (! gimp_context_set_brush (style->brush_name))
     g_message ("Style apply: Failed to set brush to '%s' in style '%s'",
                style->brush_name, style->name);
 
+  gimp_context_set_brush_default_size ();
+
   gimp_context_set_pattern (style->pattern);
 
   gimp_context_set_gradient (style->gradient);
@@ -683,10 +686,12 @@ gfig_style_set_context_from_style (Style *style)
                                &style->foreground);
   gimp_color_button_set_color (GIMP_COLOR_BUTTON (gfig_context->bg_color_button),
                                &style->background);
-  if (!gimp_context_set_brush (style->brush_name))
+  if (! gimp_context_set_brush (style->brush_name))
     g_message ("Style from context: Failed to set brush to '%s'",
                style->brush_name);
 
+  gimp_context_set_brush_default_size ();
+
   gimp_brush_select_button_set_brush (GIMP_BRUSH_SELECT_BUTTON (gfig_context->brush_select),
                                       style->brush_name, -1.0, -1, -1);  /* FIXME */
 
