@@ -4,12 +4,11 @@
 /* Externally visible variables */
 /* ============================ */
 
-extern GdkGC     *gc;
 extern GtkWidget *previewarea;
 
 /* Externally visible functions */
 /* ============================ */
 
-gboolean main_dialog (GimpDrawable *drawable);
+gboolean main_dialog (gint32 drawable_id);
 
 #endif  /* __MAPOBJECT_UI_H__ */
