@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _PYGIMP_H_
@@ -76,6 +76,10 @@ extern PyTypeObject PyGimpLayer_Type;
 #define pygimp_layer_check(v) (PyObject_TypeCheck(v, &PyGimpLayer_Type))
 PyObject *pygimp_layer_new(gint32 ID);
 
+extern PyTypeObject PyGimpGroupLayer_Type;
+#define pygimp_layer__group_check(v) (PyObject_TypeCheck(v, &PyGimpGroupLayer_Type))
+PyObject *pygimp_group_layer_new(gint32 ID);
+
 extern PyTypeObject PyGimpChannel_Type;
 #define pygimp_channel_check(v) (PyObject_TypeCheck(v, &PyGimpChannel_Type))
 PyObject *pygimp_channel_new(gint32 ID);
