@@ -317,6 +317,8 @@ run (const gchar      *name,
     {
       gchar *comment = gimp_get_default_comment ();
 
+      gimp_image_undo_disable (image_ID);
+
       if (shootvals.profile_policy == SCREENSHOT_PROFILE_POLICY_SRGB)
         {
           GimpColorProfile *srgb_profile = gimp_color_profile_new_rgb_srgb ();
@@ -342,7 +344,7 @@ run (const gchar      *name,
           g_free (comment);
         }
 
-      gimp_image_clean_all (image_ID);
+      gimp_image_undo_enable (image_ID);
 
       if (run_mode == GIMP_RUN_INTERACTIVE)
         {
@@ -752,7 +754,7 @@ shoot_dialog (GdkScreen **screen)
       gtk_widget_show (table);
     }
 
-  label = gtk_label_new (_("Screenshot delay: "));
+  label = gtk_label_new_with_mnemonic (_("Screenshot dela_y: "));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                     GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show (label);
@@ -764,6 +766,7 @@ shoot_dialog (GdkScreen **screen)
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_table_attach (GTK_TABLE (table), spinner, 1, 2, 0, 1,
                     GTK_SHRINK, GTK_SHRINK, 0, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (spinner));
   gtk_widget_show (spinner);
 
   g_signal_connect (adj, "value-changed",
